using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Steuerungsklasse fr die Festhaltung der Aufgaben der Verarbeitung.
    /// </summary>
    class CwlbBehaviour
    {
        #region Eigenschaften
        /// <summary>
        /// Liefert true, wenn die Informationen ber die Maschine angezeigt werden sollen.
        /// </summary>
        public bool ShowMachineInfo { get; private set; }

        /// <summary>
        /// Prft und definiert den Schalter Maschineninformationen.
        /// </summary>
        public void SetShowMachineInfo()
        {
            if (ShowMachineInfo)
            {
                throw new ArgumentException("Der Parameter -m darf nicht mehrfach vorkommen.");
            }
            else
            {
                if (WorkInteractive)
                {
                    throw new ArgumentException("Die Kombination mit dem interaktivem Modus ist nicht erlaubt.");
                }
                else
                {
                    ShowMachineInfo = true;
                }
            }
        }

        /// <summary>
        /// Liefert true, wenn die Informationen ber den Benutzer angezeigt werden sollen.
        /// </summary>
        public bool ShowUserInfo { get; private set; }

        /// <summary>
        /// Prft und definiert den Schalter Benutzerinformationen.
        /// </summary>
        public void SetShowUserInfo()
        {
            if (ShowUserInfo)
            {
                throw new ArgumentException("Der Parameter -u darf nicht mehrfach vorkommen.");
            }
            else
            {
                if (WorkInteractive)
                {
                    throw new ArgumentException("Die Kombination mit dem interaktivem Modus ist nicht erlaubt.");
                }
                else
                {
                    ShowUserInfo = true;
                }
            }
        }

        /// <summary>
        /// Liefert true, wenn die Informationen ber Prozesse angezeigt werden sollen.
        /// </summary>
        public bool ShowProcessInfo { get; private set; }

        /// <summary>
        /// Prft und definiert den Schalter Prozessinformationen.
        /// </summary>
        public void SetShowProcessInfo()
        {
            if (ShowProcessInfo || ShowProcessNumber > 0 )
            {
                throw new ArgumentException("Der Parameter -p darf nicht mehrfach vorkommen und nicht kombiniert mit -p:nnn werden.");
            }
            else
            {
                if (WorkInteractive)
                {
                    throw new ArgumentException("Die Kombination mit dem interaktivem Modus ist nicht erlaubt");
                }
                else
                {
                    ShowProcessInfo = true;
                }
            }
        }

        /// <summary>
        /// Liest oder definiert die Nummer des Prozesses, dessen Detailinformationen angezeigt werden.
        /// </summary>
        public int ShowProcessNumber { get; private set; }

        /// <summary>
        /// Prft und definiert die Prozessnummer, dessen Daten ausgegeben werden sollen.
        /// </summary>
        /// <param name="iProcessNumber">Nummer des Prozesses, dessen Daten ausgegeben werden sollen.</param>
        public void SetShowProcessNumber(int iProcessNumber)
        {
            if (ShowProcessInfo || ShowProcessNumber > 0)
            {
                throw new Exception("Der Parameter -p darf nicht mehrfach vorkommen und nicht kombiniert mit -p:nnn werden.");
            }
            else
            {
                if (WorkInteractive)
                {
                    throw new Exception("Die Kombination mit dem interaktiven Modus ist nicht erlaubt.");
                }
                else
                {
                    ShowProcessNumber = iProcessNumber;
                }
            }
        }

        /// <summary>
        /// Liest oder definiert den Schalter zur Steuerung der Ausgaben von Benutzungshinweisen des Programms.
        /// </summary>
        public bool ShowHelp { get; set; }

        /// <summary>
        /// Schaltet den interaktiven Modus des Programms ein.
        /// </summary>
        public bool WorkInteractive { get; private set; }

        /// <summary>
        /// Prft und definiert den interaktiven Modus.
        /// </summary>
        public void SetWorkInteractive()
        {
            if (ShowMachineInfo || ShowUserInfo || ShowProcessInfo || ShowProcessNumber > 0)
            {
                throw new Exception("Kombination mit interaktivem Modus nicht erlaubt");
            }
            else
            {
                WorkInteractive = true;
            }
        }
        #endregion

        #region BatchModus
        /// <summary>
        /// Diese Methode steuert die Ausgabefunktionen an.
        /// </summary>
        public void DoWork()
        {
            ShowTitle("Demoprogramm Konsolenanwendung");
            if (ShowMachineInfo)
            {
                ShowMachineData();
            }
            if (ShowUserInfo)
            {
                ShowUserData();
            }
            if (ShowProcessInfo)
            {
                ShowProcessData();
            }
            if (ShowProcessNumber > 0)
            {
                ShowProcessDetailData();
            }
        }

        /// <summary>
        /// Schreibt die notwendigen Leerzeilen und den Titel einer Information
        /// </summary>
        /// <param name="strTitle">Title der Information</param>
        public void ShowTitle(string strTitle)
        {
            Console.WriteLine();
            Console.WriteLine(strTitle);
            Console.WriteLine(new string('-', strTitle.Length));
        }

        /// <summary>
        /// Zeigt die Informationen an
        /// </summary>
        public void ShowInfo(string strField, params object[] aobjInfo)
        {
            // Lokale Daten
            string strLine = null;

            // Stelle das Ausgabeformat zusammen
            if (aobjInfo.Length == 1)
            {
                // Fix fr ein Elemente
                strLine = string.Format("  {0,-25} {1,-45}", strField, aobjInfo[0].ToString());
            }
            else
            {
                // Mehr oder weniger als 1 Element im Array
                string strFormat = "  {0,-20} {1}";

                // Generiere Daten
                foreach (object objData in aobjInfo)
                {
                    if (objData.GetType() == typeof(String))
                    {
                        strLine += string.Format("{0,-12} ", objData.ToString());
                    }
                    else
                    {
                        strLine += string.Format("{0,12} ", objData.ToString());
                    }
                }

                // Erstelle Zeile
                strLine = string.Format(strFormat, strField, strLine);
            }

            // Ausgabe
            // Beschneide Zeile auf Zeilenlnge
            if (strLine.Length > Console.WindowWidth)
            {
                strLine = strLine.Substring(0, Console.WindowWidth - 5) + "...";
            }
            Console.WriteLine(strLine);
        }

        /// <summary>
        /// Generiert eine Kopfzeile fr die Datentabelle.
        /// </summary>
        /// <param name="aobjHeader">berschriften der Spalten</param>
        public void ShowHeader(params object[] aobjHeader)
        {
            // Lokale Daten
            string strLine = null;
            bool bFirst = true;

            // Stelle das Ausgabeformat zusammen
            if (aobjHeader.Length == 2)
            {
                // Fixe breite fr zwei Elemente
                strLine = string.Format("  {0,-25} {1,-45}", aobjHeader[0], aobjHeader[1]);
            }
            else
            {
                // Mehr oder weniger als 2 Elemente
                strLine = "  ";
                // Generiere Daten
                foreach (object objData in aobjHeader)
                {
                    if (bFirst)
                    {
                        strLine += string.Format("{0,-20} ", objData.ToString());
                        bFirst = false;
                    }
                    else
                    {
                        strLine += string.Format("{0,-12} ", objData.ToString());
                    }
                }

                // Erstelle Zeile
                strLine = string.Format("{0}", strLine);
            }

            // Ausgabe
            Console.WriteLine(strLine);
            Console.WriteLine("  " + new string('-', strLine.Length));
        }

        /// <summary>
        /// Zeigt die Maschineninformationen an.
        /// </summary>
        public void ShowMachineData()
        {
            // Informationen anzeigen
            ShowTitle("Maschineninformationen");
            ShowHeader("Feld", "Information");
            ShowInfo("Maschinenname", Environment.MachineName);
            ShowInfo("Anzahl Prozessoren", Environment.ProcessorCount);
            ShowInfo("Systemverzeichnis", Environment.SystemDirectory);
            ShowInfo("Pfad ApplicationData", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData));
            ShowInfo("CommonApplicationData", Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData));
            ShowInfo("CommonProgramFiles", Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles));
            ShowInfo("Cookies", Environment.GetFolderPath(Environment.SpecialFolder.Cookies));
            ShowInfo("Desktop", Environment.GetFolderPath(Environment.SpecialFolder.Desktop));
            ShowInfo("DesktopDirectory", Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory));
            ShowInfo("Favorites", Environment.GetFolderPath(Environment.SpecialFolder.Favorites));
            ShowInfo("History", Environment.GetFolderPath(Environment.SpecialFolder.History));
            ShowInfo("InternetCache", Environment.GetFolderPath(Environment.SpecialFolder.InternetCache));
            ShowInfo("LocalApplicationData", Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData));
            ShowInfo("MyComputer", Environment.GetFolderPath(Environment.SpecialFolder.MyComputer));
            ShowInfo("MyDocuments", Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));
            ShowInfo("MyMusic", Environment.GetFolderPath(Environment.SpecialFolder.MyMusic));
            ShowInfo("MyPictures", Environment.GetFolderPath(Environment.SpecialFolder.MyPictures));
            ShowInfo("Personal", Environment.GetFolderPath(Environment.SpecialFolder.Personal));
            ShowInfo("ProgramFiles", Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles));
            ShowInfo("Programs", Environment.GetFolderPath(Environment.SpecialFolder.Programs));
            ShowInfo("Recent", Environment.GetFolderPath(Environment.SpecialFolder.Recent));
            ShowInfo("SendTo", Environment.GetFolderPath(Environment.SpecialFolder.SendTo));
            ShowInfo("StartMenu", Environment.GetFolderPath(Environment.SpecialFolder.StartMenu));
            ShowInfo("Startup", Environment.GetFolderPath(Environment.SpecialFolder.Startup));
            ShowInfo("System", Environment.GetFolderPath(Environment.SpecialFolder.System));
            ShowInfo("Templates", Environment.GetFolderPath(Environment.SpecialFolder.Templates));
            Console.WriteLine();

            // Umgebungsvariablen anzeigen
            foreach (DictionaryEntry objEntry in Environment.GetEnvironmentVariables())
            {
                ShowInfo(objEntry.Key.ToString(), objEntry.Value.ToString());
            }
        }

        /// <summary>
        /// Zeigt die Benutzerinformationen an
        /// </summary>
        public void ShowUserData()
        {
            // Informationen anzeigen
            ShowTitle("Userinformationen");
            ShowHeader("Feld", "Information");
            ShowInfo("Username", Environment.UserName);
            ShowInfo("Domainname", Environment.UserDomainName);
            ShowInfo("AktiverUser", Environment.UserInteractive);
        }

        /// <summary>
        /// Zeigt die Prozessinformationen an
        /// </summary>
        public void ShowProcessData()
        {
            // Informationen anzeigen
            ShowTitle("Processinformationen");
            ShowHeader("Prozessname", "Id", "Workingset", "Anzahl Threads");
            Process[] aobjProcess = Process.GetProcesses();
            foreach (Process objProcess in aobjProcess)
            {
                ShowInfo(objProcess.ProcessName, objProcess.Id, objProcess.WorkingSet64, objProcess.Threads.Count);
            }
        }

        /// <summary>
        /// Zeigt Prozess Detailinformationen an
        /// </summary>
        public void ShowProcessDetailData()
        {
            // Lokale Daten
            Process objProcess = Process.GetProcessById(ShowProcessNumber);

            // Informationen anzeigen
            ShowTitle("Detailinformationen des Prozess " + ShowProcessNumber.ToString());
            ShowHeader("Feld", "Information");
            ShowInfo("Identifikation", objProcess.Id);
            ShowInfo("Name", objProcess.ProcessName);
            ShowInfo("Basisprioritt", objProcess.BasePriority);
            ShowInfo("Priorittsklasse", objProcess.PriorityClass);
            ShowInfo("Handle Hauptfenster", objProcess.MainWindowHandle);
            ShowInfo("Fenstertitel", objProcess.MainWindowTitle);
            ShowInfo("Max Workingset", objProcess.MaxWorkingSet);
            ShowInfo("Min Workingset", objProcess.MinWorkingSet);
            ShowInfo("NonPagedSystemMemory", objProcess.NonpagedSystemMemorySize64);
            ShowInfo("PagedMemory", objProcess.PagedMemorySize64);
            ShowInfo("PagedSystemMemory", objProcess.PagedSystemMemorySize64);
            ShowInfo("PeakPagedMemeory", objProcess.PeakPagedMemorySize64);
            ShowInfo("PeakVirtualMemory", objProcess.PeakVirtualMemorySize64);
            ShowInfo("PeakWorkingSet", objProcess.PeakWorkingSet64);
            ShowInfo("PrivateMemorySize", objProcess.PrivateMemorySize64);
            ShowInfo("VirtualMemorySize", objProcess.VirtualMemorySize64);
            ShowInfo("WorkingSet", objProcess.WorkingSet64);
            ShowInfo("UserProcessorTime", objProcess.UserProcessorTime);
            ShowInfo("PrivilegedProcesserTime", objProcess.PrivilegedProcessorTime);
            ShowInfo("TotalProcessorTime", objProcess.TotalProcessorTime);
            ShowInfo("StartZeit", objProcess.StartTime);
            ShowInfo("Anzahl Threads", objProcess.Threads.Count);
        }
        #endregion

        #region Interactiv Modus
        /// <summary>
        /// Schreibt den Titel fr die interaktive Demo
        /// </summary>
        /// <param name="strTitle">Haupttitem</param>
        /// <param name="strSubtitle">Untertitel</param>
        public void ShowTitleInteractive(string strTitle, string strSubtitle)
        {
            // Konsole vorbereiten
            Console.BackgroundColor = ConsoleColor.White;
            Console.ForegroundColor = ConsoleColor.Blue;
            Console.Clear();

            // Titel schreiben
            Console.SetCursorPosition(10, 6);
            Console.Write(strTitle);
            Console.SetCursorPosition(10, 7);
            Console.Write(new string('=', strTitle.Length));

            // Schreibe Untertitel wenn vorhanden
            if (strSubtitle.Length > 0)
            {
                Console.SetCursorPosition(10, 9);
                Console.Write("Thema: " + strSubtitle);
            }
        }

        /// <summary>
        /// Warte auf einen Tastendruck
        /// </summary>
        public void ReadKey()
        {
            Console.Write("Weiter mit einem beliebigen Tastendruck ... ");
            Console.ReadKey();
        }

        /// <summary>
        /// Steuerung des Interaktiven Modus
        /// </summary>
        public void DoWorkInteractive()
        {
            // Lokale Daten
            ConsoleKeyInfo objKeyInfo;
            bool bContinue = true;

            // Lese Voraus
            while (bContinue)
            {
                // Konsole lschen
                ShowTitleInteractive("Demoprogramm interaktive Konsole", "");

                // Zeige Men
                Console.SetCursorPosition(10, 12);
                Console.Write("Auswahl:");
                Console.SetCursorPosition(10, 14);
                Console.Write("  1  -> Maschineninformationen anzeigen");
                Console.SetCursorPosition(10, 15);
                Console.Write("  2  -> Userinformationen anzeigen");
                Console.SetCursorPosition(10, 16);
                Console.Write("  3  -> Prozessinformationen anzeigen");
                Console.SetCursorPosition(10, 25);
                Console.Write("Ihre Wahl (ESC beenden): ");

                // Lese nchste Taste
                objKeyInfo = Console.ReadKey();

                // Auswertung
                switch (objKeyInfo.Key)
                {
                    case ConsoleKey.D1:
                        Console.Clear();
                        ShowMachineData();
                        ReadKey();
                        break;
                    case ConsoleKey.D2:
                        Console.Clear();
                        ShowUserData();
                        ReadKey();
                        break;
                    case ConsoleKey.D3:
                        Console.Clear();
                        ShowProcessData();
                        ReadKey();
                        break;
                    case ConsoleKey.Escape:
                        bContinue = false;
                        break;
                    default:
                        break;
                }
            }

            // Konsole lschen
            Console.Clear();
        }
        #endregion
    }
}
