﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert die Funktionalität des Hauptfensters.
    /// </summary>
    public partial class CwlbMainWindow : Form, INotifyPropertyChanged
    {
        // Zähler für die Leerlaufereignisse
        private int _iCountIdle;
        // Referenz für den nicht modalen Dialog
        private CdlgNonModalDemo _dlgNonModal;
        // Implementierung der Schnittstelle INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Liefert oder definiert den Leerlaufzähler.
        /// </summary>
        public int CountIdle
        {
            get
            {
                return _iCountIdle;
            }
            set
            {
                _iCountIdle = value;
                if (_iCountIdle % 5 == 0)
                {
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("CountIdle"));
                    }
                }
            }
        }

        /// <summary>
        /// Initialisiert ein Objekt der Klasse CwlbMainForm.
        /// </summary>
        public CwlbMainWindow()
        {
            // Aufrufen der generierten Initialisierung
            InitializeComponent();

            // Registrieren der Anwendungsereignisse
            Application.Idle += new EventHandler(Application_Idle);
            Application.EnterThreadModal += new EventHandler(Application_EnterThreadModal);
            Application.LeaveThreadModal += new EventHandler(Application_LeaveThreadModal);
            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);

            // Nutzung des zusätzlichen Ereignisses Disposed
            this.Disposed += new EventHandler(CwlbMainForm_Disposed);

            // Bindung des Leerlaufzählers auf das Textfeld
            Binding objBinding = new Binding("Text", this, "CountIdle");
            objBinding.FormattingEnabled = true;
            objBinding.FormatString = "N0";
            _tbIdle.DataBindings.Add(objBinding);
        }

        /// <summary>
        /// Formatiert und schreibt einen Text in die Liste auf dem Hauptfenster.
        /// </summary>
        /// <param name="strFormat">Format des zu schreibenden Texts.</param>
        /// <param name="aobjData">Parameter, die am Platz halter einzusetzten sind.</param>
        private void AddToLog(string strFormat, params object[] aobjData)
        {
            // Locals
            string strWrite;

            // Prüfen, ob Argumentdaten übergeben wurden
            if (aobjData == null)
            {
                strWrite = strFormat;
            }
            else
            {
                strWrite = string.Format(strFormat, aobjData);
            }

            // Wenn das Fenster bereits zerstört wurde, sollen keine Steuerelemente
            // mehr angesprochen werden
            if (!this.IsDisposed)
            {
                _lbLog.Items.Add(strWrite);
            }

            // Schreibe alle Logeinträge in die Textdatei
            Program.WriteLine(strWrite);
        }

        #region Anwendungsereignisse
        /// <summary>
        /// Ereignisbehandlung der Ausnahme des Hauptthreads.
        /// </summary>
        void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            AddToLog("Thread Ausnahme: {0}", e.Exception.Message);
        }

        /// <summary>
        /// Ereignisbehandlung für das Eintreten auf den modalen Threadzustand.
        /// </summary>
        void Application_EnterThreadModal(object sender, EventArgs e)
        {
            AddToLog("Thread modal beginnen");
        }

        /// <summary>
        /// Ereignisbehandlung für das Verlassen des modalen Threadzustands.
        /// </summary>
        void Application_LeaveThreadModal(object sender, EventArgs e)
        {
            AddToLog("Thread modal verlassen");
        }

        /// <summary>
        /// Ereignisbehandlung des Leerlaufereignisses.
        /// </summary>
        private void Application_Idle(object sender, EventArgs e)
        {
            CountIdle++;
        }
        #endregion

        #region Ereignisbehandlungen des Formulars
        #region Ereignisbehandlungen für Status und Inhalt
        /// <summary>
        /// Ereignisbehandlung für die Initialisierung des Fensters und seinen Steuerelementen.
        /// </summary>
        private void CwlbMainForm_Load(object sender, EventArgs e)
        {
            AddToLog("Formular wird geladen (load)");
        }

        /// <summary>
        /// Ereignisbehandlung für die Verändurung der Sichtbarkeit des Fensters.
        /// </summary>
        private void CwlbMainForm_VisibleChanged(object sender, EventArgs e)
        {
            AddToLog("Formularsichtbarkeit wurde verändert (visible changed)");
        }

        /// <summary>
        /// Ereignisbehandlung für die Aktivierung des Fensters.
        /// </summary>
        private void CwlbMainForm_Activated(object sender, EventArgs e)
        {
            AddToLog("Formular wurde aktiviert (activated)");
        }

        /// <summary>
        /// Ereignisbehandlung für die Anzeige nach der Initialisierung des Fensters.
        /// </summary>
        private void CwlbMainForm_Shown(object sender, EventArgs e)
        {
            AddToLog("Formular wird angezeigt (shown)");
        }

        /// <summary>
        /// Ereignisbehandlung für das Zeichnen des Inhalts des Fensters.
        /// </summary>
        private void CwlbMainForm_Paint(object sender, PaintEventArgs e)
        {
            AddToLog("Formular wird gezeichnet (paint)");
        }

        /// <summary>
        /// Ereignisbehandlung für die Deaktivierung des Fensters.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CwlbMainForm_Deactivate(object sender, EventArgs e)
        {
            AddToLog("Formular wurde deaktiviert (deactivate)");
        }

        /// <summary>
        /// Ereignisbehandlung vor dem Schließen des Fensters.
        /// </summary>
        private void CwlbMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            AddToLog("Formular wird geschlossen (closing)");

            // Deregistrieren der Anwendungsereignisse
            Application.Idle -= new EventHandler(Application_Idle);
            Application.EnterThreadModal -= new EventHandler(Application_EnterThreadModal);
            Application.LeaveThreadModal -= new EventHandler(Application_LeaveThreadModal);
            Application.ThreadException -= new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Schliessen des Fensters.
        /// </summary>
        private void CwlbMainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            AddToLog("Formular wurde geschlossen (closed)");
        }

        /// <summary>
        /// Ereignisbehandlung nach dem freigeben des Hauptfensterobjetks.
        /// </summary>
        void CwlbMainForm_Disposed(object sender, EventArgs e)
        {
            AddToLog("Formular wurde entsorgt (disposed)");
        }
        #endregion

        #region Ereignisbehandlungen für Tastatur
        /// <summary>
        /// Ereignisbehandlung
        /// </summary>
        private void CwlbMainForm_KeyDown(object sender, KeyEventArgs e)
        {
            AddToLog("Taste {0} wurde betätigt (KeyDown).", e.KeyCode);
        }

        /// <summary>
        /// Ereignisbehandlung
        /// </summary>
        private void CwlbMainForm_KeyPress(object sender, KeyPressEventArgs e)
        {
            AddToLog("Taste {0} wurde betätigt (KeyPress).", e.KeyChar);
        }

        /// <summary>
        /// Ereignisbehandlung
        /// </summary>
        private void CwlbMainForm_KeyUp(object sender, KeyEventArgs e)
        {
            AddToLog("Taste {0} wurde betätigt (KeyUp).", e.KeyCode);
        }

        /// <summary>
        /// Ereignisbehandlung
        /// </summary>
        private void CwlbMainForm_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            AddToLog("Taste {0} wurde betätigt (PreviewKeyDown).", e.KeyCode);
        }
        #endregion

        #region Ereignisbehandlungen für die Benutzung der Maus
        /// <summary>
        /// Ereignisbehandlung für Eintreten des Mauscursor in den Bereich des Hauptfensters.
        /// </summary>
        private void CwlbMainForm_MouseEnter(object sender, EventArgs e)
        {
            AddToLog("Maus benutzt (MouseEnter).");
        }

        /// <summary>
        /// Ereignisbehandlung für Betätigung einer Maustaste.
        /// </summary>
        private void CwlbMainForm_MouseDown(object sender, MouseEventArgs e)
        {
            AddToLog("Maus benutzt (MouseDown).");
        }

        /// <summary>
        /// Ereignisbehandlung für Schweben des Mauscursers auf dem Fenster.
        /// </summary>
        private void CwlbMainForm_MouseHover(object sender, EventArgs e)
        {
            AddToLog("Maus benutzt (MouseHover).");
        }

        /// <summary>
        /// Ereignisbehandlung für Bewegung der Maus.
        /// </summary>
        private void CwlbMainForm_MouseMove(object sender, MouseEventArgs e)
        {
            AddToLog("Maus benutzt (MouseMove).");
        }

        /// <summary>
        /// Ereignisbehandlung für Loslassen einer Maustaste.
        /// </summary>
        private void CwlbMainForm_MouseUp(object sender, MouseEventArgs e)
        {
            AddToLog("Maus benutzt (MouseUp).");
        }

        /// <summary>
        /// Ereignisbehandlung für Verlassen des Fensters durch den Curser.
        /// </summary>
        private void CwlbMainForm_MouseLeave(object sender, EventArgs e)
        {
            AddToLog("Maus benutzt (MouseLeave).");
        }
        #endregion
        #endregion

        #region Ereignisbehandlung für Schaltflächen
        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Unbehandelte Ausnahme' wurde betätigt.
        /// </summary>
        private void ButtinException_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Unbehandelte Ausnahme auslösen' betätigt.");
            throw new NotImplementedException("Ausnahme des Beispiels.");
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Modaler Dialog starten' wurde betätigt.
        /// </summary>
        private void ButtonModalDialog_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Modales Dialogfeld ...' betätigt.");
            CdlgModalDemo dlgDemo = new CdlgModalDemo();
            dlgDemo.ShowDialog();
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Nicht modales Dialogfeld starten' wurde betätigt.
        /// </summary>
        private void ButtonNonModalDialog_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Nicht modales Dialogfeld' betätigt.");

            // Prüfen, ob das Dialogfeld noch nicht existiert oder ob eine alte
            // Kopie als nicht mehr benutzbar gekennzeichnet ist. Das ist immer
            // dann der Fall, wenn das Dialogfeld geschlossen wurde.
            if (_dlgNonModal == null || _dlgNonModal.IsDisposed)
            {
                // Erstellen des Dialogobjektes
                _dlgNonModal = new CdlgNonModalDemo();
                // Die Zuweisung des Besitzers ist notwendig, damit beim Aktivieren
                // des Hauptfensters das etwaig offene, nicht modale Dialogfelde
                // ebenfalls angezeigt wird.
                _dlgNonModal.Owner = this;
                // Anzeigen des nicht modalen Dialogfeldes.
                _dlgNonModal.Show();
            }
            else
            {
                // Anzeigen und Fokus übergeben
                _dlgNonModal.Show();
                _dlgNonModal.Focus();
            }
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Anwendung neu starten' wurde betätigt.
        /// </summary>
        private void ButtonRestart_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Anwendung neu starten' betätigt.");
            Application.Restart();
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Fensterposition / -stil ...' wurde betätigt.
        /// </summary>
        private void ButtonWindow_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Anwendung neu starten' betätigt.");
            CdlgWindow dlgScreens = new CdlgWindow();
            dlgScreens.ShowDialog();
            dlgScreens.Dispose();
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Seitliche Fixierung ...' wurde betätigt.
        /// </summary>
        private void _btnAcnchor_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Seitliche Fixierung ...' betätigt.");
            CdlgAnchor dlgAnchor = new CdlgAnchor();
            dlgAnchor.ShowDialog();
            dlgAnchor.Dispose();
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Dialogfeld kontrolliert schließen ...' wurde betätigt.
        /// </summary>
        private void ButtonCheckedClose_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Dialogfeld kontrolliert schliessen ...' betätigt.");

            // Erstellen des Dialogfeldobjekts
            CdlgCheckedClose dlgCheckedClose = new CdlgCheckedClose();

            // Initialisieren der Inhalte des Dialogfelds

            // Anzeigen des Dialogfelds und Auswerten des Resultats
            if (dlgCheckedClose.ShowDialog() == DialogResult.OK)
            {
                // Situationsbezogene Arbeit ausführen
                MessageBox.Show("Aktion wurde ausgelöst.", "Demo");
            }
            else
            {
                // Situationsbezogene alternative Arbeit ausführen
                MessageBox.Show("Dialogfeld wurde geschlossen.", "Demo");
            }

            // Freigeben des Dailogobjekts
            dlgCheckedClose.Dispose();
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Eigenschafts-Explorer' wurde betätigt.
        /// </summary>
        private void ButtonPropertyExplorer_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Eigenschafts-Explorer ...' betätigt.");
            CDlgControlExplorer dlgControlsHierarchy = new CDlgControlExplorer();
            dlgControlsHierarchy.ShowDialog();
            dlgControlsHierarchy.Dispose();
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Log löschen' wurde betätigt.
        /// </summary>
        private void ButtonDeleteLog_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Log löschen' betätigt.");
            File.Delete(Program.LogFileName);
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Beenden' wurde betätigt.
        /// </summary>
        private void ButtonExit_Click(object sender, EventArgs e)
        {
            AddToLog("Schaltfläche 'Beenden' betätigt.");
            Close();
        }
        #endregion
    }
}
