﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Security.Permissions;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert ein konfigurierbares Ereignisfilter für ein Ereignis in einer Hierarchie von Steuerelementen.
    /// </summary>
    [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode)]
    class CwlbMessageFilter : IMessageFilter
    {
        private const int NO_FILTER = -1;

        /// <summary>
        /// Definiere das oberste Steuerelement, das die Ereignisse akzeptiert
        /// </summary>
        private Control _objControlParent;

        /// <summary>
        /// Definiert die Nummer des gefilterten Ereignisses.
        /// </summary>
        private int _iMessageFiltered;

        /// <summary>
        /// Signalisiert ein Ereignis gemäss Filterdefinition.
        /// </summary>
        public event EventHandler<CwlbMessageFilterEventArgs> MessageReceived;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbMessageFilter.
        /// </summary>
        /// <param name="objParent">Oberstes Element der Hierarchie, das Ereignisse zulässt.</param>
        /// <param name="iMessageFiltered">Nummer des Ereignisses die gefiltert werden soll.</param>
        public CwlbMessageFilter(Control objParent, int iMessageFiltered)
        {
            _objControlParent = objParent;
            _iMessageFiltered = iMessageFiltered;
        }

        public CwlbMessageFilter()
        {
            _objControlParent = null;
            _iMessageFiltered = NO_FILTER;
        }

        /// <summary>
        /// Implementierung der vorgeschriebenen Methode der Schnittstelle IMessageFilter.
        /// </summary>
        /// <param name="robjMessage">Nachricht aus der Nachrichtenwarteschlange.</param>
        /// <returns>Liefert true, wenn die Nacchrict nht weitergeleitet werdeen soll.</returns>
        public bool PreFilterMessage(ref Message robjMessage)
        {
            try
            {
                // Liefere das Steuerelement
                Control objControl = Control.FromHandle(robjMessage.HWnd);

                // Wenn kein Ereignisfilter definiert ist, werden alle Ereignisse spioniert
                if (_iMessageFiltered == NO_FILTER)
                {
                    // Prüfe ob ein Ereignishandler registriert ist.
                    if (MessageReceived != null)
                    {
                        MessageReceived(objControl, new CwlbMessageFilterEventArgs(robjMessage, objControl));
                    }
                }

                // Nur die gewünschten Ereignisse verwenden
                else if (robjMessage.Msg == _iMessageFiltered)
                {
                    // Suche das Steuerelement in der Hierarchie
                    if (objControl != null && _objControlParent.Contains(objControl))
                    {
                        // Prüfe ob ein Ereignishandler registriert ist.
                        if (MessageReceived != null)
                        {
                            MessageReceived(objControl, new CwlbMessageFilterEventArgs(robjMessage, objControl));
                        }
                    }
                }
            }
            catch
            {
                // Keine Ausnahmebehandlung
            }

            // Signalisiere in jedem Fall, dass das Ereignis nicht behandelt wurde.
            // Damit wird erreicht, dass das Originalziel normal arbeitet.
            return false;
        }
    }
}
