using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Benutzersteuerelement zur Visualisierung von Kontrollelementen
    /// </summary>
    public partial class CctrlHierarchy : UserControl
    {
        // Basiselement
        private Control _objBase = null;

        /// <summary>
        /// Liefert oder definiert das Steuerelement,
        /// das als Baum dargestelt wird
        /// </summary>
        public Control BaseControl
        {
            get
            {
                return _objBase;
            }
            set
            {
                // Wert bernehmen
                _objBase = value;
                // Baum neu aufbauen
                RefreshTree();
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CctrlHierarchy.
        /// </summary>
        public CctrlHierarchy()
        {
            // Generierter Code
            InitializeComponent();
        }

        /// <summary>
        /// Inhalt des Baumes neu aufbauen
        /// </summary>
        private void RefreshTree()
        {
            // Leere Treeview
            _tvControls.Nodes.Clear();

            // Prfe Gltigkeit der basisreferenz
            if (_objBase != null)
            {
                // Erstelle Basiselement
                TreeNode objNode = AddNode(_tvControls.Nodes, _objBase);
                // Ergnze rekursiv
                AddControl(objNode.Nodes, _objBase);
            }

            // Zeige alle Knoten
            _tvControls.ExpandAll();
            // Selektiere ersten Knoten
            if (_tvControls.Nodes.Count > 0)
            {
                _tvControls.SelectedNode = _tvControls.Nodes[0];
            }
        }

        /// <summary>
        /// Ein Steuerelement mitsamt seinen enthaltenen Elementen hinzufgen
        /// </summary>
        /// <param name="objNodes">Liste in welche neue Elemente ergnzt werden</param>
        /// <param name="objParent">Elternobjekt im Baum der Elemente</param>
        private void AddControl(TreeNodeCollection objNodes, Control objParent)
        {
            // Lokale Daten
            TreeNode objNode;

            // Lade rekursiv
            foreach (Control objCntrl in objParent.Controls)
            {

                // Fge Knoten hinzu
                objNode = AddNode(objNodes, objCntrl);
                // Prfe ob weitere Elemente enthalten sind
                if (objCntrl.Controls.Count > 0)
                {
                    // Rekursiver Aufruf
                    AddControl(objNode.Nodes, objCntrl);
                }
            }
        }

        /// <summary>
        /// Ein Element selber hinzufgen
        /// </summary>
        /// <param name="objNodes"></param>
        /// <param name="objCntrl"></param>
        /// <returns>Eingefgter Knoten</returns>
        private TreeNode AddNode(TreeNodeCollection objNodes, Control objCntrl)
        {
            // Lokale Daten
            TreeNode nodeItem = null;

            // Erzeuge neuen Knoten
            nodeItem = new TreeNode();
            // Lese den Klassennamen
            nodeItem.Text = objCntrl.GetType().Name;
            if (objCntrl.Name.Length == 0)
            {
                nodeItem.ToolTipText = "<NoName>";
            }
            else
            {
                nodeItem.ToolTipText = objCntrl.Name;
            }
            // Verknpfe den Knoten mit dem Objekt
            nodeItem.Tag = objCntrl;
            // Definiere die Abbildungen fr den Knoten
            nodeItem.ImageKey = objCntrl.GetType().Name + ".bmp";
            nodeItem.SelectedImageKey = objCntrl.GetType().Name + ".bmp";
            // Ergnze den Knoten im TreeView 
            objNodes.Add(nodeItem);

            return nodeItem;
        }

        /// <summary>
        /// Ereignishandler Selektion eines Elements im Baum ist erfolgt
        /// </summary>
        private void _tvControls_AfterSelect(object sender, TreeViewEventArgs e)
        {
            // Selektierter Knoten ber dessen Objektverknpfung 
            // mit dem PropertyGrid verbinden
            _pgrControls.SelectedObject = e.Node.Tag;
        }

        /// <summary>
        /// Dynamisches Erstellen eines Elements
        /// </summary>
        private void _ctxNew_Click(object sender, EventArgs e)
        {
            // Anzeigen des Auswahldialoges
            CdlgClassChooser dlgClassChooser = new CdlgClassChooser();
            if (dlgClassChooser.ShowDialog() == DialogResult.OK)
            {

                // Prfen der Selektion
                if (_tvControls.SelectedNode != null)
                {
                    // Element hinzufgen
                    try
                    {
                        // Typname zusammenstellen
                        // Der Einfachheit halber habe ich nur Klassen aus dem Forms
                        // Namespace zugelassen.
                        string strType = string.Format("System.Windows.Forms.{0}," +
                            "System.Windows.Forms, Version=1.0.5000.0," +
                            "Culture=neutral, PublicKeyToken=b77a5c561934e089",
                            dlgClassChooser._lvClass.SelectedItems[0].Text);

                        // Typobjekt herstellen
                        Type objType = Type.GetType(strType);

                        // Objekt erstellen
                        object objNewContrl = Activator.CreateInstance(objType);
                        Control objCntrl = _tvControls.SelectedNode.Tag as Control;
                        objCntrl.Controls.Add((Control)objNewContrl);

                        // TreeView neu aufbauen
                        RefreshTree();
                    }
                    catch (Exception exc)
                    {
                        string strError = string.Format(
                            "Steuerelement konnte nicht eingefgt werden.\r\nGrund: {0}",
                            exc.Message);
                        MessageBox.Show(
                            strError,
                            "Demo Controls",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                    }
                }
            }
        }

        /// <summary>
        /// Dynamisches Lschene eines Elements
        /// </summary>
        private void _ctxDelete_Click(object sender, EventArgs e)
        {
            // Prfen der Selektion
            if (_tvControls.SelectedNode != null)
            {

                string strCheck = string.Format(
                    "Besttigen Sie das Lschen des Elements {0} mit Ja.",
                    _tvControls.SelectedNode.ToString());
                // Besttigung zum Lschen erfolgt
                if (MessageBox.Show(strCheck,
                    "Demo Controls",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.Yes)
                {

                    // Element lschen
                    try
                    {
                        Control objParent = null;
                        Control objCntrl = _tvControls.SelectedNode.Tag as Control;
                        objParent = objCntrl.Parent;
                        objParent.Controls.Remove(objCntrl);
                        // TreeView neu aufbauen
                        RefreshTree();
                    }
                    catch (Exception exc)
                    {
                        string strError = string.Format(
                            "Steuerelement konnte nicht gelscht werden.\r\nGrund: {0}",
                            exc.Message);
                        MessageBox.Show(strError,
                        "Demo Controls",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                    }
                }
            }
        }
    }
}
