﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using System.Runtime.InteropServices;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert ein MDI-Clientfenster für die Verarbeitung von Rich-Text-Format-Textdateien.
    /// </summary>
    public partial class CmdiRichTextFormat : Form, ImdiBase
    {
        // Convertiert die Einheiten von .NET (1/100 inch) in Einheiten 
        // wie sie vom Win32 API verwendet werden (twips 1/1440 inch)
        private const double anInch = 14.4;

        // Datenstrukturen für die Kommunikation mit dem RichTextBox-Steuerelement
        [StructLayout(LayoutKind.Sequential)]
        private struct RECT
        {
            public int Left;
            public int Top;
            public int Right;
            public int Bottom;
        }

        [StructLayout(LayoutKind.Sequential)]
        private struct CHARRANGE
        {
            public int cpMin;         // Erstes Zeichen des Bereichs (0 für Start des Dokuments)
            public int cpMax;         // Letztes Zeichen des Bereichs (-1 für Ende des Dokuments)
        }

        [StructLayout(LayoutKind.Sequential)]
        private struct FORMATRANGE
        {
            public IntPtr hdc;             // Aktueller Devicekontext zum zeichnen
            public IntPtr hdcTarget;       // Ziel DeviceKontext für die Formatierung
            public RECT rc;                // Region des Devicekontexts für dei Ausgabe (in twips)
            public RECT rcPage;            // Region des gesamten Devicekontexts (Seitengrösse) (in twips)
            public CHARRANGE chrg;         // Bereich des Textes der zu Zeichnen ist
        }

        // Schnittstellendefinitionen
        private const int WM_USER = 0x0400;
        private const int EM_FORMATRANGE = WM_USER + 57;

        // Import der WIN32 Methode SendMessage()
        [DllImport("USER32.dll")]
        private static extern IntPtr SendMessage(IntPtr hWnd, int iMessage, IntPtr wParam, IntPtr lParam);

        /// <summary>
        /// Definiert das letzte gedruckte Zeichen.
        /// </summary>
        private int _iLastPrintPosition;

        /// <summary>
        /// Rendering des RichTextSteuerelements für das Drucken.
        /// </summary>
        /// <param name="iSequenceFrom">Sequenznummer des ersten zu druckenden Zeichens.</param>
        /// <param name="iSequenceTo">Sequenznummer des letzten zu druckenden Zeichens.</param>
        /// <param name="objPrintPage">Daten der aktuellen Druckumgebung.</param>
        /// <returns>
        /// Sequenznummer des letzten gedruckten Zeichens. Mit dieser Sequenznummer
        /// startet der Druckvorgang der nächstenS eite</returns>
        public int Print(int iSequenceFrom, int iSequenceTo, PrintPageEventArgs objPrintPage)
        {
            // Berechnen des zu druckenden Bereichs
            RECT rectToPrint;
            rectToPrint.Top = (int)(objPrintPage.MarginBounds.Top * anInch);
            rectToPrint.Bottom = (int)(objPrintPage.MarginBounds.Bottom * anInch);
            rectToPrint.Left = (int)(objPrintPage.MarginBounds.Left * anInch);
            rectToPrint.Right = (int)(objPrintPage.MarginBounds.Right * anInch);

            // Berechnen der Seitengrösse
            RECT rectPage;
            rectPage.Top = (int)(objPrintPage.PageBounds.Top * anInch);
            rectPage.Bottom = (int)(objPrintPage.PageBounds.Bottom * anInch);
            rectPage.Left = (int)(objPrintPage.PageBounds.Left * anInch);
            rectPage.Right = (int)(objPrintPage.PageBounds.Right * anInch);

            // Herstellen des Druckkontexts
            IntPtr hDCPrint = objPrintPage.Graphics.GetHdc();

            // Definieren der Datenstruktur für die Kommunikation mit dem RichTextBox-Steuerelement
            FORMATRANGE fmtRange;
            fmtRange.chrg.cpMax = iSequenceTo;		// Erstes zu druckendes Zeichen definieren
            fmtRange.chrg.cpMin = iSequenceFrom;    // Letztes zu druckendes Zeichen definieren
            fmtRange.hdc = hDCPrint;                // Devicekontext für das Drucken übergeben
            fmtRange.hdcTarget = hDCPrint;          // Devicekontext für das Drucken übergeben
            fmtRange.rc = rectToPrint;              // Druckbereich der Seite
            fmtRange.rcPage = rectPage;             // Seitengrösse

            // Resultat initiieren
            IntPtr iSequenceNext = IntPtr.Zero;
            IntPtr wParam = IntPtr.Zero;
            wParam = new IntPtr(1);

            // Zeiger der Übergabestruktur im Speicher erzeugen
            IntPtr lParam = IntPtr.Zero;
            lParam = Marshal.AllocCoTaskMem(Marshal.SizeOf(fmtRange));
            Marshal.StructureToPtr(fmtRange, lParam, false);

            // Rendering des definierten Bereichs auslösen
            iSequenceNext = SendMessage(_rtfEdit.Handle, EM_FORMATRANGE, wParam, lParam);

            // Speicher freigeben und Devicekontext freigeben
            Marshal.FreeCoTaskMem(lParam);
            objPrintPage.Graphics.ReleaseHdc(hDCPrint);

            // Rückgabe des letzten Druckzeichens
            return iSequenceNext.ToInt32();
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiRichTextFormat.
        /// </summary>
        public CmdiRichTextFormat()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Laden einer Datei vom Dateisystem.
        /// </summary>
        /// <param name="strFileName">Dateiname aus dem die Daten geladen werden sollen.</param>
        public void LoadFile(string strFileName)
        {
            _rtfEdit.LoadFile(strFileName);
        }

        /// <summary>
        /// Ereignisbehandlung wenn das MDI-Clientformular aktiviert wird.
        /// </summary>
        private void CmdiTypeRtf_Activated(object sender, EventArgs e)
        {
            ((CwlbMainWindow)MdiParent).Bold += new EventHandler<EventArgs>(this.ContextMenuBold_Click);
            ((CwlbMainWindow)MdiParent).Underscore += new EventHandler<EventArgs>(this.ContextMenuUnderscore_Click);
            ((CwlbMainWindow)MdiParent).List += new EventHandler<EventArgs>(this.ContextMenuList_Click);
        }

        /// <summary>
        /// Ereignisbehandlung wenn das MDI-Clientformular deaktiviert wird.
        /// </summary>
        private void CmdiTypeRtf_Deactivate(object sender, EventArgs e)
        {
            ((CwlbMainWindow)MdiParent).Bold -= new EventHandler<EventArgs>(this.ContextMenuBold_Click);
            ((CwlbMainWindow)MdiParent).Underscore -= new EventHandler<EventArgs>(this.ContextMenuUnderscore_Click);
            ((CwlbMainWindow)MdiParent).List -= new EventHandler<EventArgs>(this.ContextMenuList_Click);
        }


        /// <summary>
        /// Ereignisbehandlung des Menübefehls 'Speichern unter ...'
        /// </summary>
        private void MenuFileSaveAs_Click(object sender, EventArgs e)
        {
            // Lese den Dateinamen über das Standarddialogfeld ein und prüfe die Gültigkeit 
            string strFileName = CwlbMainWindow.SaveFile("RTF-Datei speichern", "RTF-Datei speichern|*.rtf", "*.rtf");
            if (!string.IsNullOrEmpty(strFileName))
            {
                try
                {
                    // Speichern des Inhalts als RTF Datei
                    _rtfEdit.SaveFile(strFileName);
                    this.Text = strFileName;
                }
                catch (Exception excObject)
                {
                    MessageBox.Show(
                        string.Format(
                            "Wärend dem Speichern der Datei ist folgender Fehler aufgetreten:{0}{1}",
                            Environment.NewLine,
                            excObject.Message),
                        CwlbMainWindow.TITLE,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Kontextmenübefehl 'Auswahl der Schriftart'.
        /// </summary>
        private void ContextMenuFont_Click(object sender, EventArgs e)
        {
            // Dialogfeld für die Auswahl einer Schriftart
            FontDialog dlgFont = new FontDialog();

            // Initialisieren
            dlgFont.AllowScriptChange = true;
            dlgFont.AllowSimulations = true;
            dlgFont.AllowVectorFonts = true;
            dlgFont.AllowVerticalFonts = true;
            dlgFont.ShowEffects = true;
            dlgFont.ShowColor = true;
            dlgFont.Font = _rtfEdit.SelectionFont;
            dlgFont.ScriptsOnly = false;
            dlgFont.ShowApply = true;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgFont.ShowDialog() == DialogResult.OK)
            {
                // Auf Auswahl anwenden
                _rtfEdit.SelectionFont = dlgFont.Font;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü Fettschrift
        /// </summary>
        private void ContextMenuBold_Click(object sender, EventArgs e)
        {
            // Fettschrift definieren
            Font fontSelecttion = new Font(_rtfEdit.SelectionFont, FontStyle.Bold);
            _rtfEdit.SelectionFont = fontSelecttion;
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü unterstrichene Schrift
        /// </summary>
        private void ContextMenuUnderscore_Click(object sender, EventArgs e)
        {
            // Text unterstreichen
            Font fontSelecttion = new Font(_rtfEdit.SelectionFont, FontStyle.Underline);
            _rtfEdit.SelectionFont = fontSelecttion;
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü Aufzählung
        /// </summary>
        private void ContextMenuList_Click(object sender, EventArgs e)
        {
            // Aufzählung formatieren
            _rtfEdit.SelectionBullet = true;
            _rtfEdit.SelectionHangingIndent = 10;
            _rtfEdit.SelectionIndent = 10;
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü Farbe
        /// </summary>
        private void ContextMenuColor_Click(object sender, EventArgs e)
        {
            // Dialogfeld für die Auswahl einer Farbe erstellen
            ColorDialog dlgColor = new ColorDialog();

            // Dialogfeld initialisieren
            dlgColor.AllowFullOpen = true;
            dlgColor.AnyColor = true;
            dlgColor.CustomColors = new int[] { 6579300, 13158600 };
            dlgColor.Color = _rtfEdit.SelectionColor;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgColor.ShowDialog() == DialogResult.OK)
            {

                // Farbe übernehmen
                _rtfEdit.SelectionColor = dlgColor.Color;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Beginnen des Druckvorgangs.
        /// </summary>
        public void PrintBegin(object sender, PrintEventArgs e)
        {
            // Definiere den Start des Druckens
            _iLastPrintPosition = 0;
        }

        /// <summary>
        /// Ereignisbehandlung für das Drucken der Seite
        /// </summary>
        public void PrintPage(object sender, PrintPageEventArgs e)
        {
            // Drucke den Inhalt des RichTextBox-Steuerelements
            _iLastPrintPosition = this.Print(_iLastPrintPosition, _rtfEdit.TextLength, e);

            // Prüfe ob weitere Seiten vorhanden sind
            if (_iLastPrintPosition < _rtfEdit.TextLength)
            {
                e.HasMorePages = true;
            }
            else
            {
                e.HasMorePages = false;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Beenden des Druckvorgangs.
        /// </summary>
        public void PrintEnd(object sender, PrintEventArgs e)
        {
            // Keine Aktion für diesen Druckvorgang
        }

        /// <summary>
        /// Ereignisbehandlung Link wurde geklickt
        /// </summary>
        private void RtfEdit_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            // Starte einen Prozess
            try
            {
                System.Diagnostics.Process.Start(e.LinkText);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, CwlbMainWindow.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
