﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Logik des Reflexions-PropertyGrid-Steuerelements.
    /// </summary>
    public partial class CctlReflectionPropertyGrid : UserControl
    {
        /// <summary>
        /// Randabzug bei Spalten.
        /// </summary>
        private const int MARGIN_DEDUCATION = 30;

        /// <summary>
        /// Objekt, das das Steuerelement anzeigt.
        /// </summary>
        private object _objData;

        /// <summary>
        /// Stringbuilder aus Performancegründen als Instanzobjekt angelegt.
        /// </summary>
        private StringBuilder _strBuilder = new StringBuilder();

        /// <summary>
        /// Liste für den Schnellzugriff auf die Knoten des Baums.
        /// </summary>
        private List<TreeNode> _cobjTreeNodes = new List<TreeNode>();

        /// <summary>
        /// Liefert oder selektiert das Objekt, das das Steuerelement anzeigt.
        /// </summary>
        public object SelectedObject 
        {
            get
            {
                return _objData;
            }
            set
            {
                _objData = value;
                ShowData();
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CctlReflectionPropertyGrid.
        /// </summary>
        public CctlReflectionPropertyGrid()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Zeigt die Daten im gesamten Steuerelement.
        /// </summary>
        private void ShowData()
        {
            // Typen listen löschen. Die Methoden und Eigenschaften
            // werden beim Neuaufbau der Typen automatisch gelöscht.
            _tvType.Nodes.Clear();
            _lvInterface.Items.Clear();
            _cobjTreeNodes.Clear();

            // Set Properties
            _pgrSelected.SelectedObject = _objData;

            // Set the page informations
            if (SelectedObject != null)
            {
                ShowTypeInformation();
                ShowMethodInformation();
                ShowEventInformation();
                RefreshStatistics();

                // Header Informationen anzeigen
                _lblTypeName.Text = SelectedObject.GetType().FullName;
            }
            else
            {
                // Designer Text eliminieren
                _lblTypeName.Text = string.Empty;
            }
        }

        /// <summary>
        /// Zeigt die Typeninfromationen auf der Typseite an.
        /// </summary>
        private void ShowTypeInformation()
        {
            ShowClassInformation(SelectedObject.GetType());
            ShowInterfaceInformation();
        }

        /// <summary>
        /// Zeigt die Infromationen der Klasse an
        /// </summary>
        private TreeNodeCollection ShowClassInformation(Type objType)
        {
            // Lokale Daten
            TreeNode objNode;
            TreeNodeCollection objParentNodeCollection = null;

            // Zuerst rekursiv zur Basis steigen
            if (objType != typeof(object))
            {
                objParentNodeCollection = ShowClassInformation(objType.BaseType);
            }

            // Knoten erstellen
            objNode = new TreeNode();
            objNode.Text = objType.Name;
            objNode.ToolTipText = string.Format(
                "{0}{1}{2}",
                objType.FullName,
                Environment.NewLine,
                objType.Assembly.FullName);
            objNode.ImageKey = objNode.SelectedImageKey = "Class.bmp";
            
            // Knoten hinzufügen
            if (objParentNodeCollection == null)
            {
                // Wurzelknoten
                objParentNodeCollection = _tvType.Nodes;
            }
            objParentNodeCollection.Add(objNode);
            _cobjTreeNodes.Add(objNode);

            // Beim letzten Knoten die Checkbox deaktivieren
            if (objType == SelectedObject.GetType())
            {
                objNode.EnsureVisible();
                objNode.Checked = true;
                objNode.Tag = true;
            }

            // Aktuelle Nodeliste zurückgeben
            return objNode.Nodes;
        }

        /// <summary>
        /// Zeigt die Informationen der Klasse an:
        /// </summary>
        private void ShowInterfaceInformation()
        {
            // Lokale Daten
            ListViewItem objItem;

            // Alle Schnittstelleninformationen lesen
            foreach (Type objType in SelectedObject.GetType().GetInterfaces())
            {
                objItem = new ListViewItem();
                objItem.Text = objType.Name;
                objItem.ToolTipText = string.Format(
                    "{0}{1}{2}",
                    objType.FullName,
                    Environment.NewLine,
                    objType.Assembly.FullName);
                objItem.ImageKey = "Interface.bmp";
                _lvInterface.Items.Add(objItem);
            }
        }

        /// <summary>
        /// Zeigt die Methodeninfromationen auf der Methodenseite an.
        /// </summary>
        private void ShowMethodInformation()
        {
            // Lokale Daten
            ListViewItem objItem;
            TreeNode objNode;

            // Leere Liste
            _lvMethods.Items.Clear();

            // Alle Methodeninformationen lesen
            foreach (MethodInfo objMethod in SelectedObject.GetType()
                .GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)
                .Where(objSearch => !objSearch.IsSpecialName)
                .OrderBy(objOrder => objOrder.Name))
            {
                // Prüfe ob der Knoten zu dem die Methode gehört selektiert ist
                objNode = _cobjTreeNodes.FirstOrDefault(objSearch => objSearch.Text == objMethod.DeclaringType.Name);
                if (objNode == null || !objNode.Checked)
                {
                    continue;
                }
                
                objItem = new ListViewItem();
                objItem.Text = string.Format("{0}()", objMethod.Name);
                objItem.ImageKey = "Method.bmp";
                objItem.ToolTipText = CreateMethodSignature(objMethod);
                // Knoten in Liste hinzufügen
                _lvMethods.Items.Add(objItem);
            }
        }


        /// <summary>
        /// Erstellt eine Methodensignatur.
        /// </summary>
        /// <param name="objMethod"></param>
        /// <returns></returns>
        private string CreateMethodSignature(MethodInfo objMethod)
        {
            // Locals
            bool bHasParams = false;

            // StringBuilder zurücksetzen
            _strBuilder.Clear();
            _strBuilder.Append("public");
            _strBuilder.Append(objMethod.IsStatic ? " static" : "");
            _strBuilder.AppendFormat(" {0}", objMethod.ReturnType.Name);
            _strBuilder.AppendFormat(" {0}(", objMethod.Name);
            foreach (ParameterInfo objInfo in objMethod.GetParameters())
            {
                if (bHasParams)
                {
                    _strBuilder.Append(",");
                }
                else
                {
                    bHasParams = true;
                }
                _strBuilder.Append(Environment.NewLine);
                _strBuilder.AppendFormat("     {0} {1}",
                    objInfo.ParameterType.Name,
                    objInfo.Name
                    );
            }

            // Abschluss prüfen
            if (bHasParams)
            {
                _strBuilder.Append(Environment.NewLine);
            }
            _strBuilder.AppendFormat(")", objMethod.Name);
            return _strBuilder.ToString();
        }

        /// <summary>
        /// Zeigt die Ereignisinformationen auf der Ereignisseite an.
        /// </summary>
        private void ShowEventInformation()
        {
            // Lokale Daten
            ListViewItem objItem;
            TreeNode objNode;

            // Leere Liste
            _lvEvents.Items.Clear();

            // Alle Methodeninformationen lesen
            foreach (EventInfo objEvent in SelectedObject.GetType()
                .GetEvents(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)
                .OrderBy(objOrder => objOrder.Name))
            {
                // Prüfe ob der Knoten zu dem die Methode gehört selektiert ist
                objNode = _cobjTreeNodes.FirstOrDefault(objSearch => objSearch.Text == objEvent.DeclaringType.Name);
                if (objNode == null || !objNode.Checked)
                {
                    continue;
                }

                objItem = new ListViewItem();
                objItem.ImageKey = "Event.bmp";
                objItem.Text = string.Format("{0}", objEvent.Name);
                // Knoten in Liste hinzufügen
                _lvEvents.Items.Add(objItem);
            }
        }

        /// <summary>
        /// Nachführen der Statistikinformationen.
        /// </summary>
        private void RefreshStatistics()
        {
            _statusClass.Text = _cobjTreeNodes.Count.ToString();
            _statusInterface.Text = _lvInterface.Items.Count.ToString();
            _statusMethods.Text = _lvMethods.Items.Count.ToString();
            _statusEvents.Text = _lvEvents.Items.Count.ToString();
            // Nicht ganz korrekt. Verwenden Sie die nachfolgende Zeile anstelle des
            // Codes darunter zum erreichend er korrekten Menge.
            // TypeDescriptor.GetProperties(component);
            _statusProperties.Text = SelectedObject.GetType().GetProperties().Where(objSearch =>
            {
                object[] aobjAttribute = objSearch.GetCustomAttributes(typeof(BrowsableAttribute), true);
                if (aobjAttribute.Length == 0 || ((BrowsableAttribute)aobjAttribute[0]).Browsable == true)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }).Count().ToString();
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Steuerelements.
        /// </summary>
        private void CctlReflectionPropertyGrid_Load(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Grösse des Fensters.
        /// </summary>
        private void CctlReflectionPropertyGrid_SizeChanged(object sender, EventArgs e)
        {
            // Einstellen der Spaltenbreiten in den Listen
            _colInterfaces.Width = this.Size.Width - MARGIN_DEDUCATION;
            _colMethods.Width = this.Size.Width - MARGIN_DEDUCATION;
            _colEvents.Width = this.Size.Width - MARGIN_DEDUCATION;
        }

        /// <summary>
        /// Ereignisbehandlung vor dem Checken des Klassenknotens.
        /// </summary>
        private void Type_BeforeCheck(object sender, TreeViewCancelEventArgs e)
        {
            if (e.Node.Tag != null)
            {
                e.Cancel = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Checken des Klassenknotens.
        /// </summary>
        private void Type_AfterCheck(object sender, TreeViewEventArgs e)
        {
            // Refresh der Methoden und Ereignisse.
            ShowMethodInformation();
            RefreshStatistics();
        }

        /// <summary>
        /// Ereignisbehandlung für das Label-Steuerelement der Typangabe.
        /// </summary>
        /// <remarks>
        /// Diesee Ereignisbehandlung ist wegen einem Fehler im Label-Steuerelement im Zusammenhang
        /// mit der Autoellipsis notwendig
        /// </remarks>
        private void _lblTypeName_Paint(object sender, PaintEventArgs e)
        {
            // Lokale Daten
            Size sizePreferred = _lblTypeName.GetPreferredSize(_lblTypeName.ClientSize);
            StringBuilder strDrawing = new StringBuilder();
            int iPos = 0;
            Size sizeText = new Size();

            try
            {

                // Prüfen, ob die Spezialbehandlung erfoglen muss
                if (_lblTypeName.ClientSize.Width < sizePreferred.Width || _lblTypeName.ClientSize.Height < sizePreferred.Height)
                {
                    // Löschen des Inhalts
                    using (Brush brushClear = new SolidBrush(_lblTypeName.BackColor))
                    {
                        e.Graphics.FillRectangle(brushClear, e.ClipRectangle);
                    }

                    // Darzustellenden Text zusammensetzen, solange die Textlänge kleiner ist als 
                    // die Länge des Ausgaberechtecks
                    while (sizeText.Width < e.ClipRectangle.Width - 20 && iPos < _lblTypeName.Text.Length)
                    {
                        // Zeichen an Ausgabetext anfügen
                        strDrawing.Append(_lblTypeName.Text[iPos++]);
                        // Textlänge messen
                        sizeText = TextRenderer.MeasureText(strDrawing.ToString(), _lblTypeName.Font);
                    }

                    // Prüfen ob, der ganze Text dargestellt werden kann und 
                    // Ellipiszeichen anfügen wenn nicht
                    if (iPos < _lblTypeName.Text.Length)
                    {
                        strDrawing.Remove(strDrawing.Length - 1, 1);
                        strDrawing.Append("...");
                    }

                    // Text ausgeben
                    if (strDrawing.Length > 0)
                    {
                        using (Brush brushFont = new SolidBrush(_lblTypeName.ForeColor))
                        {
                            // Text ausgeben
                            e.Graphics.DrawString(
                                strDrawing.ToString(), // Text
                                _lblTypeName.Font,     // Schriftart
                                brushFont,             // Brush 
                                new PointF(            // Position
                                    e.ClipRectangle.X,
                                    e.ClipRectangle.Y + ((e.ClipRectangle.Height - sizeText.Height) / 2)));
                        }
                    }
                }
            }
            catch
            {
                // Notfausgang, wenn ich mich oben irgendwo vertan hätte.
            }
        }
    }
}
