﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Windows.Forms.DataVisualization.Charting;
using Microsoft.VisualBasic.PowerPacks;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Hauptfensters für die Demos der Steuerelemente
    /// </summary>
    public partial class CwlbMainWindow : Form
    {
        // Definiert den WIN32 Code für das Windwos Ereignis linke Maustaste
        private const int WM_LBUTTONDOWN = 0x0201;
        // Definiert ein Ereignisfilter für die dynamische Behandlung des Dialogs
        private CwlbMessageFilter _objMessageFilter;
        // Definiert ein Synchronisationsobjekt für das UI
        public static readonly AutoResetEvent SyncUI = new AutoResetEvent(false);

        /// <summary>
        /// Zeigt sich selber an.
        /// </summary>
        public static CwlbMainWindow MainForm { get; private set; }

        /// <summary>
        /// Definiert den Titel der Anwendung.
        /// </summary>
        public static string TITLE = "Learning Windows Forms 04";

        /// <summary>
        /// Liefert oder definiert den Titeltext.
        /// </summary>
        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                base.Text = string.Format("{0} - [{1}]", TITLE, value);
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbMainWindow.
        /// </summary>
        public CwlbMainWindow()
        {
            // Initialiserung der Steuerelement des Hauptfensters
            InitializeComponent();

            // Registrieren des Nachrichtenfilters und der Leerlaufverarbeitung
            _objMessageFilter = new CwlbMessageFilter(_pnlControl, WM_LBUTTONDOWN);
            _objMessageFilter.MessageReceived += new EventHandler<CwlbMessageFilterEventArgs>(MessageFilter_Click);
            Application.AddMessageFilter(_objMessageFilter);
            Application.Idle += new EventHandler(Application_Idle);
            Application.ThreadException += new ThreadExceptionEventHandler(Application_ThreadException);
            // Singleton Zugriff ermöglichen
            MainForm = this;
        }

        /// <summary>
        /// Ereignisbehandlung für eine unbehandlete Ausnahme des Hauptthreads.
        /// </summary>
        private void Application_ThreadException(object sender, ThreadExceptionEventArgs e)
        {
            MessageBox.Show(e.Exception.Message, TITLE);
        }

        /// <summary>
        /// Ereignisbehandlung für die Leerlaufverarbeitung.
        /// </summary>
        private void Application_Idle(object sender, EventArgs e)
        {
            if (_pnlControl.Controls.Count > 1)
            {
                _lblExplanation.Visible = false;
            }
            else
            {
                _lblExplanation.Visible = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Click auf ein Steuerelement im Demobereich.
        /// </summary>
        void MessageFilter_Click(object sender, CwlbMessageFilterEventArgs e)
        {
            this._pgrControl.SelectedObject = sender;
        }

        /// <summary>
        /// Erstellt einen Knoten in der Liste der Steuerelemente.
        /// </summary>
        /// <param name="objCollection">Liste in die der Knoten eingeängt werden soll.</param>
        /// <param name="strName">Beschriftung des Knotens.</param>
        /// <param name="typePanel">Type des Panels das die Demo beinhaltet.</param>
        /// <param name="strImageKey">Name des Schlüssels in der Bildliste.</param>
        /// <param name="objControlType">Typ des .NET-Steuerelements das für die Demo benutzt wird</param>
        /// <returns>Knoten der erzeugt wurde.</returns>
        private TreeNode CreateNode(TreeNodeCollection objCollection, string strName, Type typePanel, string strImageKey = null, Type objControlType = null)
        {
            try
            {
                TreeNode objNode = new TreeNode(strName);
                if (string.IsNullOrEmpty(strImageKey))
                {
                    strImageKey = string.Format("{0}.bmp", strName);
                }
                objNode.Tag = new CwlbPanel { PanelType = typePanel, DemoControl = objControlType };
                objNode.ImageKey = objNode.SelectedImageKey = strImageKey;
                objCollection.Add(objNode);
                return objNode;
            }
            catch
            {
                // Alle Fehler schlucken
                return null;
            }
        }

        /// <summary>
        /// Erzeugt die Liste der einfachen Steuerelemente.
        /// </summary>
        private void CreateSimpleControls()
        {
            TreeNode nodeSimple = CreateNode(_tvControls.Nodes, "Einfache Steuerelemente", null, "Folder.bmp");
            if (nodeSimple != null)
            {
                CreateNode(nodeSimple.Nodes, "Label", typeof(CpnlLabel), objControlType: typeof(Label));
                CreateNode(nodeSimple.Nodes, "LinkLabel", typeof(CpnlLinkLabel), objControlType: typeof(LinkLabel));
                CreateNode(nodeSimple.Nodes, "TextBox", typeof(CpnlTextBox), objControlType: typeof(TextBox));
                CreateNode(nodeSimple.Nodes, "MaskedTextBox", typeof(CpnlMaskedTextBox), objControlType: typeof(MaskedTextBox));
                CreateNode(nodeSimple.Nodes, "DateTimePicker", typeof(CpnlDateTimePicker), objControlType: typeof(DateTimePicker));
                CreateNode(nodeSimple.Nodes, "RadioButton", typeof(CpnlRadioButton), objControlType: typeof(RadioButton));
                CreateNode(nodeSimple.Nodes, "CheckBox", typeof(CpnlCheckBox), objControlType: typeof(CheckBox));
                CreateNode(nodeSimple.Nodes, "ListBox", typeof(CpnlListBox), objControlType: typeof(ListBox));
                CreateNode(nodeSimple.Nodes, "CheckedListBox", typeof(CpnlCheckedListBox), objControlType: typeof(CheckedListBox));
                CreateNode(nodeSimple.Nodes, "ComboBox", typeof(CpnlComboBox), objControlType: typeof(ComboBox));
                CreateNode(nodeSimple.Nodes, "Button", typeof(CpnlButton), objControlType: typeof(Button));
                CreateNode(nodeSimple.Nodes, "DomainUpDown", typeof(CpnlDomainUpDown), objControlType: typeof(DomainUpDown));
                CreateNode(nodeSimple.Nodes, "NumericUpDown", typeof(CpnlNumericUpDown), objControlType: typeof(NumericUpDown));
                CreateNode(nodeSimple.Nodes, "ProgressBar", typeof(CpnlProgressBar), objControlType: typeof(ProgressBar));
                CreateNode(nodeSimple.Nodes, "TrackBar", typeof(CpnlTrackBar), objControlType: typeof(TrackBar));
                CreateNode(nodeSimple.Nodes, "PictureBox", typeof(CpnlPictureBox), objControlType: typeof(PictureBox));
            }
        }

        /// <summary>
        /// Erzeugt die Liste der mehrgliedrigen Steuerelemente.
        /// </summary>
        private void CreatePolynominalControls()
        {
            TreeNode nodePolynominal = CreateNode(_tvControls.Nodes, "Mehrgliedrige Steuerelemente", null, "Folder.bmp");
            if (nodePolynominal != null)
            {
                CreateNode(nodePolynominal.Nodes, "ListView", typeof(CpnlListView), objControlType: typeof(ListView));
                CreateNode(nodePolynominal.Nodes, "TreeView", typeof(CpnlTreeView), objControlType: typeof(TreeView));
                CreateNode(nodePolynominal.Nodes, "MonthCalendar", typeof(CpnlMonthCalendar), objControlType: typeof(MonthCalendar));
                CreateNode(nodePolynominal.Nodes, "RichTextBox", typeof(CpnlRichTextBox), objControlType: typeof(RichTextBox));
                CreateNode(nodePolynominal.Nodes, "PropertyGrid", typeof(CpnlPropertyGrid), objControlType: typeof(PropertyGrid));
                CreateNode(nodePolynominal.Nodes, "WebBrowser", typeof(CpnlWebBrowser), objControlType: typeof(WebBrowser));
                CreateNode(nodePolynominal.Nodes, "Chart", typeof(CpnlChart), objControlType: typeof(Chart));
                CreateNode(nodePolynominal.Nodes, "DataGridView", typeof(CpnlDataGridView), objControlType: typeof(DataGridView));
            }
        }

        /// <summary>
        /// Erzeugt die Liste der Steuerelemente zur Anordnung anderer Steuerelemente.
        /// </summary>
        private void CreateArrangementControls()
        {
            TreeNode nodeArrangement = CreateNode(_tvControls.Nodes, "Steuerelemente zur Anordnung", null, "Folder.bmp");
            if (nodeArrangement != null)
            {
                CreateNode(nodeArrangement.Nodes, "GroupBox", typeof(CpnlRadioButton), "GroupBox.bmp", objControlType: typeof(GroupBox));
                CreateNode(nodeArrangement.Nodes, "TabControl", typeof(CpnlTabControl), objControlType: typeof(TabControl));
                CreateNode(nodeArrangement.Nodes, "Panel", typeof(CpnlPanel), objControlType: typeof(Panel));
                CreateNode(nodeArrangement.Nodes, "Splitter", typeof(CpnlSplitter), objControlType: typeof(Splitter));
                CreateNode(nodeArrangement.Nodes, "SplitContainer", typeof(CpnlSplitContainer), objControlType: typeof(SplitContainer));
                CreateNode(nodeArrangement.Nodes, "FlowLayoutPanel", typeof(CpnlFlowLayoutPanel), objControlType: typeof(FlowLayoutPanel));
                CreateNode(nodeArrangement.Nodes, "TableLayoutPanel", typeof(CpnlTableLayoutPanel), objControlType: typeof(TableLayoutPanel));
                CreateNode(nodeArrangement.Nodes, "ToolStripContainer", typeof(CpnlToolStripContainer), objControlType: typeof(ToolStripContainer));
                CreateNode(nodeArrangement.Nodes, "ToolStripPanel", typeof(CpnlToolStripPanel), "ToolStrip.bmp", objControlType: typeof(ToolStripPanel));
            }
        }

        /// <summary>
        /// Erzeugt die Liste der Steuerelemente für die Navigation.
        /// </summary>
        private void CreateNavigationControls()
        {
            TreeNode nodeNavigation = CreateNode(_tvControls.Nodes, "Steuerelemente zur Navigation", null, "Folder.bmp");
            if (nodeNavigation != null)
            {
                CreateNode(nodeNavigation.Nodes, "MenuStrip", typeof(CpnlMenuStrip), "MainMenu.bmp", objControlType: typeof(MenuStrip));
                CreateNode(nodeNavigation.Nodes, "ContextMenuStrip", typeof(CpnlContextMenuStrip), "ContextMenu.bmp", objControlType: typeof(ContextMenuStrip));
                CreateNode(nodeNavigation.Nodes, "ToolStrip", typeof(CpnlToolStrip), objControlType: typeof(ToolStrip));
            }
        }

        /// <summary>
        /// Erzeugt die Liste der Steuerelemente für die Benutzerunterstützung.
        /// </summary>
        private void CreateUserSupportControls()
        {
            TreeNode nodeUserSupport = CreateNode(_tvControls.Nodes, "Steuerelemente zur Benutzerunterstützung", null, "Folder.bmp");
            if (nodeUserSupport != null)
            {
                CreateNode(nodeUserSupport.Nodes, "ToolTip", typeof(CpnlToolTip), objControlType: typeof(ToolTip));
                CreateNode(nodeUserSupport.Nodes, "StatusStrip", typeof(CpnlStatusStrip), "StatusBar.bmp", objControlType: typeof(StatusStrip));
                CreateNode(nodeUserSupport.Nodes, "NotifyIcon", typeof(CpnlNotifyIcon), objControlType: typeof(NotifyIcon));
                CreateNode(nodeUserSupport.Nodes, "ErrorProvider", typeof(CpnlErrorProvider), objControlType: typeof(ErrorProvider));
            }
        }

        /// <summary>
        /// Erzeugt die Liste der Steuerelemente für die Benutzerunterstützung.
        /// </summary>
        private void CreateMiscControls()
        {
            TreeNode nodeMisc = CreateNode(_tvControls.Nodes, "Sonstige Steuerelemente", null, "Folder.bmp");
            if (nodeMisc != null)
            {
                CreateNode(nodeMisc.Nodes, "ImageList", typeof(CpnlImageList), objControlType: typeof(ImageList));
                CreateNode(nodeMisc.Nodes, "Timer", typeof(CpnlTimer), objControlType: typeof(System.Windows.Forms.Timer));
                CreateNode(nodeMisc.Nodes, "ElementHost", typeof(CpnlElementHost), objControlType: typeof(System.Windows.Forms.Integration.ElementHost));
                CreateNode(nodeMisc.Nodes, "LineShape", typeof(CpnlShape), "Line.bmp", objControlType: typeof(LineShape));
                CreateNode(nodeMisc.Nodes, "RectangleShape", typeof(CpnlShape), "Rectangle.bmp", objControlType: typeof(RectangleShape));
                CreateNode(nodeMisc.Nodes, "OvalShape", typeof(CpnlShape), "Oval.bmp", objControlType: typeof(OvalShape));
                CreateNode(nodeMisc.Nodes, "ByteViewer", typeof(CpnlByteViewer), objControlType: typeof(System.ComponentModel.Design.ByteViewer));
            }
        }

        /// <summary>
        /// Zeigt das definierte Objekt in der Eigenschaftsansicht an.
        /// </summary>
        /// <param name="objControl"></param>
        public void SetPropertyControl(object objControl)
        {
            _pgrControl.SelectedObject = objControl;
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Elements.
        /// </summary>
        private void CwlbMainWindow_Load(object sender, EventArgs e)
        {
            // Erstellen der Listen für Steuerelemente
            CreateSimpleControls();
            CreatePolynominalControls();
            CreateArrangementControls();
            CreateNavigationControls();
            CreateUserSupportControls();
            CreateMiscControls();

            // Starten der Hintergrundverarbeitung
            _threadBackground.RunWorkerAsync();
        }

        /// <summary>
        /// Ereignisbehandlung für die Auswahl einer Demo eines Steuerelements im Baum.
        /// </summary>
        private void Controls_DoubleClick(object sender, EventArgs e)
        {
            // Lokale Daten
            CwlbPanel panelControl;

            // Aktuell ausgewähltes Element abfragen.
            panelControl = _tvControls.SelectedNode.Tag as CwlbPanel;
            // Wenn das Element nicht mit einder Demo verbunden ist, die Bearbeitung sofort abbrechen.
            if (panelControl.PanelType == null)
            {
                return;
            }

            // Prüfen, ob das Formular bereits hergestellt wurde. Wenn nicht dieses per
            // Reflektion herstellen und an das Verwaltungsobjekt binden.
            if (panelControl.Control == null)
            {
                panelControl.Control = Activator.CreateInstance(panelControl.PanelType) as Control;
            }

            // Angezeigtes Steuerelement austauschen
            _pnlControl.Controls.Clear();
            _pnlControl.Controls.Add(panelControl.Control);
            panelControl.Control.Dock = DockStyle.Fill;
            // Titel des Hauptfensters zurücksetzen
            if (panelControl.DemoControl != null)
            {
                Text = string.Format("Steuerelement {0}", panelControl.DemoControl.FullName);
            }
        }

        /// <summary>
        /// Verarbeitungsmethode für den Hintergrundthread.
        /// </summary>
        private void ThreadBackground_DoWork(object sender, DoWorkEventArgs e)
        {
            while (SyncUI.WaitOne(1000))
            {
                Thread.Sleep(100);
                _threadBackground.ReportProgress(0);
            }
        }

        /// <summary>
        /// Synchronisationsmethode für den Threadwechsel vom Arbeitsthread.
        /// </summary>
        private void ThreadBackground_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            _pgrControl.Refresh();
        }
    }
}
