﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert das DateTimePicker-Steuerelement zur Verwendung als Steuerlement in einer Zelle
    /// eines DataGridView-Steuerelements.
    /// </summary>
    public class CwlbDataGridDateTimePicker : DateTimePicker, IDataGridViewEditingControl
    {
        // Instanzdaten
        private DataGridView _dgrView;
        private bool _bValueChanged = false;
        private int _iRowIndex;

        /// <summary>
        /// Initialsieert eine neue Instanz der Klasse CwlbDataGridDateTimePicker
        /// </summary>
        public CwlbDataGridDateTimePicker()
        {
            this.Format = DateTimePickerFormat.Short;
        }

        /// <summary>
        /// Liefert oder definiert den formatierten Wert der Zelle.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft ist in der Schnittstelle IDataGridViewEditingControl vorgeschrieben.
        /// </remarks>
        public object EditingControlFormattedValue
        {
            get
            {
                return this.Value.ToShortDateString();
            }
            set
            {
                try
                {
                    // Wert übernehmen
                    this.Value = DateTime.Parse((String)value);
                }
                catch
                {
                    // Wennd er Wert nicht übernommen werden kann wird 
                    // das Tagesdatum definiert.
                    this.Value = DateTime.Now;
                }
            }
        }

        /// <summary>
        /// Liefert oder definiert den Index der Zelle, die das Steuerelement besitzt.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft ist in der Schnittstelle IDataGridViewEditingControl vorgeschrieben.
        /// </remarks>
        public int EditingControlRowIndex
        {
            get
            {
                return _iRowIndex;
            }
            set
            {
                _iRowIndex = value;
            }
        }

        /// <summary>
        /// Liefert true, wenn der Inhalt bei Veränderungen neu positioniert werden muss.
        /// </summary>
        public bool RepositionEditingControlOnValueChange
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Liefert oder definiert die DataGridInstanz, die dieses Steuerelement kontrolliert.
        /// </summary>
        public DataGridView EditingControlDataGridView
        {
            get
            {
                return _dgrView;
            }
            set
            {
                _dgrView = value;
            }
        }

        /// <summary>
        /// Liefert oder definiert true, wenn der Wert zwischen der Zelle und dem Steuerelemtn unterschiedlich ist.
        /// </summary>
        public bool EditingControlValueChanged
        {
            get
            {
                return _bValueChanged;
            }
            set
            {
                _bValueChanged = value;
            }
        }

        /// <summary>
        /// Liefert den Cursor, der verwendete werden soll, wenn die Maus auf der Editfläche aber nicht auf dem Steuerelement ist.
        /// </summary>
        public Cursor EditingPanelCursor
        {
            get
            {
                return base.Cursor;
            }
        }

        /// <summary>
        /// Liefert den formatierten Wert des Steuerelements.
        /// </summary>
        /// <remarks>
        /// Diese Methode ist in der Schnittstelle IDataGridViewEditingControl vorgeschrieben.
        /// </remarks>
        /// <param name="objContext">Eine Bitkombination der Auflistung DataGridViewDataErrorContexts, die den aktuellen Kontext beschreibt.</param>
        /// <returns></returns>
        public object GetEditingControlFormattedValue(DataGridViewDataErrorContexts objContext)
        {
            return EditingControlFormattedValue;
        }

        /// <summary>
        /// Synchronisiert den Stil des Steuerelements mit dem Stil der Spalte des DataGridView-Steuerelements.
        /// </summary>
        /// <remarks>
        /// Diese Methode ist in der Schnittstelle IDataGridViewEditingControl vorgeschrieben.
        /// </remarks>
        /// <param name="objDataGridViewCellStyle">Stil der verwendet werden sollte.</param>
        public void ApplyCellStyleToEditingControl(DataGridViewCellStyle objDataGridViewCellStyle)
        {
            this.Font = objDataGridViewCellStyle.Font;
            this.CalendarForeColor = objDataGridViewCellStyle.ForeColor;
            this.CalendarMonthBackground = objDataGridViewCellStyle.BackColor;
        }

        /// <summary>
        /// Entscheided ob die benutzte Taste eine Taste für das Steuerelement oder für das DataGridView ist.
        /// </summary>
        /// <remarks>
        /// Diese Methode ist in der Schnittstelle IDataGridViewEditingControl vorgeschrieben.
        /// </remarks>
        /// <param name="key">Taste die betätigt wurde.</param>
        /// <param name="bDataGridViewWantsInputKey">Indiziert ob das DataGridView diese Taste verarbeiten möchte.</param>
        /// <returns>True, wenn die Taste vom Steuerelement verarbeitet wird; anderfalls false.</returns>
        public bool EditingControlWantsInputKey(Keys key, bool bDataGridViewWantsInputKey)
        {
            // Let the DateTimePicker handle the keys listed.
            switch (key & Keys.KeyCode)
            {
                case Keys.Left:
                case Keys.Up:
                case Keys.Down:
                case Keys.Right:
                case Keys.Home:
                case Keys.End:
                case Keys.PageDown:
                case Keys.PageUp:
                    return true;
                default:
                    return !bDataGridViewWantsInputKey;
            }
        }

        /// <summary>
        /// Konfiguriert das Steuerelement für das Editieren.
        /// </summary>
        /// <remarks>
        /// Diese Methode ist in der Schnittstelle IDataGridViewEditingControl vorgeschrieben.
        /// </remarks>
        /// <param name="bSelectAll">True, wenn der Inhalt vorselektiert werden soll.</param>
        public void PrepareEditingControlForEdit(bool bSelectAll)
        {
        }

        /// <summary>
        /// Überschreibt die Methode OnValueChanged des zu Grunde liegenden DateTimePicker-Steuerelement.
        /// </summary>
        protected override void OnValueChanged(EventArgs objEvantArgs)
        {
            _bValueChanged = true;
            this.EditingControlDataGridView.NotifyCurrentCellDirty(true);
            base.OnValueChanged(objEvantArgs);
        }
    }

}
