﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert die Beschreibung eines Felds einer Klasse. Eine Instanz dieser
    /// Klasse dient dazu in einem PropertyGrid-Steuerelement als Item verwendet zu werden.
    /// </summary>
    public class CwlbFieldDescriptor : PropertyDescriptor
    {
        #region Instanzdaten
        /// <summary>
        /// Datenobjekt für das diese Eigenschaftsbeschreibung gültig ist.
        /// </summary>
        private object _objData;

        /// <summary>
        /// Reflektion-Information des darzustellenden Feldes.
        /// </summary>
        private FieldInfo _objFieldInfo;
        #endregion

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbFieldDescriptor.
        /// </summary>
        /// <param name="objFieldInfo">Reflektionsinformation des Feldes.</param>
        /// <param name="objData">Daten des Feldes.</param>
        /// <param name="aobjAttributes">Attribute des Feldes.</param>
        /// <remarks>Dieser Konstruktor ist bedingt durch die Vererbung.</remarks>
        public CwlbFieldDescriptor(FieldInfo objFieldInfo, object objData, Attribute[] aobjAttributes)
            : base(objFieldInfo.Name, aobjAttributes)
        {
            // Interne Speicherung der Daten.
            _objData = objData;
            _objFieldInfo = objFieldInfo;
        }

        /// <summary>
        /// Liefert einen Wert, der aussagt, ob das Feld zurückgesetzt werden kann.
        /// </summary>
        /// <param name="objData">Datenobjekt</param>
        /// <returns>Liefert in dieser Demo immer false. Die Rücksetzung ist nicht unetrstützt.</returns>
        public override bool CanResetValue(object objData)
        {
            return false;
        }

        /// <summary>
        /// Liefert den Typ des Objekts, zu dem dieses Feld gehört.
        /// </summary>
        public override Type ComponentType
        {
            get { return _objData.GetType(); }
        }

        /// <summary>
        /// Liefert den Wert des Feldes.
        /// </summary>
        /// <param name="objData">Objekt dessen Wert gelifert werden soll.</param>
        /// <returns>Inhalt des Feldes.</returns>
        public override object GetValue(object objData)
        {
            return _objFieldInfo.GetValue(objData);
        }

        /// <summary>
        /// Liefert den Typ des Feldes.
        /// </summary>
        public override Type PropertyType
        {
            get { return _objFieldInfo.FieldType; }
        }

        /// <summary>
        /// Setzt den Wert zurück.
        /// </summary>
        /// <remarks>Diese Methode ist in der Demo nicht unterstützt.</remarks>
        /// <param name="objData">Datenobjekt.</param>
        public override void ResetValue(object objData)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Definiert den Wert des Feldes.
        /// </summary>
        /// <param name="objData">Objekt für das der Inhalt definiert werden soll.</param>
        /// <param name="objValue">Information, die in das Feld übertragenb werden soll.</param>
        public override void SetValue(object objData, object objValue)
        {
            _objFieldInfo.SetValue(objData, objValue);
        }

        /// <summary>
        /// Liefert einen Wert, der aussagt, ob das Feld schreibgeschützt ist oder nicht.
        /// </summary>
        public override bool IsReadOnly
        {
            get { return _objFieldInfo.IsInitOnly; }
        }

        /// <summary>
        /// Liefert true, wenn der Wert persistent gehalten werden soll.
        /// </summary>
        /// <param name="objData">Datenobjekt</param>
        /// <returns>True, wenn der Wert persistent gahelten werden soll; andernfalls falsch.</returns>
        public override bool ShouldSerializeValue(object objData)
        {
            return true;
        }
    }
}
