﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert eine Beschreibung einer definierten Eigenschaft für das PropertyGrid-Steuerelement.
    /// </summary>
    /// <remarks>
    /// Die Objekt dieses Typs werden über den Namen des Schlüssels an die Datenobjekte gebunden.
    /// </remarks>
    public class CwlbPropertyGridItemDescriptor : PropertyDescriptor
    {
        // Instanzdaten
        // Referenz auf die Auflistung, in der die Daten vorhanden sind.
        private CwlbPropertyGridItemCollection _objPropertyCollection = null;
        // Schlüsselname
        private string _strKey;
        // Hilfsvariable für die Veraltung der Funktion zum Zurücksetzen des Werts.
        private bool _bResetDefaultSwitch = true;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbPropertyGridItemDescriptor.
        /// </summary>
        /// <param name="objPropertyCollection">Referenz der Auflistung für die beschreibung aller Daten.</param>
        /// <param name="strKey">Schlüsselname</param>
        public CwlbPropertyGridItemDescriptor(CwlbPropertyGridItemCollection objPropertyCollection, string strKey)
            : base(strKey, null)
        {
            this._objPropertyCollection = objPropertyCollection;
            this._strKey = strKey;
        }

        /// <summary>
        /// Liefert eine Auflistung aller Attribute, die für dieses Element gelten
        /// </summary>
        /// <remarks>
        /// Diese Version unterstützt keine Attribute für dynamische Daten des PropertyGrid-Steuerelements.
        /// </remarks>
        public override AttributeCollection Attributes
        {
            get
            {
                return new AttributeCollection(null);
            }
        }

        /// <summary>
        /// Liefert immer true, weil diese Klasse das Zurücksetzen von Daten unterstützt.
        /// </summary>
        /// <param name="component">Nicht verwendet.</param>
        /// <returns>Immer true.</returns>
        public override bool CanResetValue(object component)
        {
            return true;
        }

        /// <summary>
        /// Liefert den Typ der komponente, zu dem diese Eigenschaft gebunden ist.
        /// </summary>
        public override Type ComponentType
        {
            get
            {
                return this._objPropertyCollection.GetType();
            }
        }

        /// <summary>
        /// Liefert den Anzeigenamen.
        /// </summary>
        public override string DisplayName
        {
            get
            {
                return this._objPropertyCollection[_strKey].DisplayName;
            }
        }

        /// <summary>
        /// Liefert den Namen der Eigenschaft.
        /// </summary>
        public override string Name
        {
            get
            {
                return this._objPropertyCollection[_strKey].Name;
            }
        }

        /// <summary>
        /// Liefert den Namen der Kategorie zu der die Eigenschaft gehört.
        /// </summary>
        public override string Category
        {
            get
            {
                return this._objPropertyCollection[_strKey].Category;
            }
        }

        /// <summary>
        /// Liefert die Beschreibung der Eigenschaft.
        /// </summary>
        public override string Description
        {
            get
            {
                return this._objPropertyCollection[_strKey].Description;
            }
        }

        /// <summary>
        /// Liefert einen Wert der aussagt, ob die Eigenschaft schreibgeschützt ist.
        /// </summary>
        public override bool IsReadOnly
        {
            get
            {
                return this._objPropertyCollection[_strKey].ReadOnly;
            }
        }

        /// <summary>
        /// Liefert den Wert der Eigenschaft.
        /// </summary>
        /// <param name="objComponent">bezeiht sich immer auf sich selbst (unbenutzt).</param>
        /// <returns>Wert der Eigenschaft.</returns>
        public override object GetValue(object objComponent)
        {
            return this._objPropertyCollection[_strKey].Value;
        }

        /// <summary>
        /// Definiert den Wert der Eigenschaft.
        /// </summary>
        /// <param name="objComponent">Bezieht sich immer auf sich selbst (unbenutzt).</param>
        /// <param name="objValue">Wert der Eigenschaft.</param>
        public override void SetValue(object objComponent, object objValue)
        {
            this._objPropertyCollection[_strKey].Value = objValue;
        }

        /// <summary>
        /// Liefert das Typobjekt der Eigenschaft.
        /// </summary>
        public override Type PropertyType
        {
            get
            {
                // Erzeugt das Typobjekt 
                Type objType = Type.GetType(this._objPropertyCollection[_strKey].PropertyTypeName);
                // Prüfen, ob das Typobjekt erfolgreich hergestellt werden konnte.
                // Liefere ein Typobjekt für String, wenn das eigentliche Typobjekt
                // nicht hergestellt werden konnte.
                if (objType == null)
                {
                    objType = typeof(System.String);
                }
                return objType;
            }
        }

        /// <summary>
        /// Liefert eine Instanz des Typconverters für diese Eigenschaft.
        /// Gets an instance of the converter for the property.
        /// </summary>
        public override TypeConverter Converter
        {
            get
            {
                if (this._objPropertyCollection[_strKey].ConverterTypeName == null)
                {
                    return base.Converter;
                }
                else
                {
                    Type objType = Type.GetType(this._objPropertyCollection[_strKey].ConverterTypeName);
                    return (TypeConverter)Activator.CreateInstance(objType);
                }
            }
        }

        /// <summary>
        /// Liefert eine Instanz des Editors der zu verwenden ist.
        /// </summary>
        /// <param name="editorBaseType">Basistyp des Editors.</param>
        /// <returns>Instanz des Editors.</returns>
        public override object GetEditor(Type editorBaseType)
        {
            if (this._objPropertyCollection[_strKey].EditorTypeName == null)
            {
                return base.GetEditor(editorBaseType);
            }
            else
            {
                Type objType = Type.GetType(this._objPropertyCollection[_strKey].EditorTypeName);
                return Activator.CreateInstance(objType);
            }
        }

        /// <summary>
        /// Setzt den Wert der Eigenschaft auf ihren Standartdwert zurück.
        /// </summary>
        /// <param name="objComponent">Nicht verwendet.</param>
        public override void ResetValue(object objComponent)
        {
            if (_bResetDefaultSwitch)
            {
                this._objPropertyCollection[_strKey].Value = this._objPropertyCollection[_strKey].ResetValue;
                _bResetDefaultSwitch = false;
            }
            else
            {
                this._objPropertyCollection[_strKey].Value = this._objPropertyCollection[_strKey].DefaultValue;
                _bResetDefaultSwitch = true;
            }
        }

        /// <summary>
        /// Prüft ob der Wert zurückgegeben werden soll.
        /// </summary>
        /// <param name="objComponent">Nicht verwendeet..</param>
        /// <returns>True, wennd er Wert verändert wurde, anderfalls false..</returns>
        public override bool ShouldSerializeValue(object objComponent)
        {
            if (this._objPropertyCollection[_strKey].Value == this._objPropertyCollection[_strKey].DefaultValue)
            {
                return false;
            }
            else
            {
                return true;
            }
        }
    }
}
