﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert eine Person für die Demo der Datenbindung
    /// </summary>
    class CwlbPerson : INotifyPropertyChanged
    {
        /// <summary>
        /// Wird ausgelöst, wenn eine der folgenden Eigenschaften ändert:
        /// <list type="Bullet">
        ///     <item>Title</item>
        ///     <item>Name</item>
        ///     <item>FirstName</item>
        ///     <item>ZIP</item>
        ///     <item>City</item>
        /// </list>
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region Instanzdaten
        /// <summary>
        /// Objektnummer.
        /// </summary>
        private static int _iNextNumber;

        /// <summary>
        /// Definiert die Anrede der Person.
        /// </summary>
        private string _strTitle;

        /// <summary>
        /// Definiert den Namen der Person.
        /// </summary>
        private string _strName;

        /// <summary>
        /// Definiert den Vornamen der Person.
        /// </summary>
        private string _strFirstName;

        /// <summary>
        /// Definiert die Postleitzahl der Adresse.
        /// </summary>
        private string _strZIP;

        /// <summary>
        /// Definiert den Wohnort der Person.
        /// </summary>
        private string _strCity;
        #endregion

        #region Eigenschaften
        /// <summary>
        /// Liefert oder definiert die Objektnummer.
        /// </summary>
        public int ID { get; set; }

        /// <summary>
        /// Liefert oder definiert die Anrede.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft unterstützt die Schnittstelle INotifyPropertyChanged.
        /// </remarks>
        public string Title
        {
            get
            {
                return _strTitle;
            }
            set
            {
                _strTitle = value;
                OnPropertyChanged("Title");
            }
        }

        /// <summary>
        /// Liefert oder definiert den Namen.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft unterstützt die Schnittstelle INotifyPropertyChanged.
        /// </remarks>
        public string Name
        {
            get
            {
                return _strName;
            }
            set
            {
                _strName = value;
                OnPropertyChanged("Name");
            }
        }

        /// <summary>
        /// Liefert oder definiert den Vornamen.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft unterstützt die Schnittstelle INotifyPropertyChanged.
        /// </remarks>
        public string FirstName
        {
            get
            {
                return _strFirstName;
            }
            set
            {
                _strFirstName = value;
                OnPropertyChanged("FirstName");
            }
        }

        /// <summary>
        /// Liefert oder definiert die Strasse der Wohnadresse.
        /// </summary>
        public string Street { get; set; }

        /// <summary>
        /// Liefert oder definiert die Postleitzahl der Wohnadresse.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft unterstützt die Schnittstelle INotifyPropertyChanged.
        /// </remarks>
        public string ZIP
        {
            get
            {
                return _strZIP;
            }
            set
            {
                _strZIP = value;
                OnPropertyChanged("ZIP");
            }
        }

        /// <summary>
        /// Liefert oder definiert den Wohnort.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft unterstützt die Schnittstelle INotifyPropertyChanged.
        /// </remarks>
        public string City
        {
            get
            {
                return _strCity;
            }
            set
            {
                _strCity = value;
                OnPropertyChanged("City");
            }
        }

        /// <summary>
        /// Liefert oder definiert das Geburtsdatum der Person.
        /// </summary>
        public DateTime Birthday { get; set; }

        /// <summary>
        /// Liefert den Anzeigenamen der Person in einer Liste.
        /// </summary>
        public string DisplayName
        {
            get
            {
                return string.Format("{1} {0}, {2}, {3} {4}", Name, FirstName, Title, ZIP, City);
            }
        }
        #endregion

        /// <summary>
        /// Initialisiert ein neues Objekt der Klasse CwlbPerson.
        /// </summary>
        public CwlbPerson()
        {
            ID = ++_iNextNumber;
            Birthday = DateTime.Now - TimeSpan.FromDays(365 * 30);
            Title = "Herr";
        }

        /// <summary>
        /// Löst das Ereignis PropertyChanged aus.
        /// </summary>
        /// <param name="strPropertyName">Name der Eigenschaft, die geändert wurde.</param>
        private void OnPropertyChanged(string strPropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(strPropertyName));
            }
        }
    }
}
