﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Demo für komplexe Datenbindung in Windows Forms.
    /// </summary>
    public partial class CmdiDatabinding : Form
    {
        /// <summary>
        /// Definiert die möglichen Zustände im Dialogfeld
        /// </summary>
        private enum EwlbButtonState
        {
            Navigate,
            New
        }

        // Datenliste
        private BindingList<CwlbPerson> _cobjPerson = new BindingList<CwlbPerson>();
        // Navigationslement für die Datenbindung
        private CurrencyManager _objCurrencyManager;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiDatabinding.
        /// </summary>
        public CmdiDatabinding()
        {
            InitializeComponent();

            // Definiert die Bindung zwischen dem ListBox-Steuerelement und der Datenliste
            _lbPerson.DataSource = _cobjPerson;
            _lbPerson.DisplayMember = "DisplayName";
            // Definiert den Currency Manager für die Navigation in der Bindung
            _objCurrencyManager = _lbPerson.BindingContext[_cobjPerson] as CurrencyManager;
            // Laden der Demodaten und Bindungen zwischen Liste und Feldern aktivieren
            PrepareBindingList();
            AddDataBindingsListToFields();
            // Definieren der Schaltflächenzustände
            SetButtonState(EwlbButtonState.Navigate);
        }

        /// <summary>
        /// Definiert die Zustände der Schaltflächen auf dem Dialogfeld.
        /// </summary>
        /// <param name="enmState">Neuer Zielzustand des Dialogfeldes.</param>
        void SetButtonState(EwlbButtonState enmState)
        {
            if (enmState == EwlbButtonState.Navigate)
            {
                // Zustand Navigation.
                _btnNew.Enabled = true;
                _btnSave.Enabled = false;
                _btnCancel.Enabled = false;
                if (_lbPerson.Items.Count > 0)
                {
                    _btnDelete.Enabled = true;
                    _btnNext.Enabled = true;
                    _btnPreceding.Enabled = true;
                }
                else
                {
                    _btnDelete.Enabled = false;
                    _btnNext.Enabled = false;
                    _btnPreceding.Enabled = false;
                }
            }
            else
            {
                // Zustand Daten neue Daten einbringen
                _btnNew.Enabled = false;
                _btnSave.Enabled = true;
                _btnCancel.Enabled = true;
                _btnDelete.Enabled = false;
                _btnNext.Enabled = false;
                _btnPreceding.Enabled = false;
            }
        }

        /// <summary>
        /// Schaltet die Datenbindung zwischen dem ausgewählten Element der Liste und 
        /// den Feldern des Dialogfeldes für die Details ein.
        /// </summary>
        private void AddDataBindingsListToFields()
        {
            _tbID.DataBindings.Add("Text", _cobjPerson, "ID");
            _cbTitle.DataBindings.Add("Text", _cobjPerson, "Title");
            _tbName.DataBindings.Add("Text", _cobjPerson, "Name");
            _tbFirstName.DataBindings.Add("Text", _cobjPerson, "FirstName");
            _tbStreet.DataBindings.Add("Text", _cobjPerson, "Street");
            _tbZIP.DataBindings.Add("Text", _cobjPerson, "ZIP");
            _tbCity.DataBindings.Add("Text", _cobjPerson, "City");
            _dtpBirthday.DataBindings.Add("Value", _cobjPerson, "Birthday");
        }

        /// <summary>
        /// Schaltet die Datenbindung zwischen dem ausgewählten Element der Liste und 
        /// den Feldern des Dialogfeldes für die Details aus.
        /// </summary>
        private void RemoveDataBindingsListToFields()
        {
            _tbID.DataBindings.Clear();
            _cbTitle.DataBindings.Clear();
            _tbName.DataBindings.Clear();
            _tbFirstName.DataBindings.Clear();
            _tbStreet.DataBindings.Clear();
            _tbZIP.DataBindings.Clear();
            _tbCity.DataBindings.Clear();
            _dtpBirthday.DataBindings.Clear();
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Verändern der Auswahl.
        /// </summary>
        private void Person_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_objCurrencyManager != null && _lbPerson.SelectedIndex >= 0)
            {
                _objCurrencyManager.Position = _lbPerson.SelectedIndex;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Vordere.
        /// </summary>
        private void ButtonPreceding_Click(object sender, EventArgs e)
        {
            if (_lbPerson.SelectedIndex > 0)
            {
                _lbPerson.SelectedIndex--;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Nächste.
        /// </summary>
        private void ButtonNext_Click(object sender, EventArgs e)
        {
            if (_lbPerson.SelectedIndex < _lbPerson.Items.Count - 1)
            {
                _lbPerson.SelectedIndex++;
            }
        }


        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Neu.
        /// </summary>
        private void ButtonNew_Click(object sender, EventArgs e)
        {
            // Ein neues Objekt in der Liste erstellen
            _objCurrencyManager.AddNew();
            // Zustände der Schaltflächen kontrollieren
            SetButtonState(EwlbButtonState.New);
            // Bindung zwischen Liste und Feldern aktivieren, sofern mindestens
            // ein Element in der Liste ist.
            if (_lbPerson.Items.Count <= 1)
            {
                AddDataBindingsListToFields();
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Speichern.
        /// </summary>
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            // Eigentlich keine Aktion, da die Daten bereits synchronisiert sind
            // Zustände der Schaltflächen kontrollieren
            SetButtonState(EwlbButtonState.Navigate);
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Abbrechen.
        /// </summary>
        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            // Ausschalten der Datenbindung zwischen Liste und Feldern, wenn die 
            // Liste nach dem Löschvorgang leer wird 
            if (_lbPerson.Items.Count <= 1)
            {
                RemoveDataBindingsListToFields();
            }

            // Neu eingefügtes Element wieder löschen
            _objCurrencyManager.RemoveAt(_objCurrencyManager.Position);
            // Zustände der Schaltflächen kontrollieren
            SetButtonState(EwlbButtonState.Navigate);
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Löschen.
        /// </summary>
        private void ButtonDelete_Click(object sender, EventArgs e)
        {
            // Ausschalten der Datenbindung zwischen Liste und Feldern, wenn die 
            // Liste nach dem Löschvorgang leer wird 
            if (_lbPerson.Items.Count <= 1)
            {
                RemoveDataBindingsListToFields();
            }

            // Element löschen
            _objCurrencyManager.RemoveAt(_lbPerson.SelectedIndex);

            // Schaltflächenzustände kontrollieren
            SetButtonState(EwlbButtonState.Navigate);
        }

        /// <summary>
        /// Lädt Demodaten in die Liste.
        /// </summary>
        private void PrepareBindingList()
        {
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Herr",
                Name = "Blumiger",
                FirstName = "Peter",
                Street = "Beetweg 45",
                ZIP = "CH-3000",
                City = "Bern",
                Birthday = new DateTime(1960, 10, 01)
            });
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Frau",
                Name = "Huber",
                FirstName = "Dora",
                Street = "Rosenweg 344",
                ZIP = "CH-3000",
                City = "Bern",
                Birthday = new DateTime(1975, 11, 05)
            });
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Prof.",
                Name = "Gärtner",
                FirstName = "Judith",
                Street = "Obstweg 76",
                ZIP = "D-99192",
                City = "Apfelstädt",
                Birthday = new DateTime(1968, 01, 20)
            });
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Frau",
                Name = "Ohnestrasse",
                FirstName = "Gabi",
                ZIP = "D-80331",
                City = "München",
                Birthday = new DateTime(1975, 04, 05)
            });
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Herr",
                Name = "Fringer",
                FirstName = "Rolf",
                Street = "Horlacherweg 66",
                ZIP = "D-21382",
                City = "Breitlingen",
                Birthday = new DateTime(1970, 09, 15)
            });
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Frau",
                Name = "Ludowiga",
                FirstName = "Betina",
                Street = "Im hohlen Kreuz",
                ZIP = "CH-5078",
                City = "Schnabelingen",
                Birthday = new DateTime(1970, 09, 15)
            });
            _cobjPerson.Add(new CwlbPerson
            {
                Title = "Dr.",
                Name = "Lauber",
                FirstName = "Beat",
                Street = "Herbstweg 3a",
                ZIP = "CH-6001",
                City = "Luzern",
                Birthday = new DateTime(1960, 05, 05)
            });
        }
    }
}
