﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Demo für parallele Verarbeitung unter Windows Forms.
    /// </summary>
    public partial class CmdiParallel : Form
    {
        /// <summary>
        /// Interne Threadkontrolle
        /// </summary>
        private enum EwlbThreadState
        {
            Suspended,
            Running,
            Stopped
        }

        // Konstante Daten
        private const int _iBALL_MAX_SIZE = 25;

        // Liste der Bälle
        private readonly object _objLock = new object();
        private List<CwlbBall> _aBalls = new List<CwlbBall>();
        private Size _sizeBounds;
        private volatile EwlbThreadState _enmThreadState = EwlbThreadState.Stopped;
        private long _lCalculations;
        string _strTtitle;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiParallel
        /// </summary>
        public CmdiParallel()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Fensters.
        /// </summary>
        private void CmdiParallel_Load(object sender, EventArgs e)
        {
            // Grösse des inneren Rahmens richtig stellen
            this.CmdiParallel_SizeChanged(sender, e);
            // Titeltext festhalten
            _strTtitle = this.Text;
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Fenstergrösse.
        /// </summary>
        private void CmdiParallel_SizeChanged(object sender, EventArgs e)
        {
            // Threadsynchronisation
            lock (_objLock)
            {
                // Rechteck intern übernehmen
                _sizeBounds = new Size(_pnlShow.Size.Width, _pnlShow.Size.Height);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des Fensters.
        /// </summary>
        private void CmdiParallel_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Ist der Thread noch am laufen?
            if (_enmThreadState != EwlbThreadState.Stopped)
            {
                // Ja, Thread stoppen und Methode verlassen ohne zu beenden,
                // damit der Vordergrundthread frei wird. Timer für erneuten
                // Versuch einschalten.
                _threadWork.CancelAsync();
                e.Cancel = true;
                _timerClose.Start();
            }
        }

        /// <summary>
        /// Ereignisbehandlung Timer für das Beenden der Anwendung.
        /// </summary>
        private void TimerClose_Tick(object sender, EventArgs e)
        {
            _timerClose.Stop();
            Close();
        }

        /// <summary>
        /// Hintergrundverarbeitung für die Berechung der neuen Ballpositionen.
        /// </summary>
        private void ThreadWork_DoWork(object sender, DoWorkEventArgs e)
        {
            // Rechne die Position bis der Benutzer die Demo stoppt
            while (this._threadWork.CancellationPending != true)
            {
                // Schreibt die Anzahl Berechnungen in das Titelfeld
                WriteCalculations(_lCalculations);

                // Neue Positionen berechnen
                lock (_objLock)
                {
                    foreach (CwlbBall objBall in _aBalls)
                    {
                        // Bewege den Ball
                        objBall.Move(_sizeBounds);
                        _lCalculations++;
                    }
                }

                // Fortschritt anzeigen
                this._threadWork.ReportProgress(0);

                // Nicht zu schnell bitte
                Thread.Sleep((int)(1.0m / _numSpeed.Value * 1000));
            }
        }

        /// <summary>
        /// Schreibt die Anzahl Berechnungen in den Titelbalken.
        /// </summary>
        /// <param name="lCalculations">Anzahl Berechnungen.</param>
        private void WriteCalculations(long lCalculations)
        {
            // Prüfe den Crossthreadzugriff.
            if (InvokeRequired)
            {
                this.Invoke(new Action<long>(WriteCalculations), new object[] { lCalculations });
            }
            else
            {
                this.Text = string.Format("{0} - Anzahl Berechnungen: {1:N0}", _strTtitle, lCalculations);
            }
        }

        /// <summary>
        /// Vordergrundverarbeitung für das Zeichnen der Ballpositionen
        /// </summary>
        private void ThreadWork_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Zeichnen nachführen
            lock (_objLock)
            {
                // Benutze Doublebuffering zum Zeichnen
                using (BufferedGraphicsContext objGraphicsContext = new BufferedGraphicsContext())
                using (BufferedGraphics objGraphicsBuffer = 
                    objGraphicsContext.Allocate(_pnlShow.CreateGraphics(), _pnlShow.DisplayRectangle))
                {
                    // Farbe dem Steuerelement anpassen
                    objGraphicsBuffer.Graphics.Clear(_pnlShow.BackColor);

                    // Figuren zeichnen
                    foreach (CwlbBall objBall in _aBalls)
                    {
                        objBall.Draw(objGraphicsBuffer.Graphics);
                    }

                    // Puffer freigeben
                    objGraphicsBuffer.Render(_pnlShow.CreateGraphics());
                }
            }
        }

        /// <summary>
        /// Vordergrundverarbeitung nach dem Beenden der Hintergrundverarbeitung
        /// </summary>
        private void ThreadWork_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Fehlerprüfung
            if (e.Error != null)
            {
                // Anzeige des Fehlers
                MessageBox.Show(e.Error.Message, "Fehler bei der Ausführung des Arbeitsthreads");
            }

            // Bälle zerstören
            foreach (CwlbBall objBall in _aBalls)
            {
                objBall.Dispose();
            }
            _aBalls.Clear();

            // Elemente wieder einschalten
            _btnStart.Enabled = true;
            _btnStop.Enabled = false;
            _numBalls.Enabled = true;
            _enmThreadState = EwlbThreadState.Stopped;
            _pnlShow.Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung beim Zeichnen des Fensters.
        /// </summary>
        /// <remarks>
        /// Diese Methode wird nur beim Ändern der Fenstergrösse berwendet.
        /// </remarks>
        private void PanelShow_Paint(object sender, PaintEventArgs e)
        {
            // Zeichnen der Bälle
            lock (_objLock)
            {
                foreach (CwlbBall objBall in _aBalls)
                {
                    objBall.Draw(e.Graphics);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung der Betätigung der Schaltfläche Start.
        /// </summary>
        private void ButtonStart_Click(object sender, EventArgs e)
        {
            // Ausschalten der Bedienbarkeit der Anzahl Bälle
            _numBalls.Enabled = false;
            _btnStart.Enabled = false;
            _btnStop.Enabled = true;
            _lCalculations = 0;

            // Erzeugen der Bälle (ohne Threadsynchronisation
            // weil der Arbeitsthread erst anschliessend gestartet wird
            for (int iCount = 0; iCount < _numBalls.Value; iCount++)
            {
                _aBalls.Add(new CwlbBall(_sizeBounds, _iBALL_MAX_SIZE));
            }

            // Starten des Threads
            _threadWork.RunWorkerAsync((int)_numSpeed.Value);
            _enmThreadState = EwlbThreadState.Running;
        }

        /// <summary>
        /// Ereignisbehandlung der Betätigung der Schaltfläche Stop.
        /// </summary>
        private void ButtonStop_Click(object sender, EventArgs e)
        {
            // Stoppen des Arbeitsthreads
            _threadWork.CancelAsync();
        }
    }
}
