﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.Windows.Threading;
using System.Collections.ObjectModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Datenliste für die Handhabung der Ereignisse.
        /// </summary>
        public ObservableCollection<CwlbEventData> Data { get; private set; }

        /// <summary>
        /// Log window für Ereignisse.
        /// </summary>
        private CdlgEventLog _dlgLog;

        /// <summary>
        /// Demodialog für demo Ereignisse
        /// </summary>
        private CdlgEventDemo _dlgDemo;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse MainWindow
        /// </summary>
        public MainWindow()
        {
            Data = new ObservableCollection<CwlbEventData>();
            InitializeComponent();

            // Titel der Anwendung definieren
            Title = "Learning WPF 02";
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Ausnahme auslösen
        /// </summary>
        private void ButtonException_Click(object sender, RoutedEventArgs e)
        {
            throw new InvalidOperationException("Ausnahme im Ereignishandler des Hauptfensters.");
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Ausnahme in Hintergrundthread auslösen.
        /// </summary>
        private void ButtonThreadException_Click(object sender, RoutedEventArgs e)
        {
            // Thread starten
            ThreadPool.QueueUserWorkItem(BackgroundThread);
        }

        /// <summary>
        /// Hintergrundthread für das Auslösen einer Ausnahme.
        /// </summary>
        private void BackgroundThread(object objState)
        {
            try
            {
                // Auslösung verzögern
                Thread.Sleep(2000);
                throw new InvalidOperationException("Ausnahme im Hintergrundthread.");
            }
            catch(Exception excObject)
            {
                // Ausnahme abgefangen. Diese nun dem Dispatcher übergeben
                EventHandler<CwlbUnhandledException> handler = HandleException;
                Dispatcher.BeginInvoke(handler, new object[] { "Thread", new CwlbUnhandledException(excObject)});
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Routing einer Ausnahme in die zentrale Ausnahmebehandlung.
        /// </summary>
        private void HandleException(object sender, CwlbUnhandledException excObject)
        {
            throw new InvalidOperationException(excObject.Exception.Message, excObject.Exception);
        }

        /// <summary>
        /// Ereignisbehandlung für die Betätigung der Auswahl-Schaltfläche.
        /// </summary>
        private void ButtonSelection_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show("Ihre Auswahl war: " + ((Button)e.OriginalSource).CommandParameter.ToString());
        }

        /// <summary>
        /// Ereignisbehandlung für das Anzeigen der Demofenster.
        /// </summary>
        private void DemoEvent_Click(object sender, RoutedEventArgs e)
        {
            // Log Window erzeugen
            if (_dlgLog == null)
            {
                _dlgLog = new CdlgEventLog();
                _dlgLog.Owner = this;
                _dlgLog.Closing += new System.ComponentModel.CancelEventHandler(LogDialog_Closing);

                // Quelle für den Datenbezug definierten
                CollectionViewSource objViewSource = (CollectionViewSource)_dlgLog.FindResource("ViewSource");
                objViewSource.Source = Data;
                _dlgLog.MainWindow = this;
            }

            // Fenster anzeigen
            _dlgLog.Show();

            // Dialog objekt erzeugen
            if (_dlgDemo == null)
            {
                // Log löschen
                Data.Clear();
                // Fenster erstellen
                _dlgDemo = new CdlgEventDemo();
                _dlgDemo.Owner = this;
                _dlgDemo.Closing += new System.ComponentModel.CancelEventHandler(DemoDialog_Closing);
                // Ereignisbehandlungen initialisieren
                InitAllHandler(_dlgDemo, _dlgDemo);
            }

            // Dailog anzeigen
            _dlgDemo.Show();
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des Demofensters.
        /// </summary>
        void DemoDialog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _dlgDemo.Closing -= new System.ComponentModel.CancelEventHandler(DemoDialog_Closing);
            _dlgDemo = null;
            Focus();
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des Logfensters.
        /// </summary>
        void LogDialog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _dlgLog.Closing -= new System.ComponentModel.CancelEventHandler(LogDialog_Closing);
            _dlgLog = null;
            Focus();
        }

        /// <summary>
        /// Registrieren der Ereignisbehandlungen für das Demofenster. Diese Methode arbeitet sich rekursiv durch 
        /// den logischen Baum und registriert die enthaltenen Elemente.
        /// </summary>
        /// <param name="objElement">Element das registriert werden soll.</param>
        /// <param name="dlgDemo">Objekt das die Ereignisbehandlung zur Verfügung stellt.</param>
        private void InitAllHandler(FrameworkElement objElement, CdlgEventDemo dlgDemo)
        {
            objElement.Loaded += new RoutedEventHandler(dlgDemo.GeneralHandler);
            objElement.Unloaded += new RoutedEventHandler(dlgDemo.GeneralHandler);

            objElement.PreviewKeyDown += new KeyEventHandler(dlgDemo.GeneralHandler);
            objElement.KeyDown += new KeyEventHandler(dlgDemo.GeneralHandler);

            objElement.PreviewKeyUp += new KeyEventHandler(dlgDemo.GeneralHandler);
            objElement.KeyUp += new KeyEventHandler(dlgDemo.GeneralHandler);

            objElement.GotFocus += new RoutedEventHandler(dlgDemo.GeneralHandler);

            objElement.GotKeyboardFocus += new KeyboardFocusChangedEventHandler(dlgDemo.GeneralHandler);

            objElement.LostFocus += new RoutedEventHandler(dlgDemo.GeneralHandler);

            objElement.PreviewLostKeyboardFocus += new KeyboardFocusChangedEventHandler(dlgDemo.GeneralHandler);
            objElement.LostKeyboardFocus += new KeyboardFocusChangedEventHandler(dlgDemo.GeneralHandler);
            
            objElement.PreviewMouseDown += new MouseButtonEventHandler(dlgDemo.GeneralHandler);
            objElement.MouseDown += new MouseButtonEventHandler(dlgDemo.GeneralHandler);

            objElement.PreviewMouseUp += new MouseButtonEventHandler(dlgDemo.GeneralHandler);
            objElement.MouseUp += new MouseButtonEventHandler(dlgDemo.GeneralHandler);

            objElement.MouseEnter += new MouseEventHandler(dlgDemo.GeneralHandler);
            objElement.MouseLeave += new MouseEventHandler(dlgDemo.GeneralHandler);

            objElement.PreviewMouseMove += new MouseEventHandler(dlgDemo.GeneralHandler);
            objElement.MouseMove += new MouseEventHandler(dlgDemo.GeneralHandler);

            objElement.TextInput += new TextCompositionEventHandler((sender, objArgs) => dlgDemo.GeneralHandler(sender, objArgs));
            objElement.PreviewTextInput += new TextCompositionEventHandler((sender, objArgs) => dlgDemo.GeneralHandler(sender, objArgs));

            objElement.AddHandler(Button.ClickEvent, new RoutedEventHandler(dlgDemo.GeneralHandler));

            // Duchlaufen der enthaltenen Elemente
            foreach (object objItem in LogicalTreeHelper.GetChildren(objElement))
            {
                if (objItem is FrameworkElement)
                {
                    // Rekursiver Aufruf sofern es sich um ein FrameworkElement handelt.
                    InitAllHandler(objItem as FrameworkElement, dlgDemo);
                }
            }
        }

        private void DemoDependencyProperty_Click(object sender, RoutedEventArgs e)
        {
            CDlgDemoDependencyProperty dlgDemo = new CDlgDemoDependencyProperty();
            dlgDemo.Owner = this;
            dlgDemo.ShowDialog();
        }

        private void TEST(object sender, ContextMenuEventArgs e)
        {

        }
    }
}
