﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.ComponentModel;
using System.Diagnostics;
using Microsoft.Win32;
using System.Xml;
using System.Windows.Markup;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für das Fenster CwlbMainWindow.xaml
    /// </summary>
    public partial class CwlbMainWindow : Window, INotifyPropertyChanged
    {
        // Instanzdaten
        private DispatcherTimer _timerMain;
        private DateTime _dtApplicationTime;
        private string _strFileName;
        private bool _bModified;
        private string _strTitle;

        /// <summary>
        /// Wird ausgelöst, wenn eine Eigenschaft des Hauptfensters ändert.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Definition des eigenen Kommandos für das Beenden der Anwendung.
        /// </summary>
        public static RoutedUICommand Exit = new RoutedUICommand(
            Properties.Resources.MenuTextExit,  // Sichtbarer Text in der Benutzeroberfläche
            "cmdExit",                          // Name des Kommandoobjekts selber
            typeof(CwlbMainWindow),             // Typ der das Kommando registriert
            new InputGestureCollection          // Liste der unterstützten Gesten
                {
                    new KeyGesture(Key.X, ModifierKeys.Control, "Ctrl+X") 
                });

        /// <summary>
        /// Definition des eigenen Kommandos für das Beenden der Anwendung.
        /// </summary>
        public static RoutedUICommand Extras = new RoutedUICommand(
            Properties.Resources.MenuTextExtras, // Sichtbarer Text in der Benutzeroberfläche
            "cmdExtras",                         // Name des Kommandoobjekts selber
            typeof(CwlbMainWindow));             // Typ der das Kommando registriert

        /// <summary>
        /// Stellt ein Element für die Bindung der Anwendungszeit zur Verfügung
        /// </summary>
        public DateTime ApplicationTime
        {
            get { return _dtApplicationTime; }
            set
            {
                _dtApplicationTime = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("ApplicationTime"));
                }
            }
        }

        public string WindowTitle
        {
            get
            {
                return _strTitle;
            }
        }

        private void SetFileName(string strFileName)
        {
            _strFileName = strFileName;
            SetTitle();
        }

        private void SetTitle()
        {
            if (string.IsNullOrEmpty(_strFileName))
            {
                _strTitle = Properties.Resources.TextWindowTitle;
            }
            else
            {
                _strTitle = string.Format("{0} - {1}{2}",
                    Properties.Resources.TextWindowTitle,
                    _strFileName,
                    _bModified ? "*" : string.Empty);
            }
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs("WindowTitle"));
            }
        }

        private void SetModified(bool bModified)
        {
            _bModified = bModified;
            SetTitle();
        }

        /// <summary>
        /// initialisiert eine neue Instanz der Klasse CwlbMainWindow.
        /// </summary>
        public CwlbMainWindow()
        {
            SetTitle();
            InitializeComponent();

            // Erstellen des Timers
            _timerMain = new DispatcherTimer();
            _timerMain.Interval = TimeSpan.FromMilliseconds(400);
            _timerMain.Tick += new EventHandler(TimerMain_Tick);
            _timerMain.Start();

            // Get 
            _rtbEdit.TextChanged += new TextChangedEventHandler(TextBox_TextChanged);
        }

        void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            SetModified(true);
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Laden des Fensters.
        /// </summary>
        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            // Erstellen der Bindung des Standardkommandos OPen
            CommandBinding bindingCommand = new CommandBinding(ApplicationCommands.Open);
            bindingCommand.CanExecute += CanExecuteAlways;
            bindingCommand.Executed += ExecuteOpen;
            this.CommandBindings.Add(bindingCommand);

            // Erstellen des Menübefehls
            MenuItem mnuOpen = new MenuItem();
            mnuOpen.Command = ApplicationCommands.Open;

            // Menubefehl in die bestehenden Befehle einbringen
            _mnuFile.Items.Insert(1, mnuOpen);
        }

        /// <summary>
        /// Ereignisbehandlung für ein Intervall des Timers.
        /// </summary>
        private void TimerMain_Tick(object sender, EventArgs e)
        {
            ApplicationTime = DateTime.Now;
        }

        /// <summary>
        /// Ereignisbehandlung für die Anfrage ob ein Kommando ausgeführt werden kann.
        /// </summary>
        private void CanExecuteAlways(object sender, CanExecuteRoutedEventArgs e)
        {
            Trace.WriteLine(((RoutedUICommand)e.Command).Text);
            e.CanExecute = true;
        }

        /// <summary>
        /// Ereignisbehandlung für die Anfrage ob das Kommando Speichern ausgeführt werden kann.
        /// </summary>
        private void CanExecuteSave(object sender, CanExecuteRoutedEventArgs e)
        {
            if (_bModified)
            {
                e.CanExecute = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Ausführung des Befehls Datei öffnen.
        /// </summary>
        private void ExecuteOpen(object sender, ExecutedRoutedEventArgs e)
        {
            // Dialogfeld erzeugen
            OpenFileDialog dlgOpen = new OpenFileDialog();

            // Dialogfeld initialisieren
            dlgOpen.Filter = "Rich Text Format|*.rtf|" +
                             "Extensible Markup Application Language|*.xaml|" +
                             "Extensible Markup Language|*.xml|" +
                             "Alle Dateien|*.*";
            dlgOpen.FilterIndex = 1;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgOpen.ShowDialog(this).GetValueOrDefault())
            {
                // Daten von Dialogfeld übernehmen und verarbeiten
                try
                {
                    using (Stream streamFile = File.Open(dlgOpen.FileName, FileMode.Open))
                    {

                        _rtbEdit.Document = new FlowDocument();
                        TextRange rangeDocument = new TextRange(
                            _rtbEdit.Document.ContentStart, _rtbEdit.Document.ContentEnd);
                        switch (System.IO.Path.GetExtension(dlgOpen.FileName).ToLower())
                        {
                            case ".rtf":
                                rangeDocument.Load(streamFile, DataFormats.Rtf);
                                break;
                            case ".xaml":
                            case ".xml":
                                rangeDocument.Load(streamFile, DataFormats.XamlPackage);
                                break;
                            default:
                                MessageBox.Show(
                                    "Ungültiges Datenformat beim Laden entdeckt. Es wird nur RTF und XAML unterstützt",
                                    "Speichern",
                                    MessageBoxButton.OK, MessageBoxImage.Error);
                                break;
                        }
                        SetModified(false);
                        SetFileName(dlgOpen.FileName);
                    }
                }
                catch (Exception excObject)
                {
                    MessageBox.Show(excObject.Message, "LearningWPF 03");
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Neuerstellen eines Dokuments.
        /// </summary>
        private void ExecuteNew(object sender, ExecutedRoutedEventArgs e)
        {
            if (_bModified)
            {
                MessageBoxResult enmResult = MessageBox.Show(
                    Properties.Resources.MsgNewQuestion,
                    Properties.Resources.TextWindowTitle,
                    MessageBoxButton.YesNo,
                    MessageBoxImage.Question);
                if (enmResult == MessageBoxResult.Yes)
                {
                    ExecuteSave(sender, e);
                }
            }

            // Anwendung beenden
            _rtbEdit.Document = new FlowDocument();
            SetModified(false);
            SetFileName(null);
        }

        /// <summary>
        /// Ereignisbehandlung beim Speichern der Datei.
        /// </summary>
        private void ExecuteSave(object sender, ExecutedRoutedEventArgs e)
        {
            // Prüfen, ob bereits ein Dateinamen bekannt ist
            if (string.IsNullOrEmpty(_strFileName))
            {
                // Dateiname ist noch nicht bekannt. Abfragen desselben starten.
                _strFileName = SaveAs();
            }

            // Prüfen, ob ein Dateinamen beaknnt ist. Wenn nicht abbrechen
            if (string.IsNullOrEmpty(_strFileName))
            {
                return;
            }

            // Filestrom mit dem definierten Dateinamen herstellen
            using (Stream streamFile = File.Create(_strFileName))
            {
                // Textrange von Anfang bis Ende des Dokuments herstellen
                TextRange rangeDocument = new TextRange(_rtbEdit.Document.ContentStart, _rtbEdit.Document.ContentEnd);
                switch (System.IO.Path.GetExtension(_strFileName).ToLower())
                {
                    case ".rtf":
                        rangeDocument.Save(streamFile, DataFormats.Rtf);
                        break;
                    case ".xaml":
                    case ".xml":
                        rangeDocument.Save(streamFile, DataFormats.XamlPackage);
                        break;
                    default:
                        MessageBox.Show(
                            "Ungültiges Datenformat beim Speichern entdeckt. Es wird nur RTF und XAML unterstützt", 
                            "Speichern", 
                            MessageBoxButton.OK, MessageBoxImage.Error);
                        break;
                }

                // Zurücksetzen der Markierung für Veränderung
                SetModified(false);
                SetFileName(_strFileName);
            }
        }

        /// <summary>
        /// Zeigt das Dialogfeld zum Speichern von Dateien an und liefert den ausgewählten Datenamen.
        /// </summary>
        /// <returns>Liefert den Dateinamen zur Speicherung der Daten oder null, wenn nicht gespeichert werden soll.</returns>
        private string SaveAs()
        {
            // Dialogfeld instanziieren
            SaveFileDialog dlgSave = new SaveFileDialog();
            // Initialisieren des Dialogs
            dlgSave.Filter = "Rich Text Format|*.rtf|" + 
                             "Extensible Markup Application Language|*.xaml|" +
                             "Extensible Markup Language|*.xml|" +
                             "Alle Dateien|*.*";
            dlgSave.FilterIndex = 1;

            // Dailogfeld anzeigen und Resultat abholen
            if (dlgSave.ShowDialog(this).GetValueOrDefault())
            {
                // Rückgabe des Dateinamens
                return dlgSave.FileName;
            }
            else
            {
                // Rückgabe von null, weil der Dailog abgebrochen wurde
                return null;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Anzeigen des Optionsdialogfelds.
        /// </summary>
        private void ExecuteExtras(object sender, ExecutedRoutedEventArgs e)
        {
            // Dialogfeld definieren
            CdlgOptions dlgOptions = new CdlgOptions();

            // Dialogfeld konfigurieren
            dlgOptions.Owner = this;

            // Dialogfeld anzeigen und Resultate abholen
            if (dlgOptions.ShowDialog().GetValueOrDefault())
            {
                // Der Dialog wurde mit OK beendet
                MessageBox.Show("OK");
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Beenden der Anwendung.
        /// </summary>
        private void ExecuteExit(object sender, ExecutedRoutedEventArgs e)
        {
            if (_bModified)
            {
                MessageBoxResult enmResult = MessageBox.Show(
                    Properties.Resources.MsgExitQuestion,
                    Properties.Resources.TextWindowTitle,
                    MessageBoxButton.YesNoCancel,
                    MessageBoxImage.Question);
                if (enmResult != MessageBoxResult.Yes)
                {
                    return;
                }
            }

            // Anwendung beenden
            this.Close();
        }

        /// <summary>
        /// Ereignisbehandlung für das Drucken.
        /// </summary>
        private void ExecutePrint(object sender, ExecutedRoutedEventArgs e)
        {
            PrintDialog pd = new PrintDialog();
            if ((pd.ShowDialog() == true))
            {
                //use either one of the below      
                pd.PrintVisual(_rtbEdit as Visual, "printing as visual");
                pd.PrintDocument((((IDocumentPaginatorSource)_rtbEdit.Document).DocumentPaginator), "printing as paginator");
            }
        }
    }
}
