﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;
using System.Collections.ObjectModel;
using System.Reflection;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert Hilfsmethoden für den Aufbau von Detailinformationen des betrachteten Details.
    /// </summary>
    public static class CwlbTreeHelper
    {
        /// <summary>
        /// Erstellt den logischen Baum eines Steuerelements.
        /// </summary>
        /// <param name="objElement">Steuerelement, dessen logischer Baum erstellt werdens oll.</param>
        /// <param name="objParentNode">Knoten in dem das Detail eingetragen werden soll</param>
        public static void BuildLocicalTree(Visual objElement, ItemCollection objParentNode)
        {
            // Aktuelle Informationen löschen
            objParentNode.Clear();
            // Aufruf der rekrsiv arbeiteten Aufbaumethode
            BuildLocicalTreeInternal(objElement, objParentNode);
        }

        /// <summary>
        /// Baut den Baum rekursiv auf.
        /// </summary>
        /// <param name="objElement">Element das in den Baum eingetragen werden soll.</param>
        /// <param name="objParentNode">Elternknoten, in den das Element eingetragen wird.</param>
        private static void BuildLocicalTreeInternal(Visual objElement, ItemCollection objParentNode)
        {
            // Lokale Daten
            TreeViewItem tvItem;

            // Analyse des logischen Baums
            foreach (object objItem in LogicalTreeHelper.GetChildren(objElement))
            {
                // Erstelle einen neuen Knoten
                tvItem = new TreeViewItem();
                tvItem.Header = objItem.GetType().Name;
                tvItem.Tag = objItem;

                // Füge den Knoten in die Liste der Knoten des Elternelements hinzu
                objParentNode.Add(tvItem);

                // Rekursiver Aufruf
                if (objItem is Visual)
                {
                    BuildLocicalTree(objItem as Visual, tvItem.Items);
                }
            }
        }

        public static void BuildVisualTree(Visual objElement, ItemCollection objParentNode)
        {
            objParentNode.Clear();
            BuildVisualTreeInternal(objElement, objParentNode);
        }

        private static void BuildVisualTreeInternal(Visual objElement, ItemCollection objParentNode)
        {
            // Locals
            TreeViewItem tvItem;
            Visual objVisual;

            // Analyse the visual tree.
            for (int iCount = 0; iCount < VisualTreeHelper.GetChildrenCount(objElement); iCount++)
            {
                // Retrieve child visual at specified index value.
                objVisual = (Visual)VisualTreeHelper.GetChild(objElement, iCount);

                // Create a new node.
                tvItem = new TreeViewItem();
                tvItem.Header = objVisual.GetType().Name;
                tvItem.Tag = objVisual;

                // Add node into the collection.
                objParentNode.Add(tvItem);

                // Call myself recursively, only if it is a dependency Object.
                BuildVisualTree(objVisual, tvItem.Items);
            }
        }

        public static void BuildClassTree(object objItem, ObservableCollection<CwlbClass> objClassHierarchy, ObservableCollection<CwlbClass> cobjClassList)
        {
            objClassHierarchy.Clear();
            cobjClassList.Clear();
            CwlbClass objLast = BuildClassTreeInternal(objItem.GetType(), objClassHierarchy, cobjClassList);

            // Definiere den letzten Knoten als ausgeschaltet und ausgewählt
            objLast.Checked = true;
            objLast.Enabled = false;
        }

        private static CwlbClass BuildClassTreeInternal(Type objType, ObservableCollection<CwlbClass> objClassHierarchy, ObservableCollection<CwlbClass> cobjClassList)
        {
            // Lokale Daten
            CwlbClass objParentClass = null;
            CwlbClass objClass;

            // Zuerst rekursiv zur Basis steigen
            if (objType != typeof(object))
            {
                objParentClass = BuildClassTreeInternal(objType.BaseType, objClassHierarchy, cobjClassList);
            }

            // Knoten erstellen
            objClass = new CwlbClass
            {
                Type = objType,
                ToolTip = string.Format("{0}{1}{2}", objType, Environment.NewLine, objType.AssemblyQualifiedName)
            };

            // Knoten hinzufügen
            if (objParentClass != null)
            {
                // Wurzelknoten
                objParentClass.Child.Add(objClass);
            }
            else
            {
                objClassHierarchy.Add(objClass);
            }

            // Zu flacher Liste hinzufügen
            cobjClassList.Add(objClass);

            // Aktuelle Nodeliste zurückgeben
            return objClass;
        }

        internal static void BuildInterfaceList(object objItem, ObservableCollection<CwlbInterface> cobjInterfaces, ObservableCollection<CwlbClass> objClassHierarchy)
        {
            // Lokale Daten
            CwlbInterface objInterface;
            // Vorbereiten der Daten
            cobjInterfaces.Clear();

            // Alle Schnittstelleninformationen lesen
            foreach (Type objType in objItem.GetType().GetInterfaces().OrderBy(objOrder => objOrder.Name))
            {
                objInterface = new CwlbInterface();
                objInterface.Type = objType;
                objInterface.ToolTip = string.Format("{0}{1}{2}", objType, Environment.NewLine, objType.AssemblyQualifiedName);
                cobjInterfaces.Add(objInterface);
            }
        }

        internal static void BuildMethodList(object objItem, ObservableCollection<CwlbMethod> cobjMethods, ObservableCollection<CwlbClass> cobjClassList)
        {
            // Lokale Daten
            CwlbMethod objMethod;
            CwlbClass objClass;
            // Vorbereiten der Daten
            cobjMethods.Clear();

            // Alle Methodeninformationen lesen
            foreach (MethodInfo objMethodInfo in objItem.GetType()
                .GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)
                .Where(objSearch => !objSearch.IsSpecialName)
                .OrderBy(objOrder => objOrder.Name))
            {
                // Prüfe ob der Knoten zu dem die Methode gehört selektiert ist
                objClass = cobjClassList.FirstOrDefault(objSearch => objSearch.TypeName == objMethodInfo.DeclaringType.Name);
                if (objClass == null || !objClass.Checked)
                {
                    continue;
                }

                objMethod = new CwlbMethod();
                objMethod.Name = string.Format("{0}()", objMethodInfo.Name);
                objMethod.ToolTip = CreateMethodSignature(objMethodInfo);
                // Knoten in Liste hinzufügen
                cobjMethods.Add(objMethod);
            }
        }


        /// <summary>
        /// Erstellt eine Methodensignatur.
        /// </summary>
        /// <param name="objMethod"></param>
        /// <returns></returns>
        private static string CreateMethodSignature(MethodInfo objMethod)
        {
            // Locals
            bool bHasParams = false;

            // StringBuilder zurücksetzen
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("public");
            strBuilder.Append(objMethod.IsStatic ? " static" : "");
            strBuilder.AppendFormat(" {0}", objMethod.ReturnType.Name);
            strBuilder.AppendFormat(" {0}(", objMethod.Name);
            foreach (ParameterInfo objInfo in objMethod.GetParameters())
            {
                if (bHasParams)
                {
                    strBuilder.Append(",");
                }
                else
                {
                    bHasParams = true;
                }
                strBuilder.Append(Environment.NewLine);
                strBuilder.AppendFormat("     {0} {1}",
                    objInfo.ParameterType.Name,
                    objInfo.Name
                    );
            }

            // Abschluss prüfen
            if (bHasParams)
            {
                strBuilder.Append(Environment.NewLine);
            }
            strBuilder.AppendFormat(")", objMethod.Name);
            return strBuilder.ToString();
        }

        /// <summary>
        /// Zeigt die Ereignisinformationen auf der Ereignisseite an.
        /// </summary>
        internal static void BuildEventList(object objItem, ObservableCollection<CwlbEvent> cobjEvents, ObservableCollection<CwlbClass> cobjClassList)
        {
            // Lokale Daten
            CwlbEvent objEvent;
            CwlbClass objClass;
            // Vorbereiten der Daten
            cobjEvents.Clear();

            // Alle Methodeninformationen lesen
            foreach (EventInfo objEventInfo in objItem.GetType()
                .GetEvents(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static)
                .OrderBy(objOrder => objOrder.Name))
            {
                // Prüfe ob der Knoten zu dem die Methode gehört selektiert ist
                objClass = cobjClassList.FirstOrDefault(objSearch => objSearch.TypeName == objEventInfo.DeclaringType.Name);
                if (objClass == null || !objClass.Checked)
                {
                    continue;
                }

                objEvent = new CwlbEvent();
                objEvent.Name = string.Format("{0}", objEventInfo.Name);
                // Knoten in Liste hinzufügen
                cobjEvents.Add(objEvent);
            }
        }
    }
}
