﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml.Linq;
using System.Xml;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für das Hauptfenster der Anwendung.
    /// </summary>
    /// <remarks>
    /// Diese Klasse implementiert die Schnittstelle INotifyPropertyChanged, damit Bindungen
    /// aus den XAML-Code durchgeführt werden können.
    /// </remarks>
    public partial class CwlbMainWindow : Window, INotifyPropertyChanged
    {
        private const string DEMO = "Demo";
        private const string PANEL = "Panel";
        private const string PATH = "Path";
        private const string CONTENT = "Content";
        private const string CONTENT_PATH = "ContentPath";

        // Private Daten
        private Dictionary<string, Control> _cobjDemos = new Dictionary<string, Control>();
        private object _objCurrentDetail;
        private int _iPropertyCount;

        /// <summary>
        /// Ereignis für die Benachrichtigung von Elementänderungen.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Hierarchie der verwendeten Klassen des betrachteten Steuerelements.
        /// </summary>
        public ObservableCollection<CwlbClass> ClassHierarchy { get; private set; }

        /// <summary>
        /// Liste der verwendeten Klassen des betrachteten Steuerelements.
        /// </summary>
        public ObservableCollection<CwlbClass> ClassList { get; private set; }

        /// <summary>
        /// Liste der Schnittstellen des betrachteten Steuerelements.
        /// </summary>
        public ObservableCollection<CwlbInterface> Interfaces { get; private set; }

        /// <summary>
        /// Liste der Methoden des betrachteten Steuerelements.
        /// </summary>
        public ObservableCollection<CwlbMethod> Methods { get; private set; }

        /// <summary>
        /// Liste der Ereignisse des betrachteten Steuerelements.
        /// </summary>
        public ObservableCollection<CwlbEvent> Events { get; private set; }

        /// <summary>
        /// Liefert oder definiert die Anzahl der EIgenschaften des betrachteten Steuerelements.
        /// </summary>
        public int PropertyCount
        {
            get
            {
                return _iPropertyCount;
            }
            set
            {
                // Wert übernehmen
                _iPropertyCount = value;
                // Auslösen der Benachrtichtigung der Veränderung.
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("PropertyCount"));
                }
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbMainWindow
        /// </summary>
        public CwlbMainWindow()
        {
            // Erstellen der internen Elemente bevor das Fenster erzeugt wird. 
            // Dmit wird sichergestellt, dass die Bindung einwandfrei funktioniert.
            ClassList = new ObservableCollection<CwlbClass>();
            ClassHierarchy = new ObservableCollection<CwlbClass>();
            Interfaces = new ObservableCollection<CwlbInterface>();
            Methods = new ObservableCollection<CwlbMethod>();
            Events = new ObservableCollection<CwlbEvent>();

            // Generierter Code für die Initialisierung
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für die Initialisierung der Elemente
        /// </summary>
        private void Window_Initialized(object sender, EventArgs e)
        {
            // Fensterposition und Größe an die Daten der Properties binden
            // Siehe auch Ereignisbehandlung Window_Closing.
            this.BindProperty(LeftProperty, Properties.Settings.Default, "MainWindowLeft", BindingMode.TwoWay);
            this.BindProperty(TopProperty, Properties.Settings.Default, "MainWindowTop", BindingMode.TwoWay);
            this.BindProperty(WidthProperty, Properties.Settings.Default, "MainWindowWidth", BindingMode.TwoWay);

            // Erstelle eine Bindung als Variante explizit über die Klasse Binding
            Binding objBinding = new Binding();
            objBinding.Mode = BindingMode.TwoWay;
            objBinding.NotifyOnSourceUpdated = true;
            objBinding.Source = Properties.Settings.Default;
            objBinding.Path =  new PropertyPath("MainWindowHeight");
            this.SetBinding(HeightProperty, objBinding);
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des Fensters.
        /// </summary>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // Speichern der Einstellungen (unter anderem sind darin auch die 
            // Fensterposition und Größe)
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Ereignisbehandlung bei der Auswahl eines Elements im Baum der Steuerung der Demo.
        /// </summary>
        private void TreeViewItem_DoubleClick(object sender, MouseButtonEventArgs e)
        {
            // Lokale Daten
            string strPanelName;
            string strPath;
            string strContent = null;
            string strContentPath = null;
            string strPanel;

            // Ausgewähltes Element konvertieren
            XmlElement objDemo = ((TreeView)sender).SelectedItem as XmlElement;

            // Prüfen, ob das Element benutzt werden kann
            if (objDemo != null && objDemo.Name == DEMO)
            {
                // Abfragen der notwendigen Details
                strPanelName = objDemo.Attributes[PANEL].Value;
                strPath = objDemo.ParentNode.Attributes[PATH].Value;
                if (objDemo.Attributes[PATH] != null && !string.IsNullOrEmpty(objDemo.Attributes[PATH].Value))
                {
                    strPath = objDemo.Attributes[PATH].Value;
                }
                if (objDemo.Attributes[CONTENT] != null && !string.IsNullOrEmpty(objDemo.Attributes[CONTENT].Value))
                {
                    strContent = objDemo.Attributes[CONTENT].Value;
                }
                if (objDemo.Attributes[CONTENT_PATH] != null && !string.IsNullOrEmpty(objDemo.Attributes[CONTENT_PATH].Value))
                {
                    strContentPath = objDemo.Attributes[CONTENT_PATH].Value;
                }
                strPanel = string.Format("{0}/{1}.xaml", strPath, strPanelName);

                // Prüfen, ob das Element bereits einmal verwendet wurde
                if (!_cobjDemos.ContainsKey(strPanel))
                {
                    // Neues Element erstellen und in den Cache aufnehmen
                    Control objControl = (Control)Application.LoadComponent(new Uri(strPanel, System.UriKind.Relative));

                    // Thema adaptieren
                    //Uri uriTheme = new Uri("PresentationFramework.Classic;V3.0.0.0;31bf3856ad364e35;component\\themes/classic.xaml", UriKind.Relative);
                    //objControl.Resources = Application.LoadComponent(uriTheme) as ResourceDictionary;

                    _cobjDemos.Add(strPanel, objControl);
                }

                // Prüfen, ob das letzte Element verschieden vom aktuellen Element ist
                if (!ReferenceEquals(_vwrDemo.Child, _cobjDemos[strPanel]))
                {
                    // Prüfen, ob eine deregistrierung vorgenommen werden muss
                    if (_vwrDemo.Child != null)
                    {
                        // Ereignisbehandlung für Benachrichtigung der Details registrieren
                        if (_vwrDemo.Child is IwlbDynamicXaml)
                        {
                            ((IwlbDynamicXaml)_vwrDemo.Child).Compiled -= new EventHandler<CwlbCompiledEventArgs>(CwlbMainWindow_Compiled);
                        }
                        else
                        {
                            ((UserControl)_vwrDemo.Child).Loaded -= new RoutedEventHandler(DemoDetail_Loaded);
                        }
                    }

                    // Registration des neuen Elements vornehmen
                    // Ereignisbehandlung für Benachrichtigung der Details registrieren
                    if (_cobjDemos[strPanel] is IwlbDynamicXaml)
                    {
                        ((IwlbDynamicXaml)_cobjDemos[strPanel]).Compiled += new EventHandler<CwlbCompiledEventArgs>(CwlbMainWindow_Compiled);
                    }
                    else
                    {
                        _cobjDemos[strPanel].Loaded += new RoutedEventHandler(DemoDetail_Loaded);
                    }

                    // Neues Element dem vorgesetzten Element anfügen
                    _vwrDemo.Child = _cobjDemos[strPanel];
                }

                // Prüfen, ob Inhalt in das enthaltene Element gegeben werden muss
                if (_cobjDemos[strPanel] is IwlbDynamicXaml && !string.IsNullOrEmpty(strContent))
                {
                    ((IwlbDynamicXaml)_cobjDemos[strPanel]).LoadFile(string.Format("{0}\\{1}", strContentPath, strContent));
                    ((IwlbDynamicXaml)_cobjDemos[strPanel]).Focus();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung bei Kompilation im XAML-Pad.
        /// </summary>
        private void CwlbMainWindow_Compiled(object sender, CwlbCompiledEventArgs e)
        {
            // Build the tree
            CwlbTreeHelper.BuildLocicalTree(e.Root, _tvLogic.Items);
            CwlbTreeHelper.BuildVisualTree(e.Root, _tvVisual.Items);
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Demodetails.
        /// </summary>
        private void DemoDetail_Loaded(object sender, RoutedEventArgs e)
        {
            // Build the tree
            CwlbTreeHelper.BuildLocicalTree(_vwrDemo.Child, _tvLogic.Items);
            CwlbTreeHelper.BuildVisualTree(_vwrDemo.Child, _tvVisual.Items);
        }

        /// <summary>
        /// Hierarchische Verarbetung der Vorschau eines Mausklicks auf einem Element.
        /// </summary>
        private new void PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            FillInDetails(e.OriginalSource);    
        }

        /// <summary>
        /// Ereignisbehandlung für Auswahl eines Elements im logischen oder visuellen Baum.
        /// </summary>
        private void LogicVisualTreeItem_Selected(object sender, RoutedEventArgs e)
        {
            // Prüfen der Auswahl
            if (e.OriginalSource is TreeViewItem)
            {
                // Details des Typs einfüllen
                FillInDetails(((TreeViewItem)e.OriginalSource).Tag);
            }
        }

        /// <summary>
        /// Füllt die Details einer Auswahl in die Steuerelemente ein.
        /// </summary>
        /// <param name="objItem">Steuerelement, dessen Details eingefüllt werden sollen</param>
        private void FillInDetails(object objItem)
        {
            // Typ anzeigen
            _lblType.Content = objItem.GetType().FullName;
            _objCurrentDetail = objItem;

            // Objekt dem Eigenschaftseditor (Forms) zuweisen
            _pgrProperties.SelectedObject = objItem;

            // Detail informationen evaluieren
            CwlbTreeHelper.BuildClassTree(_objCurrentDetail, ClassHierarchy, ClassList);
            CwlbTreeHelper.BuildInterfaceList(_objCurrentDetail, Interfaces, ClassHierarchy);
            CwlbTreeHelper.BuildMethodList(_objCurrentDetail, Methods, ClassList);
            CwlbTreeHelper.BuildEventList(_objCurrentDetail, Events, ClassList);

            // Eigenschaften zählen
            PropertyCount = _objCurrentDetail.GetType().GetProperties().Where(objSearch =>
            {
                object[] aobjAttribute = objSearch.GetCustomAttributes(typeof(BrowsableAttribute), true);
                if (aobjAttribute.Length == 0 || ((BrowsableAttribute)aobjAttribute[0]).Browsable == true)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }).Count();
        }

        /// <summary>
        /// Ereignisbehandlung beim Markieren einer Klasse in der Klassen hierarchie
        /// </summary>
        /// <remarks>
        /// Die Liste der Methoden und Ereignisse sind Abhängig von den ausgewählten
        /// Klassen in der Hierarchie.
        /// </remarks>
        private void ClassChecked_Click(object sender, RoutedEventArgs e)
        {
            // Aktualisieren der Informationen
            CwlbTreeHelper.BuildMethodList(_objCurrentDetail, Methods, ClassList);
            CwlbTreeHelper.BuildEventList(_objCurrentDetail, Events, ClassList);
        }

    }
}
