﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Diagnostics;
using System.Threading;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für CwlbParallelDemo.xaml
    /// </summary>
    public partial class CwlbParallelDemo : Page
    {
        // Instanzdaten
        private PointCollection cobjPoints = new PointCollection();
        private PathFigure myPathFigure = new PathFigure();
        private double dOffset;
        private Path objCurrentPath;

        /// <summary>
        /// Liefert oder definiert die Verzögerung des Hintergrundthreads.
        /// </summary>
        public double Delay { get; set; }

        /// <summary>
        /// Liefert oder definiert den Versatz pro Berechnung.
        /// </summary>
        public double Offset { get; set; }

        /// <summary>
        /// Liefert oder definiert die Amplitude der mathematischen Form.
        /// </summary>
        public double Amplitude { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbParallelDemo.
        /// </summary>
        public CwlbParallelDemo()
        {
            // Festlegen der Standardwerte
            Delay = 20.0;
            Offset = 0.2;
            Amplitude = 200;

            // Initialisierung (generiert)
            InitializeComponent();

            // Figur berechnen
            CalculateCurve(0);

            // Hintergrundthread für die Berechnung starten
            Thread threadCalculate = new Thread(CalculateFlowingFigure);
            threadCalculate.IsBackground = true;
            threadCalculate.Start();
        }

        /// <summary>
        /// Threadmethode für die Berechnung der Figur
        /// </summary>
        /// <param name="objState">Status der Daten (unbenutzt).</param>
        private void CalculateFlowingFigure(object objState)
        {
            // Definiert denmaximalen Offset für die Korrektur der Ortsberechnung (unendlicher Lauf)
            double dMaxOffset = 40.0;

            // Der Thread läuft unendlich
            while (true)
            {
                // Minimale Verzögerung prüfen
                int iSleep = Delay < 5.0 ? 5 : (int)Delay;
                // Verzögern
                Thread.Sleep(iSleep);
                // Funktion berechnen
                CalculateCurve(dOffset);

                // Offset korrigieren
                dOffset += Offset;
                if (dOffset > Math.PI * dMaxOffset)
                {
                    dOffset -= Math.PI * dMaxOffset;
                }
                else if (dOffset < Math.PI * dMaxOffset)
                {
                    dOffset += Math.PI * dMaxOffset;
                }
            }
        }

        /// <summary>
        /// Definiert das Delegate für den Threadwechsel.
        /// </summary>
        /// <param name="dOffset">Offset, für die Berechnung der Grafik.</param>
        private delegate void DelegateCalculateCurve(double dOffset);

        /// <summary>
        /// Berechnet die Grafik und erstellt die entsprechenden UI Objekte.
        /// </summary>
        /// <param name="dOffset"></param>
        private void CalculateCurve(double dOffset)
        {
            // Prüfen, ob ein Threadwechsel notwendig ist
            if (!Dispatcher.CheckAccess())
            {
                // Delegate auf sich selber definieren und aufrufen
                DelegateCalculateCurve objDelegate = CalculateCurve;
                Dispatcher.Invoke(objDelegate, DispatcherPriority.Background, dOffset);
            }
            else
            {
                // Herstellung einer neuen Figur für die Ausgabe
                cobjPoints = new PointCollection();
                myPathFigure = new PathFigure();
                Point ptValue;
                for (double dX = 20 * -Math.PI; dX < 20 * Math.PI; dX += 0.05)
                {
                    ptValue = new Point(dX * 20 + dOffset, Amplitude * Math.Sin(dX * 1.5) * Math.Cos(dX / 2.0));
                    if (myPathFigure.StartPoint == new Point(0, 0))
                    {
                        myPathFigure.StartPoint = ptValue;
                    }
                    cobjPoints.Add(ptValue);
                }

                // Pfadgeometrie herstellen
                PolyQuadraticBezierSegment objBezierSegment = new PolyQuadraticBezierSegment();
                objBezierSegment.Points = cobjPoints;
                PathSegmentCollection objSegmentCollection = new PathSegmentCollection();
                objSegmentCollection.Add(objBezierSegment);
                myPathFigure.Segments = objSegmentCollection;
                PathFigureCollection objFigureCollection = new PathFigureCollection();
                objFigureCollection.Add(myPathFigure);
                PathGeometry objPathGeometry = new PathGeometry(objFigureCollection);

                // Erstellen des Pfadobjekts um die Grafik zu zeichnen.
                Path objPath = new Path();
                objPath.Stroke = Brushes.Red;
                objPath.StrokeThickness = 1;
                objPath.Data = objPathGeometry;

                // Achtung, die Koordiantenachsen werden von der Canvas selber gezeichnet.
                // Figur zur Canvas hinzufügen
                if (objCurrentPath != null)
                {
                    // Alte Figur löschen, wenn eine vorhanden ist.
                    _canvasFigure.Children.Remove(objCurrentPath);
                }
                objCurrentPath = objPath;
                _canvasFigure.Children.Add(objPath);
            }
        }
    }
}
