﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Datensammlung für das Beispiel. Diese Klasse simuliert quai den Datenbankzugriff.
    /// </summary>
    public class CwlbInMemoryData
    {
        /// <summary>
        /// Liefert eine Instanz der Daten.
        /// </summary>
        public static CwlbInMemoryData Data { get; private set; }

        // Hält alle Kontake im Speicher
        private readonly List<CwlbContact> _cobjContacts;

        /// <summary>
        /// Initialisiert das einzige Objekt der Klasse (Singleton).
        /// </summary>
        static CwlbInMemoryData()
        {
            Data = new CwlbInMemoryData();
        }

        /// <summary>
        /// Initialisiert die Daten der Klasse.
        /// </summary>
        private CwlbInMemoryData()
        {
            // Erstellen der Liste and Anfügen der Daten für die Demo
            _cobjContacts = new List<CwlbContact>();
            _cobjContacts.Add(new CwlbPersonContact
            {
                ID = "0001",
                Title = "Frau",
                Name = "Sonnenhunger",
                FirstName = "Hella",
                Birthdate = new DateTime(1976, 3, 25),
                Address = "Sonnenhalde 35 \r\nD-00000 Sonnenberg",
                Email = "H.Sonnenhunger@email.com",
                Phone = "0000 12 345 678"
            });
            _cobjContacts.Add(new CwlbPersonContact
            {
                ID = "0002",
                Title = "Frau",
                Name = "Blumiger",
                FirstName = "Rosa",
                Birthdate = new DateTime(1966, 6, 5),
                Address = "Blumenbeet 7 \r\nCH-0000 Blumenstadt",
                Email = "R.Blumiger@email.com",
                Phone = "0000 11 987 564"
            });
            _cobjContacts.Add(new CwlbPersonContact
            {
                ID = "0003",
                Title = "Herr",
                Name = "Garten",
                FirstName = "Peter",
                Birthdate = new DateTime(1970, 6, 25),
                Address = "An der Hacke\r\nCH-0000 Gartenstadt",
                Email = "info@gartenland.com",
                IsCustomer = true,
                TotalSales = 45736,
                Phone = "0000 31 345 987"
            });
            _cobjContacts.Add(new CwlbCompanyContact
            {
                ID = "0004",
                Name = "Computer Handels und Reparatur GmbH",
                Address = "Softwarestrasse 99\r\nCH-0000 Hardturm",
                Email = "info@compuhare.com",
                TaxIdentifier = "A11 874-48857 GA",
                IsCustomer = true,
                TotalSales = 96536,
                Phone = "0000 41 475 321"
            });
            _cobjContacts.Add(new CwlbCompanyContact
            {
                ID = "0005",
                Name = "Hutmacher und Berber AG",
                Address = "Unter dem Teppich\r\nCH-3456 Auf dem Berg",
                Email = "welcome@humabe.com",
                TaxIdentifier = "B 23.5867.8573",
                IsCustomer = true,
                TotalSales = 7465,
                Phone = "0000 68 93 382 123"
            });
            _cobjContacts.Add(new CwlbCompanyContact
            {
                ID = "0006",
                Name = "Stahlhart AG",
                Address = "Auf dem Ambos\r\nD-13456 Erzberg",
                Email = "welcome@stahlhart.de",
                TaxIdentifier = "FE 45887",
                IsCustomer = false,
                Phone = "3456 522 345 6545"
            });
            _cobjContacts.Add(new CwlbPersonContact
            {
                ID = "0007",
                Title = "Professor",
                Name = "Düsentrieb",
                FirstName = "Daniel",
                Birthdate = new DateTime(1948, 11, 3),
                Address = "Erfinderweg 5\r\nD-0000 Entenhausen",
                Email = "Dadue@Erfinder.com",
                IsCustomer = false,
                Phone = "0000 1 2 3 4 5 6 7 8 9"
            });
            _cobjContacts.Add(new CwlbPersonContact
            {
                ID = "0008",
                Title = "Fräulein",
                Name = "Gute",
                FirstName = "Mine",
                Birthdate = new DateTime(1968, 1, 31),
                Address = "Zum bösen Spiel\r\nD-12121 Spieldorf",
                Email = "GuteMine@BoesesSpiel.de",
                IsCustomer = false,
                Phone = "0000 12 765 123"
            });
            _cobjContacts.Add(new CwlbPersonContact
            {
                ID = "0009",
                Title = "Herr",
                Name = "Haber",
                FirstName = "Hanspeter",
                Birthdate = new DateTime(1980, 4, 15),
                Address = "Warenstrasse 56\r\nD-21212 Essensdorf",
                Email = "Hanspeter@Haber.de",
                IsCustomer = true,
                Phone = "0909 09 090 909"
            });
        }

        /// <summary>
        /// Liefert die Liste der Kontakte.
        /// </summary>
        /// <returns>Liefert die Kontakte</returns>
        public List<CwlbContact> GetContacts()
        {
            return _cobjContacts;
        }

        /// <summary>
        /// Prüft, ob ein definierter Kontakt bereits in der Liste der Daten vorhanden ist.
        /// </summary>
        /// <param name="objContact">Kontakt, der geprüft werden soll.</param>
        /// <returns>True, wenn der Kontakt bereits erfasst ist.</returns>
        public bool ContainsContact(CwlbContact objContact)
        {
            return _cobjContacts.Contains(objContact);
        }

        /// <summary>
        /// Wird ausgelöst, wenn ein neuer Kontakt hinzugefügt wird.
        /// </summary>
        public event EventHandler<CwlbCustomerEventArgs> EventContactAdded = delegate { };

        /// <summary>
        /// Wird ausgelöst, wenn ein neuer Kontakt gelöscht wird.
        /// </summary>
        public event EventHandler<CwlbCustomerEventArgs> EventContactDeleted = delegate { };

        /// <summary>
        /// Fügt einen Kontakt in die Liste der Kontakte hinzu.
        /// </summary>
        /// <param name="objContact"></param>
        public void AddContact(CwlbContact objContact)
        {
            // Überprüfung des Arguments
            if (objContact == null)
            {
                throw new ArgumentNullException("Null-Werte sind beim Gebrauch dieser Methode nicht erlaubt.");
            }

            // Prüfen, ob der Kunde schon erfasst ist
            if (_cobjContacts.FirstOrDefault(objCheck => objCheck.ID == objContact.ID) != null)
            {
                throw new InvalidOperationException(string.Format("Der Kontakt mit der ID {0} exisitiert bereits und kann kein zweites mal erfasst werden.", objContact.ID));
            }

            // Fügt den Kontakt hinzu und löst das Event aus.
            _cobjContacts.Add(objContact);
            EventContactAdded(this, new CwlbCustomerEventArgs(objContact));
        }

        /// <summary>
        /// Löscht einen Kontakt aus der Sammlung.
        /// </summary>
        /// <param name="objContact">Kontakt, der gelöscht werden soll.</param>
        public void DeleteContact(CwlbContact objContact)
        {
            // Überprüfung des Arguments
            if (objContact == null)
            {
                throw new ArgumentNullException("Null-Werte sind beim Gebrauch dieser Methode nicht erlaubt.");
            }

            // Fügt den Kontakt hinzu und löst das Event aus.
            _cobjContacts.Remove(objContact);
            EventContactDeleted(this, new CwlbCustomerEventArgs(objContact));
        }
    }
}
