﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Stellt einen Befehl zur Verfügung, der die Befehlsausführung an eine definierte Methode delegiert.
    /// </summary>
    public class CwlbDelegatingCommand : ICommand
    {
        // Delegat für die Befehlsausführung
        private readonly Action<object> _delegateExecute;
        // Delegat für den Status der Befehlsausführung
        private readonly Predicate<object> _delegateCanExecute;

        /// <summary>
        /// Liefert oder definiert das besitzende Objekt.
        /// </summary>
        public object Owner { get; set; }

        /// <summary>
        /// Liefert oder definiert den programmiertechnischen Namen.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Instanziiert eine neue Instanz der Klasse CwlbDelegatingCommand.
        /// </summary>
        /// <param name="delegateExecute">Delegat für die Aktionslogik.</param>
        public CwlbDelegatingCommand(Action<object> delegateExecute, string strName = null)
            : this(delegateExecute, null, strName)
        {
        }

        /// <summary>
        /// Instanziert eine neue Instanz der Klasse CwlbDelegatingCommand.
        /// </summary>
        /// <param name="delegateExecute">Delegat für die Aktionslogik.</param>
        /// <param name="delegateCanExecute">Delegat für den Status der Aktionslogik.</param>
        public CwlbDelegatingCommand(Action<object> delegateExecute, Predicate<object> delegateCanExecute, string strName = null)
        {
            if (delegateExecute == null)
            {
                throw new ArgumentNullException("Der Delegat für die Aktionslogik muss definiert sein.");
            }

            _delegateExecute = delegateExecute;
            _delegateCanExecute = delegateCanExecute;
            Name = strName;
        }

        /// <summary>
        /// Prüft die mögliche Ausführung des Befehls.
        /// </summary>
        /// <param name="objParameter">Parameter für die Weitergabe an den Delegaten.</param>
        /// <returns>Liefert true, wenn der Befehl ausgeführt werden kann, andernfalls false.</returns>
        public bool CanExecute(object objParameter)
        {
            return _delegateCanExecute == null ? true : _delegateCanExecute(objParameter);
        }

        /// <summary>
        /// Ereignis für die Kontrolle des Status der Befehlsausführung.
        /// </summary>
        /// <remarks>
        /// Dieses Ereignis delegiert die Registrierung der Kontrolle der Befehlsausführung an den Befehlsmanager der Anwendung.
        /// </remarks>
        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        /// <summary>
        /// Führt den Befehl aus.
        /// </summary>
        /// <param name="objParameter">Parameter für die Befehlsausführung.</param>
        public void Execute(object objParameter)
        {
            _delegateExecute(objParameter);
        }
    }
}
