﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Reflection;
using System.Windows.Input;
using System.Windows.Controls;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Stellt die Übersetzung von konfigurierten weitergeleiteten Ereignissen in Befehle sicher.
    /// </summary>
    public class CwlbEventCommandMapper : DependencyObject
    {
        /// <summary>
        /// Interne Liste mit allen registirierten Elementen
        /// </summary>
        private static Dictionary<FrameworkElement, List<CwlbEventCommandMapping>> _cobjEventCmdMappingCollection =
            new Dictionary<FrameworkElement, List<CwlbEventCommandMapping>>();

        /// <summary>
        /// Liefert die definierten Mappings eines Objektes
        /// </summary>
        /// <param name="objDependency">Definiert das abhängige Objekt, für das die Methode ausgeführt werden soll.</param>
        /// <returns>Array der definierten Übersetzungsobjekte.</returns>
        public static CwlbEventCommandMapping[] GetMappings(DependencyObject objDependency)
        {
            return (CwlbEventCommandMapping[])objDependency.GetValue(MappingsProperty);
        }

        /// <summary>
        /// Definiert die Mappings eines Objektes.
        /// </summary>
        /// <param name="objDependency">Definiert das abhängige Objekt, für das die Methode ausgeführt werden soll.</param>
        /// <param name="aobjMapping">Array der definierten Übersetzungsobjekte.</param>
        public static void SetMappings(DependencyObject objDependency, CwlbEventCommandMapping[] aobjMapping)
        {
            objDependency.SetValue(MappingsProperty, aobjMapping);
        }

        /// <summary>
        /// Registriert die Eigenschaft für die Verwendung als abhängige Eigenschaft.
        /// </summary>
        public static readonly DependencyProperty MappingsProperty =
            DependencyProperty.RegisterAttached(
            "Mappings", 
            typeof(CwlbEventCommandMapping[]), typeof(CwlbEventCommandMapper), 
            new UIPropertyMetadata(null, MappingsChanged));

        /// <summary>
        /// Ereignisbehandlung, wenn die Daten eines Objekts ändern.
        /// </summary>
        /// <param name="objDependencyObject">Abhängiges Objekt für das die Übersetzung defineirt wird.</param>
        /// <param name="objPropertyChangedArguments">Argument für die Veränderung.</param>
        public static void MappingsChanged(DependencyObject objDependencyObject, DependencyPropertyChangedEventArgs objPropertyChangedArguments)
        {
            // Lokale Daten
            List<CwlbEventCommandMapping> cobjDependencyMapperList = null;
            bool bRegisterSomething = false;

            // Umwandeln des abhängigen Elements in ein Framwork element
            FrameworkElement objElement = objDependencyObject as FrameworkElement;
            if (objElement == null)
            {
                throw new InvalidOperationException("Die Übersetzung von weitergeleiteten Eigenschaft in Befehle wird nur für Elemente der Klasse 'FrameworkElement' unterstützt.");
            }

            // Abfragen, ob das definierte Objekt in der eigenen Liste registriert ist
            if (_cobjEventCmdMappingCollection.ContainsKey(objElement))
            {
                // Bestehende Liste holen
                cobjDependencyMapperList = _cobjEventCmdMappingCollection[objElement];
            }
            else
            {
                // Neue Liste erstellen
                cobjDependencyMapperList = new List<CwlbEventCommandMapping>();
                _cobjEventCmdMappingCollection.Add(objElement, cobjDependencyMapperList);
            }

            // Registrieren der Mappings
            foreach (CwlbEventCommandMapping objMapping in (CwlbEventCommandMapping[])objPropertyChangedArguments.NewValue)
            {
                // Prüfen, ob das Element schon existiert
                if (cobjDependencyMapperList.FirstOrDefault(objSearch => objSearch.EventName == objMapping.EventName) != null)
                {
                    // Ignorieren
                    continue;
                }
                else
                {
                    // Hinzufügen des Mappings in die Liste
                    cobjDependencyMapperList.Add(objMapping);
                    RegisterEventHandler(objElement, objMapping);
                    bRegisterSomething = true;
                }
            }

            // Wenn eine Registrierung erfolgte, sollten wir auch um die Aufhebung derselben besorgt sein.
            // Die Registrierung des Unloaded-Ereignisses wird als Trick angewendet, da die WPF
            // keinen Aufruf für die Aufhebung der Registrierung vorsieht. Mit Hilfe des Unloaded-
            // Ereignisses können wir die Aufhebung der Registrierung korrekt erledigen.
            if (bRegisterSomething)
            {
                objElement.AddHandler(FrameworkElement.UnloadedEvent, new RoutedEventHandler(Unloaded));
            }
        }

        /// <summary>
        /// Registriert das weitergeleitete Ereignis.
        /// </summary>
        /// <param name="objElement">Frameworkelement für das eine Registrierung erfolgen soll.</param>
        /// <param name="objMapping">Das dazu passende Mappingobjekt, das die Ereignisbehanldung ausführen wird.</param>
        private static void RegisterEventHandler(FrameworkElement objElement, CwlbEventCommandMapping objMapping)
        {
            // Lokale Daten
            FieldInfo objField = null;

            // Prüfen, ob ein Pfad gesetzt ist
            if (string.IsNullOrEmpty(objMapping.ElementType))
            {
                // Objekt für das definiert weitergeleitete Ereignis abfragen. Beachten Sie,
                // dass der definiert Name fest mit dem Suffix 'Event' erweitert wird.
                // Das ermöglicht die Programmierung des Namens gemäß Onlinehilfe.
                objField = objElement.GetType().GetField(objMapping.EventName + "Event", BindingFlags.FlattenHierarchy | BindingFlags.Static | BindingFlags.Public);
            }
            else
            {
                Type objType = Type.GetType(objMapping.ElementType);
                if (objType == null)
                {
                    throw new InvalidOperationException(
                        "Fehler beim Registrieren des Mappings. Die angegebene Elementtyp des weitergeleitetes Ereignis konnte nicht gealden werden. " +
                        "Prüfen Sie die Angabe und defineren Sie den starken Namen des Typs.");
                }

                objField = objType.GetField(objMapping.EventName + "Event", BindingFlags.FlattenHierarchy | BindingFlags.Static | BindingFlags.Public);
            }
            if (objField == null)
            {
                throw new InvalidOperationException(
                    "Fehler beim Registrieren des Mappings. Die Metainformation des weitergeleitetes Ereignis wurde nicht gefunden. " +
                    "Prüfen Sie den Namen und verwenden Sie diesen ohne Suffix 'Event'.");
            }

            objMapping.RoutedEvent = objField.GetValue(null) as RoutedEvent;
            if (objMapping.RoutedEvent == null)
            {
                throw new InvalidOperationException("Fehler beim Registrieren des Mappings. Das gefunden Ereignis konnte nicht als 'RoutedEvent' umgewandelt werden.");
            }

            // Handler registrieren
            objElement.AddHandler( objMapping.RoutedEvent, new RoutedEventHandler(objMapping.Mapping));
        }

        /// <summary>
        /// Ereignisbehandlung beim Entladen des Elements.
        /// </summary>
        private static void Unloaded(object objSender, RoutedEventArgs objEventArgs)
        {
            // Sender in intern verwendeten Typ konvertieren
            FrameworkElement objElement = objSender as FrameworkElement;

            // Abfragen, ob dieses Element wirklich aus der eigenen Verwaltung stammt.
            if (objElement != null && _cobjEventCmdMappingCollection.ContainsKey(objElement))
            {
                // Die Registrierungen der Übersetzungen aufheben
                foreach (CwlbEventCommandMapping objMapping in _cobjEventCmdMappingCollection[objElement])
                {
                    // Unregister(objElement, objMapping);
                    objElement.RemoveHandler(objMapping.RoutedEvent, new RoutedEventHandler(objMapping.Mapping));
                }

                // Die Registrierung des Unload-Handlers selber aufheben
                objElement.RemoveHandler(FrameworkElement.UnloadedEvent, new RoutedEventHandler(Unloaded));
            }
        }

        /// <summary>
        /// Registrierung eines übersetzten Elements aufheben.
        /// </summary>
        /// <param name="objElement">Frameworkelement für das eine Registrierung aufgehoben werden soll.</param>
        /// <param name="objMapping">Das dazu passende Mappingobjekt, dessen Ereignisbehanldung aufgehoben wird.</param>
        private static void Unregister(FrameworkElement objElement, CwlbEventCommandMapping objMapping)
        {
            // Registrierung aufheben
            objElement.RemoveHandler(objMapping.RoutedEvent, new RoutedEventHandler(objMapping.Mapping));
        }
    }
}
