﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Data;
using System.Collections.Specialized;
using System.Windows.Input;
using System.Windows;
using System.Windows.Controls;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Stellt das ViewModel für die Hauptansicht zur Verfügung.
    /// </summary>
    public class CwlbMainViewModel : CwlbCommandPanelViewModel
    {
        /// <summary>
        /// Daten mit denen gearbeitet wird.
        /// </summary>
        private CwlbInMemoryData _objData;

        /// <summary>
        /// Definiert den aktuellen Befehl für das Speichern zur Verfügung.
        /// </summary>
        private CwlbCommandViewModel _cmdSaveCurrentView;

        /// <summary>
        /// Definiert einen Befehl zum Speichern zur Steuerung des UI.
        /// </summary>
        private CwlbCommandViewModel _cmdSaveNOP;

        /// <summary>
        /// Aktueller Befehl für das Speichern.
        /// </summary>
        /// <remarks>
        /// Diese Befahl referenziert den Befehl zu Speichern der aktiven View. 
        /// Verwenden Sie diese Refernez zum Beispiel zur Ansteuerung aus einer Symbolleiste 
        /// oder einem allgmeinen Menüeintrag speichern.
        /// </remarks>
        public CwlbCommandViewModel SaveCommandCurrentView 
        {
            get
            {
                return _cmdSaveCurrentView;
            }
            private
            set
            {
                _cmdSaveCurrentView = value;
                OnPropertyChanged("SaveCommandCurrentView");
            }
        }

        // Liste der aktuellen Ansicht
        public static CwlbMainViewModel Current { get; private set; }

        /// <summary>
        /// Liefert die Liste der Befehle, die im Menü aufgenommen werden sollen.
        /// </summary>
        public ReadOnlyCollection<CwlbCommandViewModel> MenuCommands
        {
            get
            {
                return new ReadOnlyCollection<CwlbCommandViewModel>(
                    Commands.Where(objItem => objItem.MenuCommand == true).ToList());
            }
        }

        // Liste der Ansichten
        ObservableCollection<CwlbCommandPanelViewModel> _cobjViews;

        /// <summary>
        /// Liefert die Liste der Ansichten
        /// </summary>
        public ObservableCollection<CwlbCommandPanelViewModel> Views
        {
            get
            {
                // Verzögerte Erstellung der Ansichten
                if (_cobjViews == null)
                {
                    _cobjViews = new ObservableCollection<CwlbCommandPanelViewModel>();
                    _cobjViews.CollectionChanged += this.OnViewCollectionChanged;
                }
                return _cobjViews;
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz des ViewModel der Hauptansicht.
        /// </summary>
        public CwlbMainViewModel()
        {
            // Ansichtsname festlegen
            base.ViewName = Properties.MVVMDemo.MainViewName;

            Current = this;

            // Angezeigte Daten verbinden
            _objData = CwlbInMemoryData.Data;

            // Registrieren der Commands
            _cobjCommands.Add(new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameAllContacts,
                new CwlbDelegatingCommand(objParam => this.ShowAllContacts())) 
                {
                    MenuCommand = true
                });

            _cobjCommands.Add( new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameCreatePerson,
                new CwlbDelegatingCommand(objParam => this.CreateNewPerson()))
                {
                    MenuCommand = true
                });

            _cobjCommands.Add(new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameCreateCompany,
                new CwlbDelegatingCommand(objParam => this.CreateNewCompany()))
                {
                    MenuCommand = true
                });


            // Befehl für den Wechsel der Registerkarte
            _cobjCommands.Add(new CwlbCommandViewModel(
                "TabSelectionChanged",
                new CwlbDelegatingCommand(objParam => this.SelectionChanged(objParam as RoutedEventArgs), "SelectionChanged")));

            // Definiert einen speziellen Befehl für das Speichern, der aktiviert wird, wenn keine Ansicht aktiv ist
            _cmdSaveNOP = new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameSave,
                new CwlbDelegatingCommand(objParam => this.DoNothing(), objParam => this.CanNeverExecute(), GwlbMVVM.COMMAND_SAVE));
            _cobjCommands.Add(_cmdSaveNOP);
            SaveCommandCurrentView = _cmdSaveNOP;
        }

        /// <summary>
        /// Ereignisbehandlung für die Veränderung der Auswahl
        /// </summary>
        /// <remarks>
        /// Bei dieser Ereignisbehandlung muss darauf geachtet werden, dass sie im Objektbaum auftritt. Aufgrund dessen 
        /// wird in der Methode nur eine Verabreitung der betroffenen Views vorgenommen.
        /// </remarks>
        /// <param name="objArgs">Ereignisargument.</param>
        void SelectionChanged(RoutedEventArgs objArgs)
        {
            SelectionChangedEventArgs objSelectionChangedArgs = objArgs as SelectionChangedEventArgs;
            if ( objSelectionChangedArgs == null )
            {
                // Stille Verweigerung der Arbeit.
                return;
            }

            // Prüfen einer leeren View Liste
            if (CwlbMainViewModel.Current.Views.Count == 0)
            {
                SaveCommandCurrentView = this._cmdSaveNOP;
                return;
            }

            // Aktiv setzen wenn der Wechsel zu einer neuen View erfolgte
            if (objSelectionChangedArgs.AddedItems.Count == 1)
            {
                // Prüfen des Typs der aktivierten Sicht
                if (((objSelectionChangedArgs.AddedItems[0] is CwlbContactViewModel && ! (objArgs.OriginalSource is ListView))) ||
                    (objSelectionChangedArgs.AddedItems[0] is CwlbContactListViewModel))
                {
                    // Wenn eine Kontaktseite geöffnet wird, Aktiv 
                    SetActiveView(objSelectionChangedArgs.AddedItems[0] as CwlbCommandPanelViewModel);
                }
                else
                {
                    // Alle andern Möglichkeiten werden ignoriert
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Liste der kontrollierten Ansichten.
        /// </summary>
        /// <remarks>
        /// Registriert und deregistriert die Ereignisbehanldung für das Schliessen der Ansicht.
        /// </remarks>
        void OnViewCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            // Registrieren des Ereignisbehandlung zum Schliessen 
            // der neuen Ansichten
            if (e.NewItems != null && e.NewItems.Count != 0)
            {
                foreach (CwlbCommandPanelViewModel view in e.NewItems)
                {
                    view.EventClose += OnViewClose;
                }
            }

            // Deregistrieren des Ereignisbehandlung zum Schliessen 
            // der gelöschten Ansichten
            if (e.OldItems != null && e.OldItems.Count != 0)
            {
                foreach (CwlbCommandPanelViewModel view in e.OldItems)
                {
                    view.EventClose -= OnViewClose;
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung zum Schliessen der Ansicht.
        /// </summary>
        void OnViewClose(object sender, EventArgs e)
        {
            CwlbCommandPanelViewModel view = sender as CwlbCommandPanelViewModel;
            view.Dispose();
            Views.Remove(view);
        }

        /// <summary>
        /// Definiert die sichtbare Ansicht.
        /// </summary>
        /// <param name="objActivePanel">Ansicht, die aktiv werden soll.</param>
        internal void SetActiveView(CwlbCommandPanelViewModel objActivePanel)
        {
            if (objActivePanel is CwlbContactViewModel)
            {
                SaveCommandCurrentView = objActivePanel.Commands.FirstOrDefault(
                    objSearch => StringComparer.CurrentCultureIgnoreCase.Compare(
                        ((CwlbDelegatingCommand)objSearch.Command).Name,
                        GwlbMVVM.COMMAND_SAVE) == 0);
            }
            else
            {
                SaveCommandCurrentView = this._cmdSaveNOP;
            }

            ICollectionView cobjView = CollectionViewSource.GetDefaultView(Views);
            if (cobjView != null)
            {
                cobjView.MoveCurrentTo(objActivePanel);
                if (!(cobjView.CurrentItem is CwlbContactListViewModel))
                {
                    cobjView.Refresh();
                }
            }
        }

        /// <summary>
        /// Zeigt die Ansicht für alle Kontakte.
        /// </summary>
        private void ShowAllContacts()
        {
            // Ansichtsmodell erstellen
            CwlbContactListViewModel vmContactList = Views.FirstOrDefault(vmSearch => vmSearch is CwlbContactListViewModel) as CwlbContactListViewModel;
            if (vmContactList == null)
            {
                // Verspätetes Erstellen der Ansicht
                vmContactList = new CwlbContactListViewModel(_objData);
                Views.Add(vmContactList);
            }

            // Modell aktiv schalten
            SetActiveView(vmContactList);
        }

        /// <summary>
        /// Zeigt die Ansicht für das Erfassen eines neuen Personenkontakts an.
        /// </summary>
        /// <returns></returns>
        private void CreateNewPerson()
        {
            // Erstellen eines neuen Personenobjekts
            CwlbPersonContact objPerson = CwlbPersonContact.Create();
            CwlbContactPersonViewModel vmContactPerson = new CwlbContactPersonViewModel(objPerson, _objData);
            Views.Add(vmContactPerson);

            // Ansicht aktiv schalten
            SetActiveView(vmContactPerson);
        }

        /// <summary>
        /// Zeigt die Ansicht für das Erfassen eines neuen Firmenkontaks an.
        /// </summary>
        /// <returns></returns>
        private void CreateNewCompany()
        {
            // Erstellen eines neuen Firmenkontaks
            CwlbCompanyContact objCompany = CwlbCompanyContact.Create();
            CwlbContactCompanyViewModel vmContactCompany = new CwlbContactCompanyViewModel(objCompany, _objData);
            Views.Add(vmContactCompany);

            // Ansicht aktiv schalten
            SetActiveView(vmContactCompany);
        }

        /// <summary>
        /// Methode für die Definition von Befehlen, die keinen Inhalt haben.
        /// </summary>
        private void DoNothing()
        {
            throw new InvalidOperationException("Diese Operation darf nie ausgeführt werden.");
        }

        /// <summary>
        /// Methode, für die immerwährende Sperrung von Befehlen.
        /// </summary>
        /// <returns>Liefert immer false.</returns>
        private bool CanNeverExecute()
        {
            return false;
        }
    }
}
