﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Reflection;
using System.Web.UI.HtmlControls;
using System.Drawing;
using System.Collections;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert ein Steuerelement für die Betrachtung der Eigenschaften eines Objekts.
    /// </summary>
    [DefaultProperty("AssociatedControlID")]
    [ToolboxData("<{0}:CwlbPropertyGrid runat=server></{0}:CwlbPropertyGrid>")]
    public class CwlbPropertyGrid : WebControl
    {
        /// <summary>
        /// Definiert die Liste der Eigenschaften, die das Steuerelement kontrolliert
        /// </summary>
        private List<CwlbProperty> _cobjProperty = new List<CwlbProperty>();

        /// <summary>
        /// Definiert das Steuerelement, das von dem Property Grid bearbeitet wird.
        /// </summary>
        private Control _objControl;

        #region Eigenschaften
        /// <summary>
        /// Liefert oder definiert die ID des Steuerelements, das dargestallt wird.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Localizable(false)]
        public string AssociatedControlID
        {
            get
            {
                string sAssociatedControlId = (string)ViewState["AssociatedControlID"];
                return ((sAssociatedControlId == null) ? String.Empty : sAssociatedControlId);
            }
            set
            {
                ViewState["AssociatedControlID"] = value;
            }
        }
        #endregion

        #region Lebenszykluskontrolle
        /// <summary>
        /// Ereignisbehandlung für die Vorbereitung der Inhaltsaufbereitung.
        /// </summary>
        protected override void OnInit(EventArgs e)
        {
            // Aufruf der gleichnamigen Methode der Basisklasse
            base.OnInit(e);
            // Die Benützung des Control State anmelden
            Page.RegisterRequiresControlState(this);
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Steuerelements.
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            // Aufruf der gleichnamigen Methode der Basisklasse
            base.OnLoad(e);

            // Erstelle die Elemente des Propertygrids
            CreatePropertiesFromControl();
        }

        /// <summary>
        /// Ereignisbehandlung für die Vorbereitung des Renderings.
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // Zuweisung der Werte des kontrollierten Steuerelements
            foreach (CwlbProperty objProperty in _cobjProperty)
            {
                AssignValue(objProperty);
            }
        }

        /// <summary>
        /// Rendert die Steuerelemente.
        /// </summary>
        /// <param name="output">Ausgabedatenstrom.</param>
        protected override void RenderContents(HtmlTextWriter output)
        {
            RegisterScript(output);
            base.RenderContents(output);
        }

        /// <summary>
        /// Lädt den Status des Steuerelements.
        /// </summary>
        /// <param name="savedState">Gespeicherter Status.</param>
        protected override void LoadControlState(object savedState)
        {
            string strState = (string)savedState;
            if (strState != AssociatedControlID)
            {
                AssociatedControlID = strState;
            }
        }

        /// <summary>
        /// Speichert den Status des Steuerelements.
        /// </summary>
        /// <returns>Zu speichernder Status.</returns>
        protected override object SaveControlState()
        {
            return _objControl.ID;
        }
        #endregion

        /// <summary>
        /// Initialisiert das Element in Bezug auf das definierte Steuerelement.
        /// </summary>
        public void CreatePropertiesFromControl()
        {
            // Lokale Daten
            object[] aobjAttribute;
            CwlbProperty objProperty;

            // Suche das konfigurierte Steuerelement für die Beobachtung
            _objControl = FindControlRecursive(Page, AssociatedControlID);
            if (_objControl != null)
            {
                // Das konfigurierte Steuerelement wurde gefunden. Es werden alle 
                // Eigenschaften in die interne Kontroll-Liste aufgenommen
                foreach (PropertyInfo objPropertyInfo in _objControl.GetType().GetProperties())
                {
                    // Prüfen, ob die Eigenschaft "browsable" konfiguriert ist, und die Eigenschaft 
                    // unterdrücken, wenn das Attribut das explizit auschaltet
                    aobjAttribute = objPropertyInfo.GetCustomAttributes(typeof(BrowsableAttribute), true);
                    if (aobjAttribute.Length > 0)
                    {
                        if (((BrowsableAttribute)aobjAttribute[0]).Browsable == false)
                        {
                            continue;
                        }
                    }

                    // Erstellen und Einfügen des Propertyobjekts
                    objProperty = new CwlbProperty();
                    _cobjProperty.Add(objProperty);
                    // Konfigurieren des Property
                    objProperty.PropertyInfo = objPropertyInfo;
                    objProperty.Name = objPropertyInfo.Name;

                    // Lese die Kategorie, zu der das Element konfiguriert ist. Eigenschaften ohne
                    // konfigurierte Eigenschaft werden der Kategorie Misc zugewiesen
                    aobjAttribute = objPropertyInfo.GetCustomAttributes(typeof(CategoryAttribute), true);
                    if (aobjAttribute.Length > 0)
                    {
                        objProperty.Category = ((CategoryAttribute)aobjAttribute[0]).Category;
                    }
                    else
                    {
                        objProperty.Category = "Misc";
                    }

                    // Prüfen, ob das Element read only ist
                    if (objPropertyInfo.Name == "ID")
                    {
                        objProperty.IsReadOnly = true;
                    }
                }

                // Erstelle die dynamischen Steuerelemente
                CreatePropertyControls();
                this.Controls.Add(GeneratePropertyGrid());
            }
        }

        public void ReInitialize()
        {
            // Lösche alle vorhandenen Steuerelemente des Propertygrids
            this.Controls.Clear();
            _cobjProperty.Clear();
            // Initialisiere neu
            CreatePropertiesFromControl();
            // Zuweisung der Steuerelemente zu den Eigenschaften
            foreach (CwlbProperty objProperty in _cobjProperty)
            {
                AssignValue(objProperty);
            }
        }

        /// <summary>
        /// Zuweisung des Werts an die Eigenschaft des zu kontrollierenden Steuerelements.
        /// </summary>
        /// <param name="objProperty">Eigenschaft deren Wert übernommen werden soll.</param>
        private void AssignValue(CwlbProperty objProperty)
        {
            // Lokale Variablen
            TypeCode enmTypeCode;

            // Typcode abfragen
            enmTypeCode = Type.GetTypeCode(objProperty.PropertyInfo.PropertyType);

            try
            {
                // Aufzählungen werden mit einem Select-Steuerelement dargestellt
                if (objProperty.PropertyInfo.PropertyType.IsEnum && objProperty.PropertyInfo.CanWrite)
                {
                    objProperty.PropertyInfo.SetValue(
                        _objControl,
                        Enum.Parse(objProperty.PropertyInfo.PropertyType, ((DropDownList)objProperty.Control).SelectedItem.Text),
                        null);
                }
                // Boolean konvertieren
                else if (enmTypeCode == TypeCode.Boolean && objProperty.PropertyInfo.CanWrite)
                {
                    objProperty.PropertyInfo.SetValue(
                        _objControl,
                        ((CheckBox)objProperty.Control).Checked,
                        null);
                }
                // Numerische Werte konvertieren
                else if (enmTypeCode == TypeCode.Int32 && objProperty.PropertyInfo.CanWrite)
                {
                    objProperty.PropertyInfo.SetValue(
                        _objControl,
                        Convert.ToInt32(((TextBox)objProperty.Control).Text),
                        null);
                }
                // Texte verwenden
                else if (enmTypeCode == TypeCode.String && objProperty.PropertyInfo.CanWrite)
                {
                    objProperty.PropertyInfo.SetValue(
                        _objControl,
                        ((TextBox)objProperty.Control).Text,
                        null);
                }
                else
                {
                    // Spezialtypen konvertieren
                    // Farben konvertieren
                    if (objProperty.PropertyInfo.PropertyType == typeof(Color))
                    {
                        ColorConverter objConverter = new ColorConverter();
                        objProperty.PropertyInfo.SetValue(
                            _objControl,
                            objConverter.ConvertFromString(((TextBox)objProperty.Control).Text),
                            null);
                    }
                    // Font informationen konvertieren
                    else if (objProperty.PropertyInfo.PropertyType == typeof(FontInfo))
                    {
                        FontInfo objFontInfo = objProperty.PropertyInfo.GetValue(_objControl, null) as FontInfo;
                        objFontInfo.Name = ((TextBox)objProperty.Control).Text;
                    }
                    // Einheiten und Größen konvertieren
                    else if (objProperty.PropertyInfo.PropertyType == typeof(Unit))
                    {
                        UnitConverter objConverter = new UnitConverter();
                        objProperty.PropertyInfo.SetValue(
                            _objControl,
                            objConverter.ConvertFromString(((TextBox)objProperty.Control).Text),
                            null);
                    }
                }
            }
            catch
            {
                // Alle Ausnahmen fangen und ignorieren
            }
        }

        /// <summary>
        /// Erstellt die Steuerelemente für die Eingabe der Steuerelemente
        /// </summary>
        private void CreatePropertyControls()
        {
            // Lokale Variablen
            TypeCode enmTypeCode;

            // Alle gefundenen Eigenschaften durchgehen
            foreach (CwlbProperty objProperty in _cobjProperty)
            {
                // Typcode abfragen
                enmTypeCode = Type.GetTypeCode(objProperty.PropertyInfo.PropertyType);

                // Aufzählungen werden mit einem Select-Steuerelement dargestellt
                if (objProperty.PropertyInfo.PropertyType.IsEnum)
                {
                    // Erstellen eines DropDownList-Steuerelements
                    DropDownList ddlSelect = new DropDownList();
                    objProperty.Control = ddlSelect;
                    // Erstellen der Auswahl
                    foreach (object objValue in Enum.GetValues(objProperty.PropertyInfo.PropertyType))
                    {
                        ddlSelect.Items.Add(objValue.ToString());
                    }
                    // Initalieren des Werts
                    ddlSelect.SelectedIndex = ddlSelect.Items.IndexOf(
                        ddlSelect.Items.FindByText(objProperty.PropertyInfo
                            .GetValue(_objControl, null).ToString()));

                    if (objProperty.IsReadOnly)
                    {
                        ddlSelect.Enabled = false;
                    }
                    else
                    {
                        ddlSelect.Enabled = true;
                    }
                }
                else if (enmTypeCode == TypeCode.Boolean)
                {
                    CheckBox cbProperty = new CheckBox();
                    objProperty.Control = cbProperty;
                    // Initialisieren des Werts
                    if ((bool)objProperty.PropertyInfo.GetValue(_objControl, null))
                    {
                        cbProperty.Checked = true;
                    }
                    else
                    {
                        cbProperty.Checked = false;
                    }

                    if (objProperty.IsReadOnly)
                    {
                        cbProperty.Enabled = false;
                    }
                    else
                    {
                        cbProperty.Enabled = true;
                    }
                }
                else if (objProperty.PropertyInfo.PropertyType.GetInterface("ICollection") != null)
                {
                    // Alle andern Typen werden als Textfeld erstellt
                    TextBox tbProperty = new TextBox();
                    objProperty.Control = tbProperty;
                    // Initialsieren des Werts
                    object objValue = objProperty.PropertyInfo.GetValue(_objControl, null);
                    tbProperty.Text = objValue.ToString();
                    tbProperty.ReadOnly = true;
                    objProperty.IsReadOnly = true;
                }
                else if (objProperty.PropertyInfo.PropertyType == typeof(Color))
                {
                    // Alle andern Typen werden als Textfeld erstellt
                    TextBox tbProperty = new TextBox();
                    objProperty.Control = tbProperty;
                    // Initialsieren des Werts
                    Color objValue = (Color)objProperty.PropertyInfo.GetValue(_objControl, null);
                    if (objValue.IsEmpty)
                    {
                        tbProperty.Text = string.Empty;
                    }
                    else
                    {
                        tbProperty.Text = objValue.ToString();
                    }
                }
                else
                {
                    // Alle andern Typen werden als Textfeld erstellt
                    TextBox tbProperty = new TextBox();
                    objProperty.Control = tbProperty;
                    // Initialsieren des Werts
                    object objValue = objProperty.PropertyInfo.GetValue(_objControl, null);
                    if (objValue == null)
                    {
                        tbProperty.Text = "null";
                    }
                    else
                    {
                        tbProperty.Text = objValue.ToString();
                    }
                    if (objProperty.IsReadOnly)
                    {
                        tbProperty.ReadOnly = true;
                    }
                    else
                    {
                        tbProperty.ReadOnly = false;
                    }
                }

                // ID festlegen
                objProperty.Control.ID = _objControl.ID + objProperty.Name;

                // Editierbarkeit prüfen
                if (objProperty.IsVisible)
                {
                    objProperty.Control.Visible = true;
                }
                else
                {
                    objProperty.Control.Visible = false;
                }
            }
        }

        /// <summary>
        /// Regisitriert das Script für die Darstellung der Eigenschften als Accordeon.
        /// </summary>
        /// <param name="output">Ausgabedatenstrom.</param>
        private void RegisterScript(HtmlTextWriter output)
        {
            StringBuilder strText = new StringBuilder();
            strText.Append("<script type=\"text/javascript\">");
            strText.Append("$(document).ready(function () {");
            strText.Append("$('#Properties').accordion({");
            strText.Append("header: \"h3\",");
            strText.Append("animated: \"bounceslide\",");
            strText.Append("autoHeight: false");
            strText.Append("});});</script>");

            output.Write(strText.ToString());
        }

        /// <summary>
        /// Erstellt das eigentliche Propertygrid-Steuerelement.
        /// </summary>
        /// <returns></returns>
        private HtmlGenericControl GeneratePropertyGrid()
        {
            // Lokale Daten
            HtmlGenericControl objPropertyGrid = new HtmlGenericControl("div");
            objPropertyGrid.ID = "Properties";
            objPropertyGrid.ClientIDMode = System.Web.UI.ClientIDMode.Static;
            objPropertyGrid.Attributes.Add("class", "wroPropertyGrid");

            // Das angezeigte Steuerelement beschreiben
            Label objLabel = new Label();
            objPropertyGrid.Controls.Add(objLabel);
            objPropertyGrid.Controls.Add(new HtmlGenericControl("br"));
            objLabel.Text = string.Format("ID: <b>{0}</b> ({1})", _objControl.ID, _objControl.GetType().FullName);
          
            // Die gefundenen Steuerelemente gruppiert auflisten
            foreach (var objGroup in _cobjProperty
                .Where( objSelect => objSelect.Control.Visible)
                .GroupBy( objGroup => objGroup.Category)
                .OrderBy(objGroup => objGroup.Key))
            {
                // Kategorieneintrag erstellen (für das JQuery Accordeon)
                HtmlGenericControl objCategory = new HtmlGenericControl("h3");
                HtmlAnchor objCategoryAnchor = new HtmlAnchor();
                objCategoryAnchor.HRef = "#";
                objCategoryAnchor.InnerText = objGroup.Key;
                objPropertyGrid.Controls.Add(objCategory);
                objCategory.Controls.Add(objCategoryAnchor);

                // Container für Daten und Datentabelle erstellen
                HtmlGenericControl objCategoryContent = new HtmlGenericControl("div");
                HtmlTable objContentTable = new HtmlTable();
                objPropertyGrid.Controls.Add(objCategoryContent);
                objCategoryContent.Controls.Add(objContentTable);

                // Inhalt der Datentabelle erstellen
                foreach (CwlbProperty objProperty in objGroup.OrderBy(objOrder => objOrder.PropertyInfo.Name))
                {
                    // Zeile erstellen
                    HtmlTableRow objRow = new HtmlTableRow();
                    objContentTable.Rows.Add(objRow);
                    // Zelle mit Label erstellen
                    HtmlTableCell objCell = new HtmlTableCell();
                    objCell.Attributes.Add("class", "PropertyLabel");
                    objCell.InnerText = objProperty.PropertyInfo.Name;
                    objRow.Cells.Add(objCell);
                    // Zelle mit Steuerelement erstellen
                    objCell = new HtmlTableCell();
                    if (objProperty.IsReadOnly)
                    {
                        ((WebControl)objProperty.Control).CssClass = "PropertyReadOnly";
                    }
                    objCell.Controls.Add(objProperty.Control);
                    objRow.Cells.Add(objCell);
                }
            }

            // Schaltfläche für das Anwenden der Einstellungen erstellen
            Button btnApply = new Button();
            btnApply.Text = "Anwenden";
            objPropertyGrid.Controls.Add(new HtmlGenericControl("br"));
            objPropertyGrid.Controls.Add(btnApply);

            return objPropertyGrid;
        }

        /// <summary>
        /// Auffinden eines Steuerelements durch Rekursion im Baum
        /// </summary>
        /// <param name="objStartSearch">Steuerelement bei welchem die Suche beginnt.</param>
        /// <param name="strId">Identifikation des gesuchten Steuerelements.</param>
        /// <returns>Liefert das gefundene Steuerelement oder null wenn nicht gefunden.</returns>
        private Control FindControlRecursive(Control objStartSearch, string strId)
        {
            if (objStartSearch.ID == strId)
            {
                return objStartSearch;
            }
            else
            {
                Control objControl;
                foreach (Control objInnerControl in objStartSearch.Controls)
                {
                    objControl = FindControlRecursive(objInnerControl, strId);
                    if (objControl != null)
                    {
                        return objControl;
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Definiert den Wert einer Eigenschaft.
        /// </summary>
        /// <param name="strName">Name der Eigenschaft, deren Wert definiert werden soll.</param>
        /// <param name="objValue">Wert der der Eigenschaft zugewiesen werden soll.</param>
        internal void SetProperty(string strName, object objValue)
        {
            CwlbProperty objProperty = _cobjProperty.FirstOrDefault(objSearch => objSearch.Name == strName);
            if (objProperty != null)
            {
                if (objProperty.Control.GetType() == typeof(TextBox))
                {
                    ((TextBox)objProperty.Control).Text = objValue.ToString();
                }
                if (objProperty.Control.GetType() == typeof(CheckBox))
                {
                    ((CheckBox)objProperty.Control).Checked = (bool)objValue;
                }
                if (objProperty.Control.GetType() == typeof(DropDownList))
                {
                    ListItem objItem = ((DropDownList)objProperty.Control).Items.FindByText(objValue.ToString());
                    ((DropDownList)objProperty.Control).SelectedIndex = ((DropDownList)objProperty.Control).Items.IndexOf(objItem);
                }
            }
        }
    }
}