﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WeroSoft.Samples.Models;
using WeroSoft.Samples.Helper;

namespace WeroSoft.Samples.Controllers
{
    /// <summary>
    /// Controller für die Verarbeitung von Anforderungen an die Verwaltung von Personen.
    /// </summary>
    public class PersonController : Controller
    {
        /// <summary>
        /// Liefert die Ansicht für die Erstellung einer neuen Person
        /// </summary>
        /// <returns>Leere Ansicht zum erfassen einer Person.</returns>
        public ActionResult Create()
        {
            return View();
        }

        /// <summary>
        /// Erstellt ein Personenobjekt nach der Erfassung durch den Benutzer
        /// </summary>
        /// <param name="objPerson">Person, die es zu erfassen gilt.</param>
        [HttpPost]
        [LogingActionFilterAttribute(LogFileName="ActionLog.log")]
        public ActionResult Create(CwlbPerson objPerson)
        {
            try
            {
                // Imperative Prüfung von Daten
                if (string.IsNullOrWhiteSpace(objPerson.Address))
                {
                    ModelState.AddModelError("Address", "Die Adresse darf nicht leer sein. Geben Sie mindestens die Postleitzahl und die Ortschaft ein.");
                }

                if (ModelState.IsValid)
                {
                    CwlbDataContext.Current.Create(objPerson);
                    return RedirectToAction("Index");
                }
                else
                {
                    return View(objPerson);
                }
            }
            catch
            {
                return View();
            }
        }

        /// <summary>
        /// Aktion für die Handhabung der Liste der Personen.
        /// </summary>
        /// <returns>Liefert die Resultatliste der Personen.</returns>
        public ActionResult Index(string sort)
        {
            if (StringComparer.InvariantCultureIgnoreCase.Compare(sort, "desc") == 0)
            {
                return View(CwlbDataContext.Current.Persons.OrderByDescending(objPerson => objPerson.Name));
            }
            else
            {
                return View(CwlbDataContext.Current.Persons.OrderBy(objPerson => objPerson.Name));
            }
        }

        /// <summary>
        /// Liefert eine Detailelement
        /// </summary>
        /// <param name="Id">Identifikation der Person deren Detaildaten gelifert werden sollen.</param>
        /// <returns>Unveränderliche Ansicht.</returns>
        public ActionResult Details(string Id)
        {
            return View(CwlbDataContext.Current.Details(Id));
        }

        /// <summary>
        /// Liefert eine Detailelement
        /// </summary>
        /// <param name="Id">Identifikation der Person deren Detaildaten gelifert werden sollen.</param>
        /// <returns>Ansicht für dei Veränderung der Daten.</returns>
        public ActionResult Edit(string Id)
        {
            return View(CwlbDataContext.Current.Details(Id));
        }

        /// <summary>
        /// Definiert die Personendaten nach dem Editieren.
        /// </summary>
        /// <param name="Id">Identifikation der Person.</param>
        /// <param name="name">Name der Person.</param>
        /// <param name="FirstName">Vorname der Person.</param>
        /// <param name="Address">Adresse der Person.</param>
        /// <param name="Email">E-Mail der Person.</param>
        /// <param name="Birthdate">Geburtsdatum der Person.</param>
        /// <returns>Liefert die Liste der erfassen Personen.</returns>
        [HttpPost]
        public ActionResult Edit(string Id, string Name, string FirstName, string Address, string Email, string Birthdate)
        {
            try
            {
                CwlbDataContext.Current.Update(Id, Name, FirstName, Address, Email, Birthdate);
 
                return RedirectToAction("Index");
            }
            catch
            {
                return View();
            }
        }

        /// <summary>
        /// Zeigt eine Person die gelöscht werden soll.
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public ActionResult Delete(string Id)
        {
            return View(CwlbDataContext.Current.Details(Id));
        }

        /// <summary>
        /// Löscht eine Person aus der Liste.
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="collection"></param>
        /// <returns></returns>
        [HttpPost]
        public ActionResult Delete(string Id, FormCollection collection)
        {
            try
            {
                CwlbDataContext.Current.Delete(Id);
                return RedirectToAction("Index");
            }
            catch
            {
                return View();
            }
        }
    }
}
