﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.IO;

namespace WeroSoft.Samples.Helper
{
    /// <summary>
    /// Definiert ein Aktionsfilter für die Erstellung eines Logeintrages.
    /// </summary>
    public class LogingActionFilterAttribute : ActionFilterAttribute
    {
        /// <summary>
        /// Liefert oder definiert den Dateinamen der verwendeten Logdatei.
        /// </summary>
        public string LogFileName { get; set; }

        /// <summary>
        /// Liefert den vollqualifizierten Dateinamen für den Logeintrag.
        /// </summary>
        /// <returns>Dateinamen mit vollqualifiziertem Pfad.</returns>
        private string GetFullName(HttpContextBase objContext)
        {
            return Path.Combine(objContext.Request.PhysicalApplicationPath, "App_Data", LogFileName);
        }

        /// <summary>
        /// Writes the log entry to the file.
        /// </summary>
        /// <param name="strController">Name of the controller which is executed.</param>
        /// <param name="strAction">Name of the Action which is executed.</param>
        /// <param name="strText">Description of the action which is executing.</param>
        private void WriteLogEntry(HttpContextBase objContext, string strController, string strAction, string strText)
        {
            try
            {
                File.AppendAllText(GetFullName(objContext), string.Format(
                    "{0}; {1}; {2}; {3}{4}",
                    DateTime.UtcNow.ToLongTimeString(),
                    strController, strAction, strText,
                    Environment.NewLine));
            }
            catch
            {
                // Ignore all Errors
            }
        }

        /// <summary>
        /// Ereignisbehandlung vor der Ausführung der Action.
        /// </summary>
        /// <param name="filterContext"></param>
        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            WriteLogEntry(filterContext.HttpContext,
                filterContext.ActionDescriptor.ControllerDescriptor.ControllerType.Name,
                filterContext.ActionDescriptor.ActionName,
                "Executing");
            base.OnActionExecuting(filterContext);
        }

        /// <summary>
        /// Ereignisbehandlung nach der Ausführung der Action.
        /// </summary>
        /// <param name="filterContext"></param>
        public override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            WriteLogEntry(filterContext.HttpContext,
                filterContext.ActionDescriptor.ControllerDescriptor.ControllerType.Name,
                filterContext.ActionDescriptor.ActionName,
                "Executed");
            base.OnActionExecuted(filterContext);
        }

        /// <summary>
        /// Ereignisbehandlung vor der Bereitstellung des Resultats.
        /// </summary>
        /// <param name="filterContext"></param>
        public override void OnResultExecuting(ResultExecutingContext filterContext)
        {
            WriteLogEntry(filterContext.HttpContext,
                filterContext.Controller.ControllerContext.Controller.GetType().Name,
                "",
                "Result Executing");
            base.OnResultExecuting(filterContext);
        }

        /// <summary>
        /// Ereignisbehandlung nach der Bereitstellung des Resultats.
        /// </summary>
        /// <param name="filterContext"></param>
        public override void OnResultExecuted(ResultExecutedContext filterContext)
        {
            WriteLogEntry(filterContext.HttpContext,
                filterContext.Controller.ControllerContext.Controller.GetType().Name,
                "",
                "Result Executed");
            base.OnResultExecuted(filterContext);
        }
    }
}