﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Collections.ObjectModel;

namespace WeroSoft.Samples.Models
{
    /// <summary>
    /// Stellt das Datenmodell dar.
    /// </summary>
    public sealed class CwlbDataContext
    {
        /// <summary>
        /// Liefert den aktuellen Datenkontext der Anwendung.
        /// </summary>
        public static CwlbDataContext Current { get; private set; }

        /// <summary>
        /// Liste der Personen
        /// </summary>
        private List<CwlbPerson> _cobjPersons = new List<CwlbPerson>();

        /// <summary>
        /// Definiert ein Synchronisationsobjekt für die Daten.
        /// </summary>
        private readonly object WorkerLock = new object();

        /// <summary>
        /// Liefert eine unveränderliche Liste mit Personen.
        /// </summary>
        public ReadOnlyCollection<CwlbPerson> Persons
        {
            get
            {
                lock (WorkerLock)
                {
                    return new ReadOnlyCollection<CwlbPerson>(_cobjPersons);
                }
            }
        }

        /// <summary>
        /// Erstellt das Modell
        /// </summary>
        static CwlbDataContext()
        {
            Current = new CwlbDataContext();
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDataContract.
        /// </summary>
        private CwlbDataContext()
        {
            _cobjPersons.Add(new CwlbPerson
            {
                Id = "001",
                Name = "Albligen",
                FirstName = "Peter",
                Address = "Albisweg 34\r\nCH-5678 Berg",
                Email = "p.a@sample.de",
                Birthdate = new DateTime(1985, 12, 23)
            });
            _cobjPersons.Add(new CwlbPerson
            {
                Id = "002",
                Name = "Zerzuben",
                FirstName = "Angela",
                Address = "Gartenweg 4\r\nCH-3000 Bern",
                Email = "a.z@sample.de",
                Birthdate = new DateTime(1980, 2, 3)
            });
            _cobjPersons.Add(new CwlbPerson
            {
                Id = "003",
                Name = "Tanzen",
                FirstName = "Judith",
                Address = "Tanzdiele\r\nCH-1000 Lausanne",
                Email = "j.t@sample.de",
                Birthdate = new DateTime(1991, 1, 17)
            });
        }

        /// <summary>
        /// Erfasst ein Personenobjekt in der Datenliste.
        /// </summary>
        /// <param name="objPerson">Personenobjekt mit den Daten.</param>
        public void Create(CwlbPerson objPerson)
        {
            lock (WorkerLock)
            {
                // Erstellen einer ID, wenn keine vorhanden ist
                if (string.IsNullOrEmpty(objPerson.Id))
                {
                    objPerson.Id = Guid.NewGuid().ToString();
                }

                // Prüfen, ob das Objekt bereits existiert
                if (_cobjPersons.FirstOrDefault(objSearch => objSearch.Id == objPerson.Id) != null)
                {
                    throw new InvalidOperationException("Es ist bereits eine Person mit dieser Identifikation erfasst worden.");
                }

                // Erfassen der Person
                _cobjPersons.Add(objPerson);
            }
        }

        /// <summary>
        /// Liefert die Personendaten mit einer definierten Identifikation.
        /// </summary>
        /// <param name="strId">Identifikation der gesuchten Personendaten.</param>
        /// <returns>Personendaten.</returns>
        public CwlbPerson Details(string strId)
        {
            lock (WorkerLock)
            {
                return _cobjPersons.FirstOrDefault(objSearch => objSearch.Id == strId);
            }
        }

        /// <summary>
        /// Verandert die Daten einer definierten Person.
        /// </summary>
        /// <param name="strId">Identifikation der Person.</param>
        /// <param name="strName">Name der Person.</param>
        /// <param name="strFirstName">Vorname der Person.</param>
        /// <param name="strAddress">Adresse der Person.</param>
        /// <param name="strEmail">E-Mail der Person.</param>
        /// <param name="strBirthdate">Geburtsdatum der Person.</param>
        public void Update(string strId, string strName, string strFirstName, string strAddress, string strEmail, string strBirthdate)
        {
            lock (WorkerLock)
            {
                // Gewünschte Person aus der Liste lesen
                CwlbPerson objPerson = _cobjPersons.FirstOrDefault(objSearch => objSearch.Id == strId);
                if (objPerson == null)
                {
                    throw new InvalidOperationException("Die Person mit der Id: '{0}' ist nicht definiert.");
                }

                // Daten verändern
                objPerson.Name = strName;
                objPerson.FirstName = strFirstName;
                objPerson.Address = strAddress;
                objPerson.Email = strEmail;
                objPerson.Birthdate = Convert.ToDateTime(strBirthdate);
            }
        }

        /// <summary>
        /// Löscht eine Person aus der Liste.
        /// </summary>
        /// <param name="strId">Identifikation der zu löschenden Person.</param>
        public void Delete(string strId)
        {
            lock (WorkerLock)
            {
                _cobjPersons.RemoveAll(objPerson => objPerson.Id == strId);
            }
        }
    }
}